"""
    8/14/03
        7 parts fixed for blender 2.28 by jms
        
    7/10/03
        Added Code to direct import external xml as shader ( for crafter )
    6/26/03
        Added Cached Pathlight Buttons and code

    Modified from yablex20030618-leope.py

    Added Timer Function
    Added View Function Set with VIEWERAPP variable below
    Added Multi-Assign on Meshes
    Added Multi-Assign on Lights


    YaBle - Yet Another Blender Exporter

        Version: 0.30-3 (beta forever!)

    (Support yafray.0.0.3, NOT fully compatible with older !)   

    written by:
            Andrea Carbone

            http://kino3d.com/~yable/

    Feel free to contact me for questions or opinions
    to this email address: 

            tredee@katamail.com

    Thanks to Jandro, Eeshlo and all the guys from www.kino3d.com
    Thanks to Christoffer Green (BlenderMan) for the gui code

    Visit 
        http://www.yafray.org

    for new releases or announcements about YafRay or
    other contributions.
"""

import Blender

from Blender.Draw import *
from Blender.BGL import *
from Blender import NMesh 
import os, sys, time
import string
from math import *

# Depending on the python compilation/installation, gzip might not be available on some systems.
# The required zlib module is however also included with Publisher, so that will probably always work.
try:
    import gzip
    GZIPOK = 1
except ImportError:
    print "Gzip module not supported!"
    GZIPOK = 0


BL_VERSION = Blender.Get('version')
if BL_VERSION <= 223:
    import Blender210

else:
    from Blender import  Object, Scene #, Mesh

print "Blender Version", BL_VERSION

#####################################################################

#####################################################################
#File System Operations

#RENAME
#YABLEROOT = "/mnt/gigadue/archivio/linux/grafica/datigrafica/yafraydata"
#YABLEROOT = "D:\Grafica\datileo\yafraydata"

YABLEROOT = "G:/tmp/YFexport"

"""
Examples:

YABLEROOT = "C:\\Documents\\YFexport"
YABLEROOT"/home/username/YFexport"

"""
VIEWERAPP = "c:\windows\explorer"
CRAFTERFILE = "C:\crafter\data\preview.mat"
#####################################################################

outname = Blender.Get('filename')       # NAME OF BLENDFILE
HT = os.path.split(outname)             # (DIR, FILE)
outname = os.path.splitext(HT[1])[0]    # FILE MINUS EXTENSION (.blend)


#####################################################################
#
#   CREATING DIRECTORIES
#
#####################################################################
# creating root output directory
if not os.path.exists(YABLEROOT):
    print "%s directory created" % YABLEROOT
    try:
        os.mkdir(YABLEROOT)
    except OSError:
        raise "Could not create YABLEROOT directory"
else:
    print "%s already exists" % YABLEROOT
#creating Archive directory
archdir = os.path.join(YABLEROOT,"Archive")

if not os.path.exists(archdir):
    print "%s directory created" %archdir
    try:
        os.mkdir(archdir)
    except OSError:
        raise "Could not create Archive directory"
else:
    print "%s already exists" % archdir
    
#Global Settings directory
setsdir = os.path.join(YABLEROOT, "Settings")
if not os.path.exists(setsdir):
    print "%s directory created" %setsdir
    try:
        os.mkdir(setsdir)
    except OSError:
        raise "Could not create Settings directory"
else:
    print "%s already exists" %setsdir

# creating file output directory
outdir = os.path.join(YABLEROOT, outname)
if not os.path.exists(outdir):
    print "%s directory created" % outdir
    try:
        os.mkdir(outdir)
    except OSError:
        raise "Could not create file output directory"
else:
    print "%s already exists" % outdir

# Creo la cartella locale dei materiali
matdir = os.path.join(outdir, "Materials")

if not os.path.exists(matdir):

    try:
        os.mkdir(matdir)
    except OSError:
        raise "Could not create Material output directory"
else:
    print "%s already exists" % matdir

# Creo la cartella locale delle textures
texdir = os.path.join(matdir, "Textures")

if not os.path.exists(texdir):

    try:
        os.mkdir(texdir)
    except OSError:
        raise "Could not create Texture directory"
else:
    print "%s already exists" % texdir

# Creo la cartella locale delle mesh
meshdir = os.path.join(outdir, "Meshes")

if not os.path.exists(meshdir):

    try:
        os.mkdir(meshdir)
    except OSError:
        raise "Could not create Meshes output directory"
else:
    print "%s already exists" % meshdir

print "Running Operating System:  ", os.name

if os.name=="posix":
    POSIX = 1
else:
    POSIX = 0

def scanlocal(dir, ext):
    goodones = {}
    names=os.listdir(dir)
    if len(names):
        for name in names:
            root,fext = os.path.splitext(name)
            if fext == ext:
                goodones[root] = name
    return goodones

def scantextures():
    goodones = {}
    names=os.listdir(texdir)
    modulatordict={}
    uvdict={}
    moduldict={}
    if len(names):
        for name in names:
            real = string.split(name, "__")
            if real[1] == "UV":
                uvdict[real[0]] = name
            elif real[1] == "Modulators":
                modulatordict[real[0]] = name
            elif real[1] == "Modul":
                moduldict[real[0]] = name
    goodones["UV"] = uvdict
    goodones["Modulators"]   =  modulatordict
    goodones["Modul"]   =  moduldict
    return goodones

scantex = scantextures()

modulatorlist= scantex["Modulators"]
uvlist=scantex["UV"]
modullist=scantex["Modul"]
WRONGTEX=[]

def retrieve_modulators():
    ret = "-do nothing- |ADD Image |ADD Clouds |ADD Marble"
    if len(modulatorlist):
        if modulatorlist.has_key(selectedname):
            mm = Modulators(selectedname)
            mm.load()
            if len(mm.modulators):
                for m in range(len(mm.modulators)):
                    ret += "|"+ mm.modulators[m]
            del mm
    return ret



mod_data = []
def retrieve_mod_data(num):
    modlis = string.split(texmenu, "|")
    mm = Modulator(modlis[num-1])
    mm.load()
    if mm.type == "image":
        tt = imageTexture(mm.texture)
        tt.load()
    elif mm.type == "clouds":
        tt = cloudsTexture(mm.texture)
        tt.load()
    else:
        tt = marbleTexture(mm.texture)
        tt.load()
    return [mm, tt]

#Useful functions from Blenderman
def writeln(f,x):
    try:
        if (type(x).__name__ == 'string'): # this thing is to combat a bug that saves strings with a \n, (we dont want that)
            nn = string.find(x, '\n')
            if (nn > 0):
                x = x[:nn] + x[nn + 1:]
        f.write(str(x))
        f.write('\n')
    except:
        pass

"""
def readf(f):
    loop =1
    #possible endless loop
    while loop=1:
        s = (f.readline())
        nn = string.find(s, '<*>')
        if (nn > 0):
            loop = 0
            s = s[:nn] 
            tipo =s[nn + 3:]
"""

def readstr(f):
    try:
        loop = 1
        while loop:
            s = (f.readline())
            if not s == "\n":
                loop=0

        nn = string.find(s, '\n')
        if (nn > 0):
            #s[nn+1:] could be skipped...
            s = s[:nn] + s[nn + 1:]
        return s
    except:
        pass

def readint(f):
    try:
        loop = 1
        while loop:
            i=(f.readline())
            if not i == "/n":
                ret = int(i)
                loop = 0
        return ret
    except:
        print "no int"


def readfloat(f):
    try:
        loop = 1
        while loop:
            s = (f.readline())
            if not s == "\n":
                loop=0
                ret = float(s)
        return ret

    except:
        print "no float"
#Classes##############################################
"""
More Classes Will come.... :)
"""
class Color:
    def __init__(self, R=0.8, G=0.8, B=0.8):
        self.R = R
        self.G = G
        self.B = B

    def setColorv(self, value):
        self.R = value[0]
        self.G = value[1]
        self.B = value[2]

    def setColorf(self, R, G, B):
        self.R = R
        self.G = G
        self.B = B

#This contain all modulators assigned to the mesh which name is "name"
#Used to store additional textures than the one defined in Blender
class Modulators:
    def __init__(self, name):
        self.name = name
        self.path = os.path.join(texdir, self.name+"__Modulators")
        self.path = self.path.replace('\\', '/')
        self.modulators = []

    def addModulator(modulator):
        self.modulators.append(modulator)

    def save(self):
        m = open(self.path, "w")
        for i in range(len(self.modulators)):
            writeln(m , self.modulators[i])
        m.write("END")
        m.close()

    def load(self):
        esc = 0
        fd = open(self.path, "r")
        ln=readstr(fd)
        while not esc:
            if not (ln== "END"):
                self.modulators.append(ln)
                ln=readstr(fd)
            else: 
                esc = 1
        fd.close()

class Modulator:
    def __init__(self, name, type = "",mode ="mul", \
            kc=1, ks = -1, kr=-1, kt =-1, knorm = 1, kh = -1, size = 1,sizex =1,\
            sizey=1,sizez = 1,tex=None):
        modes = ["add", "mul", "mix"]

        self.name = name
        self.path = os.path.join(texdir, self.name+"__Modul")
        self.path = self.path.replace('\\', '/')
        self.type = type
        if mode in modes:
            self.mode = mode
        else: 
            self.mode = "mul"
        self.size = size
        self.sizex = sizex
        self.sizey= sizey
        self.sizez=sizez
        self.kc = kc
        self.ks = ks
        self.kr= kr
        self.kt = kt
        self.knorm = knorm
        self.kh = kh
        self.texture = tex

    #Tags are not necessary at now...I put here
    #just as a kind of key index for future use
    def save(self):
        fd = open(self.path, "w")
        writeln(fd,"mode")
        writeln(fd,self.mode)
        writeln(fd,"type")
        writeln(fd,self.type)

        writeln(fd, "kc")
        writeln(fd,self.kc)
        writeln(fd, "ks")
        writeln(fd,self.ks)
        writeln(fd, "kr")
        writeln(fd,self.kr)
        writeln(fd, "kt")
        writeln(fd,self.kt)
        writeln(fd, "knorm")
        writeln(fd,self.knorm)
        writeln(fd, "kh")
        writeln(fd,self.kh)
        writeln(fd,"size")
        writeln(fd,self.size)
        writeln(fd,"sizex")
        writeln(fd,self.sizex)
        writeln(fd,"sizey")
        writeln(fd,self.sizey)
        writeln(fd,"sizez")
        writeln(fd,self.sizez)
        writeln(fd, "texture")
        writeln(fd,self.texture)
        fd.write("END")
        fd.close()

    def load(self):
        fd= open(self.path, "r")
        ln = readstr(fd)
        self.mode = readstr(fd)
        ln = readstr(fd)
        self.type = readstr(fd)
        ln = readstr(fd)
        self.kc = readfloat(fd)
        ln = readstr(fd)
        self.ks = readfloat(fd)
        ln = readstr(fd)
        self.kr = readfloat(fd)
        ln = readstr(fd)
        self.kt=readfloat(fd)
        ln = readstr(fd)
        self.knorm=readfloat(fd)
        ln = readstr(fd)
        self.kh=readfloat(fd)
        ln = readstr(fd)
        self.size = readfloat(fd)
        ln = readstr(fd)
        self.sizex = readfloat(fd)
        ln = readstr(fd)
        self.sizey = readfloat(fd)
        ln = readstr(fd)
        self.sizez = readfloat(fd)
        ln = readstr(fd)
        self.texture = readstr(fd)
        fd.close()

class UVmodulator(Modulator):
    def __init__(self, name, mode ="mul", \
            kc=1, ks = -1, kr=-1, tex=None):

        Modulator.__init__(self, name, "image", mode)
        self.path = os.path.join(texdir, self.name+"__UV")
        self.path = self.path.replace('\\', '/')

class Shader: #non ancora usata ?
    def __init__(self, nome, tipo):
        self.name = nome
        self.type = tipo
        self.path = os.path.join(matdir, name+".yafmat")
        self.path = self.path.replace('\\', '/')
        
    def getType():
        return self.type


class Generic(Shader): #non ancora usata ?

    def __init__(self, nome, col = Color() , h=25, mrf=.2,\
        sspec=None,rrefl=None,ttran=None,iior=None, modlist=[]):

        Shader.__init__(self, nome, "Generic")
        self.color= col
        self.hard = h
        self.minref=mrf
        self.spec = sspec
        self.refl = rrefl
        self.trans = ttran
        self.IOR = iior
        self.modulators = modlist

    def getModulators(self):
        return self.modulators

    def setname(self, nome):
        self.name = nome

    def addModulator(self, modulator):
        self.modulators.append(modulator)

    def hasModulator(self):
        return len(self.modulators)


class Texture:
    tipi = ["image", "clouds", "marble"]
    def __init__(self, nome, tipo):
        self.name = nome
        self.type = tipo
        self.path = os.path.join(texdir, self.name+"__Texture") 
        self.path = self.path.replace('\\', '/')
        
class imageTexture(Texture):
    def __init__(self, nome, texfile= ""):
        Texture.__init__(self, nome, "image")
        self.texpath=texfile
        self.texpath = self.texpath.replace('\\', '/')

    def save(self):
        t = open(self.path,"w")
        writeln(t,"texpath")
        writeln(t, self.texpath)
        t.write("END")
        t.close()

    def load(self):
        t=open(self.path, "r")
        ln = readstr(t)
        self.texpath = readstr(t)
        t.close()

    def export(self, ofile):
        pass

class cloudsTexture(Texture):
    def __init__(self, nome, depth=5, col1=Color(), col2=Color()):
        Texture.__init__(self, nome, "clouds" ) 
        self.depth = depth
        self.color1=col1
        self.color2=col2

    def save(self):
        t = open(self.path,"w")
        writeln(t, "depth")
        writeln(t, self.depth)
        writeln(t, "col1")
        writeln(t,self.color1.R)
        writeln(t,self.color1.G)
        writeln(t,self.color1.B)
        writeln(t, "col2")
        writeln(t,self.color2.R)
        writeln(t,self.color2.G)
        writeln(t,self.color2.B)
        t.write("END")
        t.close()

    def load(self):
        t = open(self.path, "r")
        ln = readstr(t)
        self.depth= readint(t)
        ln = readstr(t)
        self.color1.R = readfloat(t)
        self.color1.G = readfloat(t)
        self.color1.B = readfloat(t)
        ln = readstr(t)
        self.color2.R = readfloat(t)
        self.color2.G = readfloat(t)
        self.color2.B = readfloat(t)
        t.close()
    def export(self, ofile):
        pass

class marbleTexture(Texture):
    def __init__(self, nome, depth=5, col1=Color(), col2=Color(), hard=0, turbulence=10, sharpness=1):
        Texture.__init__(self, nome, "marble")
        self.depth = depth
        self.color1 = col1
        self.color2 = col2
        self.hard = hard
        self.turbulence = turbulence
        self.sharpness = sharpness

    def save(self):
        t = open(self.path, "w")
        writeln(t, "depth")
        writeln(t, self.depth)
        writeln(t, "col1")
        writeln(t, self.color1.R)
        writeln(t, self.color1.G)
        writeln(t, self.color1.B)
        writeln(t, "col2")
        writeln(t, self.color2.R)
        writeln(t, self.color2.G)
        writeln(t, self.color2.B)
        writeln(t, "hard")
        writeln(t, self.hard)
        writeln(t, "turbulence")
        writeln(t, self.turbulence)
        writeln(t, "sharpness")
        writeln(t, self.sharpness)
        t.write("END")
        t.close()

    def load(self):
        t = open(self.path, "r")
        ln = readstr(t)
        self.depth = readint(t)
        ln = readstr(t)
        self.color1.R = readfloat(t)
        self.color1.G = readfloat(t)
        self.color1.B = readfloat(t)
        ln = readstr(t)
        self.color2.R = readfloat(t)
        self.color2.G = readfloat(t)
        self.color2.B = readfloat(t)
        ln = readstr(t)
        self.hard = readint(t)
        ln = readstr(t)
        self.turbulence = readint(t)
        ln = readstr(t)
        self.sharpness = readint(t)
        t.close()
    def export(self, ofile):
        pass


class PathLight:
    pass

class Hemilight:
    pass

#Stores Global Settings
class Settings:
    def __init__(self, name):
        self.name = name
        self.path = os.path.join(setsdir, self.name+"__Settings")
        self.path = self.path.replace('\\', '/')
        self.texturepath = ""
        self.settings = {}
    """
    def save(self):
        fd = open(self.path,"w")
        writeln(fd,"texturepath")
        writeln(fd,"%s\n"%self.texturepath)
        for i in range(len(self.settings)):
            writeln(fd,"%s\n"%self.settings.items()[i][0])
            writeln(fd,"%s\n"%self.settings.items()[i][1])
        fd.write("END")
        fd.close()
    """
    def save(self):
        fd = open(self.path,"w")
        writeln(fd,"texturepath")
        writeln(fd,"%s"%self.texturepath)
        for i in range(len(self.settings)):
            writeln(fd,"%s"%self.settings.items()[i][0])
            writeln(fd,"%s"%self.settings.items()[i][1])
        fd.write("END")
        fd.close()
        
    def load(self):
        fd = open(self.path, "r")
        esc = 0

        while not esc:
            ln = readstr(fd)
            if ln == "texturepath":
                self.texturepath = readstr(fd)
            elif ln == "END":
                esc = 1
            elif GLOBTYPES.has_key(ln):
                ty=GLOBTYPES[ln]
                
                if  ty == "s":
                    self.settings[ln]=readstr(fd)
                elif ty == "i":
                    self.settings[ln]=readint(fd)
                elif ty == "f":
                    self.settings[ln]=readfloat(fd)


def scansettings(EXT):
    goodones = {}
    names=os.listdir(setsdir)
    if len(names):
        for name in names:
            real = string.split(name, "__")
            if real[1] == EXT:
                goodones[real[0]] = name

    return goodones


def startup():
    global TEXROOT
    EXT = "Settings"
    settinglist     = scansettings(EXT)
    if len(settinglist) > 0:
        #print "SettingList ", settinglist
        if settinglist.has_key(outname):
            return 0
        else: 
            
            #Try to guess a value for TEXROOT
            dummy = Settings(settinglist.items()[0][0])
            dummy.load()
            TEXROOT = Create(dummy.texturepath)
            del dummy
            return 1
    else:
        return 1



#Inizializzo meshmat  -- Consente di avere memoria delle passate sessioni-- Materiali 
meshmat     = scanlocal(matdir,".mat")  #Material Dictionary
lightlist   = scanlocal(matdir,".luz")      #Light Dict
meshattr    = scanlocal(matdir,".objattr")   #Attribute Dict

#Some Globals
#E: global list of texture names written to xml file
tex_written = []
triangles = 0
assigned = 1
targetok = 0
topos    = [0,0,0]
pos = [0,0,0]
toposphlD = [0,0,0]
toposphlC = [0,0,0]
to_focus    = [0,0,0]
sunpos  = [0,0,0]
focus       = 0
verso    = 0
blcolors = [[0,0,0],[0,0,0],[0,0,0]]
#lampdat =[[r,g,b], size, blend]
lampdat  = [[0,0,0],0,0]
sunname = "NO_SUN_SEL"
corners = [[0,0,0,0], [0,0,0,0],[0,0,0,0],[0,0,0,0]]

# The bevent constants
exportevent     =       1   # press export button
animevent       =   1122    # press anim button
animsetevt      =   1133    #go to animsettings
renderevent         =   666 # press the render button
exitevent   =       2   # press exit button
updateguievent  =       333 # update the gui
assignmeshevt   =       444 #assign shader to mesh
assignallmeshevt =      443 #assign shared to all selected meshes
assignlightevt  =       5   #assign shader to light
assignalllightevt   =   442 #assign shader to all selected lights
savematevt      =       6   #save shader
loadmatevt      =       7   #load shader
oksave          =       8
okload          =       9 
cancel          =       10
seltargetevt        =       12
seltargetphlDevt = 121
seltargetphlCevt = 122
selfocusevt     =       13
getblcol        =       14
getblspec       =       15
getblref        =       16
ignore_evt      =       17
gototexevt      =       18
texbrowserevt       =       19
mixmodeevt      =       20
mulmodeevt      =       21
addmodeevt      =       22
oth_mixmodeevt      =       23
oth_mulmodeevt      =       24
oth_addmodeevt      =       25

selcol1evt      =       26
selcol2evt      =       27
marbleselcol1evt = 261
marbleselcol2evt = 262

oktexevt        =       28
canctexevt      =       29
deltexevt       =       30

hemievt         =       31
ptrevt          =       32

## background zone ##
constbgevt      =       33
sunbgevt        =       34
selsunevt       =       35
hdribgevt       =       351
hdriprobeevt = 3511
normbgevt       =       352
normbgimgevt = 3521

fogrevt         =       36
foggevt         =       37
fogbevt         =       38

getspcolevt     =       39
getspsievt      =       40
getspblevt      =       41

noexpevt        =       42
defexpevt       =       43
nogammaevt      =       44

alloffevt       =       45
allonevt        =       46
okbackevt       =       47

layersetevt     =       48

previewmatevt   =   400 # launch mat preview
phprevevt   =   410     # set photon light in mat preview

glset           =       100
shset           =       200
luzset          =       300

shadlist        =       "fooshader" #Shaders in archive 
texmenu         =       ""

linklightevt = 9000

getCrafter = 467601


#Contiene la lista dei parametri addizionali asociati alla mesh
meshparams  =       {}
expshaders  =       []
savedmesh   =       {}

################################################################
#                        BUTTONS
################################################################

#Name of The selected Item
selectedname    =       ""
selectedtype    =       "Type: "
shadertype      =       "Generic| Constant |Default |Crafter"
lighttype   =       "Point Light | Sun Light | Soft Light | Area Light | Spot Light"

OutSize     =       "320x240 |480x320  |640x480  |640x512  |768x470  |1024x576  |1024x768  |1280x960  |Custom"
OutSizeList     =       ["320x240", "480x320","640x480","640x512","768x470","1024x576","1024x768","1280x960" ]
samplemenu  =       "2|4|6|8|10|12|14|16|25"

#Layer Bitmask
bmask = -1

#----#
OutPicMenu  =       Create(1)
CustSizeX   =       Create(320)
CustSizeY   =       Create(240)
#---#
gamma       =       Create(1.)
exposure    =       Create(0.)

#Screens
globalsettings  =       Create(1)
shadereditor    =       Create(0)
lighteditor     =       Create(0)

#----#
#Hemilight
hemi        =       Create(0)
HRed        =           Create(0.80)
HGreen      =       Create(0.90)
HBlu        =       Create(1.00)
HPow        =       Create(1.50)
HRandSamp   =       Create(0)
HSamples = Create(4)
HSamplesQMC     =       Create(16)
HQMC    =   Create(0)
usebground  =       Create(0)


## background zone ##
constbg     =       Create(0)
sunbg       =       Create(0)
setsun      =       Create(0)
addsun      =       Create(0)
turb        =       Create(4.0)
sunbgsunpower = Create(1.0)
sunbgavar = Create(1.0)
sunbgbvar = Create(1.0)
sunbgcvar = Create(1.0)
sunbgdvar = Create(1.0)
sunbgevar = Create(1.0)
hdribg = Create(0)
hdriexp = Create(0)
hdriprobe = Create("EMPTY")
normbg = Create(0)
normbgpower = Create(1.0)
normbgimg = Create("EMPTY")

fog     =       Create(0.00)

fogr        =       Create(1)
fogg        =       Create(0)
fogb        =       Create(0)

fogR        =       Create(1.0)
fogG        =       Create(1.0)
fogB        =       Create(1.0)
    #---#
#PathTracing
#JWM
PathCache   =       Create(0)
cacheSize   =       Create(0.008)
angleThreshold =    Create(0.2)
shadowThreshold=    Create(0.2)


pathtrace   =       Create(0)
ptrPow      =       Create(1.0)
ptrDepth    =       Create(2)
ptrSamples  =       Create(4)
ptrSamplesQMC = Create(16)
#ptrRand        =       Create(0)
ptrQMC  = Create(0)


finname         =           Create(0)

gzipit      =       Create(0)
update      =       Create(0)
DISABLE     =       Create(0)
#JWM AppView
appview     =       Create(0)
timer       =       Create(0)
anim        =       Create(0)
#----#
FilterDOF   =       Create(0)
FilterANoise    =       Create(0)

#Crafter Shader
cft_Name  = Create("Material")
cft_Text  = Create(" ")

#Generic Shader
GenR        =       Create(0.80)
GenG        =       Create(0.80)
GenB        =       Create(0.80)


GenSpecR    =       Create(1.00)
GenSpecG    =       Create(1.00)
GenSpecB    =       Create(1.00)

GenRefR     =       Create(1.00)
GenRefG     =       Create(1.00)
GenRefB     =       Create(1.00)
GenRef2R    =       Create(1.00)
GenRef2G    =       Create(1.00)
GenRef2B    =       Create(1.00)

GenTraR     =       Create(0.00)
GenTraG     =       Create(0.00)
GenTraB     =       Create(0.00)
GenTra2R        =       Create(0.00)
GenTra2G    =       Create(0.00)
GenTra2B        =       Create(0.00)

GenHard     =       Create(25)
addIOR      =       Create(0)
GenIOR      =       Create(1.00)
GenMinRef   =       Create(0.2)
GenFastFresnel = Create(0)

AutoSmooth  =       Create(0)
Degr        =       Create(30)

additional  =       Create(0)

castsh      =       Create(1)
addCausIOR  =       Create(0)
CausIOR     =       Create(1.00)
recrad      =       Create(0)
emirad      =       Create(0)
addCausTCol =       Create(0)
CausTR      =       Create(1.00)
CausTG      =       Create(1.00)
CausTB      =       Create(1.00)
addCausRCol =       Create(0)
CausRR      =       Create(1.00)
CausRG      =       Create(1.00)
CausRB      =       Create(1.00)

#Constant Shader
ConstR      =       Create(0.80)
ConstG      =       Create(0.80)
ConstB      =       Create(0.80)

#----#
#Cam Settings (render)
#CamSamples =       Create(2)

cpu     =       Create(1)
CamRayDepth =       Create(5)
CamBias     =       Create(0.3)
CamIndSamples = Create(0)
#tolerance  =       Create(0.06)

AAPasses    =   Create(2)
AAMinSamples = Create(2)
AAPixelWidth = Create(1.5)
AAThreshold = Create(0.04)

bground     =       Create(0)
bgroundR    =       Create(0.80)
bgroundG    =       Create(0.94)
bgroundB    =       Create(1.00)
#---#

image       =       Create(outname)
alphatga = Create(0)        # To save a rendered targa image with the alpha channel
fname       =       Create(outname)
fullpath    =       Create(0)

frstart     =       Create(1)
frend       =       Create(250)

#Menu
surfshader      =       Create(1)
lightshader =       Create(1)


## lights zone ##

#Point Light
plpow       =       Create(1.0)
plshad      =       Create(0)
plR     =       Create(1.00)
plG     =       Create(1.00)
plB     =       Create(1.00)

#Sun Light
slpow       =       Create(1.0)
slshad      =       Create(0)
slR     =       Create(1.00)
slG     =       Create(1.00)
slB     =       Create(1.00)

#Soft Light
sflR        =       Create(1.00)
sflG        =       Create(1.00)
sflB        =       Create(1.00)
sflpow      =       Create(1.0)
sflres      =       Create(200)
sflbias     =       Create(0.3)
sflrad      =       Create(5)
shadernames =       Create(1)

#Photon Light
addphldiff = Create(0)
addphlcaus = Create(0)
phlDpow     =       Create(100.0)
phlDR       =       Create(1.0)
phlDG       =       Create(1.0)
phlDB       =       Create(1.0)
phlDdep     =       Create(3)
phlDsearch  =       Create(200)
phlDang     =       Create(15.0)
phlDfix     =       Create(1.0)
phlDcluster =       Create(0.5)
phlDph      =       Create(50000)
phlDQMC     =       Create(0)
linkcolordiffdir = Create(0)
linkcolorcausdiff = Create(0)
linkcolorcausdir = Create(0)
linkpowercausdiff = Create(0)
linkphotonscausdiff = Create(0)
linkdepthcausdiff = Create(0)
linksearchcausdiff = Create(0)
linkanglecausdiff = Create(0)
linkfixedcausdiff = Create(0)
linkclustercausdiff = Create(0)
linkqmccausdiff = Create(0)
linktargetdiffdir = Create(0)
linktargetcausdiff = Create(0)
linktargetcausdir = Create(0)

phlCpow     =       Create(100.0)
phlCR       =       Create(1.0)
phlCG       =       Create(1.0)
phlCB       =       Create(1.0)
phlCdep     =       Create(3)
phlCsearch  =       Create(100)
phlCang     =       Create(15.0)
phlCfix     =       Create(0.1)
phlCcluster =       Create(0.01)
phlCph      =       Create(50000)
phlCQMC     =       Create(0)


#Spot
splpow      =       Create(100)
splR        =       Create(1.0)
splG        =       Create(1.0)
splB        =       Create(1.0)
splbeoff    =       Create(2.)
splsize     =       Create(30.)
splbl       =       Create(20.)
splshadow = Create(0)
splhaloADD = Create(0)       #Inizio parametri halo
splhalofogR = Create(0.8)
splhalofogG = Create(0.8)
splhalofogB = Create(0.8)
splhalofogdensity = Create(0.2)
splhaloblur = Create(0.0)
splhalores = Create(64)
splhalosamples = Create(64)     #Fine parametri halo

#Area Light
alpow       =       Create(25)
alR     =       Create(1.00)
alG     =       Create(1.00)
alB     =       Create(1.00)
alSamp      =       Create(64)
alPSamp     =       Create(64)
alLato      =       Create(5.0)

SelTargB    =       Create(0)
SelTargphlDB = Create(0)
SelTargphlCB = Create(0)


#------#
addSpec     =       Create(0)
addRef      =       Create(0)
addRef2     =       Create(0)
addTra      =       Create(0)
addTra2     =       Create(0)

setfocus    =       Create(0)


#----#
nearblur    =       Create(10.)
farblur     =       Create(10.)
scale       =       Create(2.)
anoise_radius   =       Create(1.0)

#E: max_delta default set to 0.1
anoise_maxdelta =       Create(0.1)
#---#


#preview photons
phlprv  =   Create(0)


savemenu    =       0
loadmenu    =       0
shname      =       Create("fooname")
# texture directory
TEXROOT = Create('')
#Stores the types of the global data (it's a quite tricky way to do....)
GLOBTYPES={"OutPicMenu":"i",\
    "CustSizeX":"i",\
    "CustSizeY":"i",\
    "hemi":"i","HRed":"f",\
    "HGreen":"f","HBlu":"f",\
    "HPow":"f",\
    "HSamples":"i","HSamplesQMC":"i","HQMC":"i",\
    "pathtrace":"i","ptrPow":"f",\
    "ptrDepth":"i","ptrSamples":"i","ptrSamplesQMC":"i","ptrQMC":"i",\
    "usebground":"i","FilterDOF":"i",\
    "FilterANoise":"i",\
    "AAPasses":"i","AAMinSamples":"i","AAPixelWidth":"f","AAThreshold":"f", \
    "CamRayDepth":"i","CamBias":"f","CamIndSamples":"i",\
    "bground":"i","bgroundR":"f",\
    "bgroundG":"f","bgroundB":"f",\
    "nearblur":"f","farblur":"f",\
    "scale":"f","anoise_radius":"f",\
    "anoise_maxdelta":"f","tofocusx":"f",\
    "tofocusy":"f","tofocusz":"f",\
    "sunposx":"f","sunposy":"f","sunposz":"f",\
    "sunname":"s","addsun": "i","sunbg":"i","turb":"f","sunbgsunpower":"f",\
    "sunbgavar":"f","sunbgbvar":"f","sunbgcvar":"f","sunbgdvar":"f","sunbgevar":"f",\
    "hdribg":"i","hdriexp":"i","hdriprobe":"s",\
    "normbg":"i","normbgpower":"f","normbgimg":"s","fog":"f","fogR":"f","fogG":"f","fogB":"f",\
    "gamma":"f","exposure":"f","fullpath":"i",\
    "frame":"i","bmask":"i","cpu":"i","frstart":"i",\
    "frend":"i","gzipit":"i","alphatga":"i","cacheSize":"f","angleThreshold":"f","shadowThreshold":"f"}

GLOBDB = {}

# JWM - Multiselect
global allSelected
global appviewfilename

def cooksettings():

    gdb=Settings(outname)
    if TEXROOT.val!="":
        gdb.texturepath = TEXROOT.val
    else:
        gdb.texturepath = "None"
    for k in GLOBDB.items():
        gdb.settings[k[0]] = GLOBDB[k[0]]
    gdb.save()
    del gdb

#TODO: Use A compact form....
def eatsettings():
    global TEXROOT
    global  OutPicMenu, CustSizeX, CustSizeY, hemi, HQMC, HRed, HGreen, sunpos
    global HBlu,HPow,HRandSamp,HSamples,HSamplesQMC,usebground,FilterDOF,FilterANoise,\
        addsun, sunbg, turb, sunbgsunpower, sunbgavar, sunbgbvar, sunbgcvar, sunbgdvar, sunbgevar,\
        hdribg, hdriexp, hdriprobe, normbg, normbgpower, normbgimg, gamma, exposure, sunname
    global  CamRayDepth,CamBias,CamIndSamples,bground,bgroundR,bgroundG,bgroundB,nearblur,farblur
    global  scale,anoise_radius,anoise_maxdelta,to_focus, pathtrace,\
        ptrPow, ptrDepth, ptrSamples, ptrSamplesQMC, ptrQMC, fog, fogR,fogG, fogB, finname, fullpath,\
        bmask, cpu, frstart, frend, gzipit, alphatga
    global AAPasses, AAMinSamples, AAPixelWidth, AAThreshold
    global cacheSize,angleThreshold,shadowThreshold

    s=Settings(outname)
    s.load()
    se = s.settings
    
    TEXROOT = Create(s.texturepath)
    
    """
    OutPicMenu=     Create(int(se["OutPicMenu"]))
    CustSizeX =     Create(int(se["CustSizeX"]))
    CustSizeY =     Create(int(se["CustSizeY"]))
    hemi    =   Create(int(se["hemi"]))
    HRed    =   Create(float(se["HRed"]))
    HGreen  =   Create(float(se["HGreen"]))
    HPow    =   Create(float(se["HPow"]))
    HSamples =  Create(int(se["HSamples"]))
    HSamplesQMC = Create(int(se["HSamplesQMC"]))
    HQMC    =   Create(int(se["HQMC"]))
    usebground= Create(int(se["usebground"]))
    FilterDOF=  Create(int(se["FilterDOF"]))
    FilterANoise=   Create(int(se["FilterANoise"])) 
    AAPasses= Create(int(se["AAPasses"]))
    AAMinSamples= Create(int(se["AAMinSamples"]))
    AAPixelWidth= Create(float(se["AAPixelWidth"]))
    AAThreshold= Create(float(se["AAThreshold"]))
    CamRayDepth=    Create(int(se["CamRayDepth"]))
    CamBias=    Create(float(se["CamBias"]))
    CamIndSamples = Create(int(se["CamIndSamples"]))
    bground=    Create(int(se["bground"]))
    bgroundR=   Create(float(se["bgroundR"]))
    bgroundG=   Create(float(se["bgroundG"]))
    bgroundB=   Create(float(se["bgroundB"]))
    nearblur=   Create(float(se["nearblur"]))
    farblur=    Create(float(se["farblur"]))
    scale  =    Create(float(se["scale"]))
    HBlu    =   Create(float(se["HBlu"]))
    anoise_radius=  Create(float(se["anoise_radius"]))
    anoise_maxdelta=Create(float(se["anoise_maxdelta"]))
    addsun  =   Create(se["addsun"])
    to_focus=   [float(se["tofocusx"]),float(se["tofocusy"]),float(se["tofocusz"])]
    sunpos  =   [float(se["sunposx"]),float(se["sunposy"]),float(se["sunposz"])]
    if se.has_key("sunname"):
        sunname=se["sunname"]
    pathtrace=  Create(se["pathtrace"]) 
    ptrPow=     Create(se["ptrPow"])
    ptrDepth=   Create(se["ptrDepth"])
    ptrSamples= Create(se["ptrSamples"])
    ptrSamplesQMC=  Create(se["ptrSamplesQMC"])
    ptrQMC  =   Create(se["ptrQMC"])
    sunbg   =   Create(se["sunbg"]) 
    turb    =   Create(se["turb"])      
    sunbgsunpower = Create(se["sunbgsunpower"])
    sunbgavar = Create(se["sunbgavar"])
    sunbgbvar = Create(se["sunbgbvar"])
    sunbgcvar = Create(se["sunbgcvar"])
    sunbgdvar = Create(se["sunbgdvar"])
    sunbgevar = Create(se["sunbgevar"]) 
    hdribg  =   Create(se["hdribg"])
    hdriexp =   Create(se["hdriexp"])
    hdriprobe   =   Create(se["hdriprobe"]) 
    normbg  =   Create(se["normbg"])
    normbgpower =   Create(se["normbgpower"])
    normbgimg   =   Create(se["normbgimg"])
    fog =   Create(se["fog"])
    fogR    =   Create(se["fogR"])
    fogG    =   Create(se["fogG"])
    fogB    =   Create(se["fogB"])
    """
    
    # jms : fix for urers of older version of yagle which have not
    # the same settings in thier YABLEROOT
    if se.has_key("OutPicMenu") :     OutPicMenu=     Create(int(se["OutPicMenu"]))
    if se.has_key("CustSizeX") :     CustSizeX =     Create(int(se["CustSizeX"]))
    if se.has_key("CustSizeY") :     CustSizeY =     Create(int(se["CustSizeY"]))
    if se.has_key("hemi") :     hemi    =   Create(int(se["hemi"]))
    if se.has_key("HRed") :     HRed    =   Create(float(se["HRed"]))
    if se.has_key("HGreen") :     HGreen  =   Create(float(se["HGreen"]))
    if se.has_key("HPow") :     HPow    =   Create(float(se["HPow"]))
    if se.has_key("HSamples") :     HSamples =  Create(int(se["HSamples"]))
    if se.has_key("HSamplesQMC") :     HSamplesQMC = Create(int(se["HSamplesQMC"]))
    if se.has_key("HQMC") :     HQMC    =   Create(int(se["HQMC"]))
    if se.has_key("usebground") :     usebground= Create(int(se["usebground"]))
    if se.has_key("FilterDOF") :     FilterDOF=  Create(int(se["FilterDOF"]))
    if se.has_key("FilterANoise") :     FilterANoise=   Create(int(se["FilterANoise"])) 
    if se.has_key("AAPasses") :     AAPasses= Create(int(se["AAPasses"]))
    if se.has_key("AAMinSamples") :     AAMinSamples= Create(int(se["AAMinSamples"]))
    if se.has_key("AAPixelWidth") :     AAPixelWidth= Create(float(se["AAPixelWidth"]))
    if se.has_key("AAThreshold") :     AAThreshold= Create(float(se["AAThreshold"]))
    if se.has_key("CamRayDepth") :     CamRayDepth=    Create(int(se["CamRayDepth"]))
    if se.has_key("CamBias") :     CamBias=    Create(float(se["CamBias"]))
    if se.has_key("CamIndSamples") :     CamIndSamples = Create(int(se["CamIndSamples"]))
    if se.has_key("bground") :     bground=    Create(int(se["bground"]))
    if se.has_key("bgroundR") :     bgroundR=   Create(float(se["bgroundR"]))
    if se.has_key("bgroundG") :     bgroundG=   Create(float(se["bgroundG"]))
    if se.has_key("bgroundB") :     bgroundB=   Create(float(se["bgroundB"]))
    if se.has_key("nearblur") :     nearblur=   Create(float(se["nearblur"]))
    if se.has_key("farblur") :     farblur=    Create(float(se["farblur"]))
    if se.has_key("scale") :     scale  =    Create(float(se["scale"]))
    if se.has_key("HBlu") :     HBlu    =   Create(float(se["HBlu"]))
    if se.has_key("anoise_radius") :     anoise_radius=  Create(float(se["anoise_radius"]))
    if se.has_key("anoise_maxdelta") :     anoise_maxdelta=Create(float(se["anoise_maxdelta"]))
    if se.has_key("addsun") :     addsun  =   Create(se["addsun"])
    to_focus=   [float(se["tofocusx"]),float(se["tofocusy"]),float(se["tofocusz"])]
    sunpos  =   [float(se["sunposx"]),float(se["sunposy"]),float(se["sunposz"])]
    if se.has_key("sunname"):        sunname=se["sunname"]
    if se.has_key("pathtrace") :     pathtrace=  Create(se["pathtrace"]) 
    if se.has_key("ptrPow") :     ptrPow=     Create(se["ptrPow"])
    if se.has_key("ptrDepth") :     ptrDepth=   Create(se["ptrDepth"])
    if se.has_key("ptrSamples") :     ptrSamples= Create(se["ptrSamples"])
    if se.has_key("ptrSamplesQMC") :     ptrSamplesQMC=  Create(se["ptrSamplesQMC"])
    if se.has_key("ptrQMC") :     ptrQMC  =   Create(se["ptrQMC"])
    if se.has_key("sunbg") :     sunbg   =   Create(se["sunbg"]) 
    if se.has_key("turb") :     turb    =   Create(se["turb"])      
    if se.has_key("sunbgsunpower") :     sunbgsunpower = Create(se["sunbgsunpower"])
    if se.has_key("sunbgavar") :     sunbgavar = Create(se["sunbgavar"])
    if se.has_key("sunbgbvar") :     sunbgbvar = Create(se["sunbgbvar"])
    if se.has_key("sunbgcvar") :     sunbgcvar = Create(se["sunbgcvar"])
    if se.has_key("sunbgdvar") :     sunbgdvar = Create(se["sunbgdvar"])
    if se.has_key("sunbgevar") :     sunbgevar = Create(se["sunbgevar"]) 
    if se.has_key("hdribg") :     hdribg  =   Create(se["hdribg"])
    if se.has_key("hdriexp") :     hdriexp =   Create(se["hdriexp"])
    if se.has_key("hdriprobe") :     hdriprobe   =   Create(se["hdriprobe"]) 
    if se.has_key("normbg") :     normbg  =   Create(se["normbg"])
    if se.has_key("normbgpower") :     normbgpower =   Create(se["normbgpower"])
    if se.has_key("normbgimg") :     normbgimg   =   Create(se["normbgimg"])
    if se.has_key("fog") :     fog =   Create(se["fog"])
    if se.has_key("fogR") :     fogR    =   Create(se["fogR"])
    if se.has_key("fogG") :     fogG    =   Create(se["fogG"])
    if se.has_key("fogB") :     fogB    =   Create(se["fogB"])


    
    if se.has_key("gamma"):
        gamma   =   Create(se["gamma"])
    if se.has_key("exposure"):
        exposure=   Create(se["exposure"])

    if se.has_key("fullpath"):
        fullpath=Create(se["fullpath"])

    if se.has_key("frame"):
        finname = Create(se["frame"])

    if se.has_key("bmask"):
        bmask = se["bmask"]

    if se.has_key("cpu"):
        cpu = Create(se["cpu"])
    if se.has_key("frstart"):
        frstart=Create(se["frstart"])
    if se.has_key("frend"):
        frend=Create(se["frend"])
    if se.has_key("gzipit"):
        gzipit= Create(se["gzipit"])
    if se.has_key("alphatga"):
        alphatga= Create(se["alphatga"])
    del se


def gui():

    global  globalsettings , shadereditor, lighteditor
    global  gamma, exposure
    global  hemi, HRed, HGreen, HPow, HBlu, HSamples, HSamplesQMC, HQMC, usebground
    global  shadertype, surfshader
    global  shadlist, shadernames, AutoSmooth, Degr
    global  GenR, GenG, GenB, GenSpecR, GenSpecG, GenSpecB, GenRefR, GenRefG, GenRefB, GenRef2R, GenRef2G, GenRef2B, \
        GenTraR, GenTraG, GenTraB, GenTra2R, GenTra2G, GenTra2B
    global  GenHard, GenIOR, GenMinRef, GenFastFresnel,ConstR, ConstG, ConstB
    global  lighttype, lightshader
    global  plpow, plshad, plR, plG, plB, slpow, slshad, \
        slR, slG, slB,sflR, sflG, sflB, sflpow, sflres, sflbias, sflrad
    global  splpow, splR, splG, splB, splbl, splsize, splbeoff, splshadow, splhaloADD, splhalofogR, splhalofogG, splhalofogB, splhalofogdensity, \
                    splhaloblur, splhalosamples, splhalores
    global  addphldiff, addphlcaus, \
                    phlDpow, phlDR, phlDG, phlDB, phlDdep, phlDsearch, phlDmode, phlDang, phlDfix, phlDph, phlDcluster, phlDQMC, \
                    phlCpow, phlCR, phlCG, phlCB, phlCdep, phlCsearch, phlCmode, phlCang, phlCfix, phlCph, phlCcluster, phlCQMC 
    global  linkcolordiffdir, linkcolorcausdiff, linkcolorcausdir, linkpowercausdiff, linkphotonscausdiff, linkdepthcausdiff, \
                    linksearchcausdiff, linkanglecausdiff, linkfixedcausdiff, linkclustercausdiff, linkqmccausdiff, linktargetdiffdir, \
                    linktargetcausdiff, linktargetcausdir
    global  AAPasses, AAMinSamples, AAPixelWidth, AAThreshold
    global  CamRayDepth, CamBias, CamIndSamples, bground, bgroundR, bgroundG, bgroundB  
    #JWM AppView, timer
    global  PathCache,cacheSize,angleThreshold,shadowThreshold, image, appview, timer, alphatga, fname, OutPicMenu, CustSizeX, CustSizeY
    global  additional, addCausIOR, recrad, emirad, addCausTCol,CausIOR,\
        CausTR, CausTG, CausTB, addCausRCol
    global  CausIOR, CausRR, CausRG,CausRB, castsh
    global  loadmenu, savemenu, shname, DISABLE
    global  targetok, alpow, alR, alG, alB, alSamp, alPSamp, alLato, \
        SelTargB, SelTargphlDB, SelTargphlCB, addSpec, addIOR, addRef, addRef2, addTra, addTra2
    global  gzipit, update, FilterDOF, FilterANoise, setfocus,\
        nearblur, farblur, scale, anoise_radius, anoise_maxdelta
    global  TEXROOT, GLOBDB, to_focus, frstart, frend, \
        pathtrace, ptrPow, ptrDepth, ptrSamples, ptrSamplesQMC, ptrQMC, constbg,\
        sunbg, setsun, addsun, turb, sunbgsunpower, sunbgavar, sunbgbvar, sunbgcvar, sunbgdvar, sunbgevar,\
        hdribg, hdriexp, hdriprobe, normbg, normbgpower, normbgimg, fog, fogr, fogg, fogb,\
        fogR, fogG, fogB, fullpath, finname, cpu, anim
    global phlprv
    global cft_Name

    glClearColor(0.4,0.48,0.57, 0.0)    # blue background   
    glClear(GL_COLOR_BUFFER_BIT)
    glColor3f(0, 0, 0)          # main black back
    glRectf(2, 2, 514, 520)
    glColor3f(0.4, 0.48, 0.57)      # main blue back
    glRectf(4, 4, 512, 480)
    glColor3f(0.27, 0.3, 0.35)      # blue top
    glRectf(4, 475,512, 510)

    glColor3f(1,1,1)
    glRasterPos2i(250, 8)
    Text("Yable-0.30  Yet Another Blender Exporter")

    GLOBDB={"OutPicMenu":OutPicMenu.val,\
    "CustSizeX":CustSizeX.val,\
    "CustSizeY":CustSizeY.val,\
    "hemi":hemi.val,"HRed":HRed.val,\
    "HGreen":HGreen.val,"HBlu":HBlu.val,\
    "HPow":HPow.val,\
    "HSamples":HSamples.val,"HSamplesQMC":HSamplesQMC.val,"HQMC":HQMC.val,\
    "usebground":usebground.val,"FilterDOF":FilterDOF.val,\
    "FilterANoise":FilterANoise.val,\
    "AAPasses":AAPasses.val,"AAMinSamples":AAMinSamples.val,"AAPixelWidth":AAPixelWidth.val,"AAThreshold":AAThreshold.val,\
    "CamRayDepth":CamRayDepth.val,"CamBias":CamBias.val,"CamIndSamples":CamIndSamples.val,\
    "bground":bground.val,"bgroundR":bgroundR.val,\
    "bgroundG":bgroundG.val,"bgroundB":bgroundB.val,\
    "nearblur":nearblur.val,"farblur":farblur.val,\
    "scale":scale.val,"anoise_radius":anoise_radius.val,\
    "anoise_maxdelta":anoise_maxdelta.val, "tofocusx":to_focus[0],\
    "tofocusy":to_focus[1],"tofocusz":to_focus[2],\
    "pathtrace": pathtrace.val, "ptrPow":ptrPow.val,\
    "ptrDepth":ptrDepth.val,"ptrSamples":ptrSamples.val,"ptrSamplesQMC":ptrSamplesQMC.val,"ptrQMC":ptrQMC.val,\
    "sunposx":sunpos[0],"sunposy":sunpos[1],"sunposz":sunpos[2],\
    "sunname":sunname,"addsun":addsun.val,"turb":turb.val,"sunbg":sunbg.val,"sunbgsunpower":sunbgsunpower.val,\
    "sunbgavar":sunbgavar.val,"sunbgbvar":sunbgbvar.val,"sunbgcvar":sunbgcvar.val,"sunbgdvar":sunbgdvar.val,"sunbgevar":sunbgevar.val,\
    "hdribg":hdribg.val,"hdriexp":hdriexp.val,"hdriprobe":hdriprobe.val,\
    "normbg":normbg.val,"normbgpower":normbgpower.val,"normbgimg":normbgimg.val,"fog":fog.val, "fogR":fogR.val,"fogG":fogG.val,"fogB":fogB.val,\
    "gamma": gamma.val, "exposure":exposure.val, "fullpath":fullpath.val,\
    "frame": finname.val, "bmask": bmask, "cpu":cpu.val, "frstart":frstart.val,\
    "frend":frend.val, "gzipit":gzipit.val, "alphatga":alphatga.val,
    "cacheSize":cacheSize.val,"angleThreshold":angleThreshold.val,"shadowThreshold":shadowThreshold.val}

    # Change of screenbuttons
    globalsettings = Toggle("Global Settings",  glset,  75,  480, 100, 20, globalsettings.val, \
                                    "Global render settings")
    shadereditor = Toggle("Material Settings",  shset, 175, 480, 100, 20, shadereditor.val, \
                            "Edit and create shaders to apply to objects")
    lighteditor = Toggle("Light Editor",        luzset,  275, 480, 100, 20, lighteditor.val, \
                    "Edit lights for objects that Blender does not support")

    # Global Settings
    if (globalsettings.val == 1):

        glColor3f(0.27, 0.3, 0.35)      # darkblue
        glRectf(275, 438, 465, 288)      # background zone
        
        # texture directory at top of window
        TEXROOT = String("Texture dir: ", ignore_evt, 40, 445, 420, 20, TEXROOT.val, 160, \
            "Main texture directory, must be defined before exporting UV textures!")


        hemi = Toggle("Hemi Light", hemievt,  40, 408, 90, 28, hemi.val, \
                    "Use Hemilight")
        pathtrace = Toggle("Path Light", ptrevt,  132, 408, 90, 28, pathtrace.val, \
                    "Use Path Light")

        if hemi.val:

            if (constbg.val) or (sunbg.val):
                usebground = Toggle("Use Back Ground Color",    updateguievent,  40, 384, 180, 18, usebground.val,\
                        "Add Bg Color to the scene")
                if not(usebground.val):
                    HRed   = Slider("Red     " ,    updateguievent, 40, 366, 160, 15, HRed.val, 0.0, 1.0, 0)
                    HGreen = Slider("Green   " ,    updateguievent, 40, 350, 160, 15, HGreen.val, 0.0, 1.0, 0)
                    HBlu   = Slider("Blu     " ,    updateguievent, 40, 334, 160, 15, HBlu.val, 0.0, 1.0, 0)
                    drawcolorbox(202,381,220,334, HRed.val, HGreen.val,HBlu.val) 

            else:
                    HRed   = Slider("Red     " ,    updateguievent, 40, 366, 160, 15, HRed.val, 0.0, 1.0, 0, "If R=G=B=0 HDRI Active if selected" )
                    HGreen = Slider("Green   " ,    updateguievent, 40, 350, 160, 15, HGreen.val, 0.0, 1.0, 0, "If R!=0 or G!=0 or B!=0 no HDRI, only image background")
                    HBlu   = Slider("Blu     " ,    updateguievent, 40, 334, 160, 15, HBlu.val, 0.0, 1.0, 0, "See Tips on R and G")
                    drawcolorbox(202,381,220,334, HRed.val, HGreen.val,HBlu.val) 

            HPow   = Slider("Power   " , updateguievent, 40, 315, 180, 15, HPow.val, 0.0, 25, 0)
            #HRandSamp = Toggle("Random Sampling",   updateguievent, 40, 299, 180, 15, HRandSamp.val)
            #HSamples = Slider("Samples   ", updateguievent, 40, 299, 180, 15, HSamples.val, 0.0, 512, 0)
            if HQMC.val:                 
                HSamplesQMC = Slider("Samples  ", updateguievent, 40, 299, 180, 15, HSamplesQMC.val, 1, 256, 0, "HemiLight Samples")    
            else:   
                HSamplesSquare = HSamples.val*HSamples.val              
                HSamples = Number("", updateguievent, 40, 299, 99, 15, HSamples.val, 1, 16, "HemiLight Samples controller")             
                glRasterPos2i(142, 303)
                Text("%d Samples" %HSamplesSquare)
            HQMC    =   Toggle("QMC", updateguievent, 40, 283, 89, 15, HQMC.val, "use Quasi Montecarlo sampling")
        
        if pathtrace.val:           
            ptrPow   = Slider("Power   " ,  updateguievent, 40, 366, 180, 15, ptrPow.val, 0.0, 15, 0)           
            #ptrRand = Toggle("Random Sampling",   updateguievent, 40, 350, 180, 15, ptrRand.val)           
            ptrDepth = Slider("Depth: ", updateguievent,40, 350, 180, 15,  ptrDepth.val, 0, 20, 0)
            #ptrSamples = Slider("Samples  ", updateguievent, 40, 350, 180, 15, ptrSamples.val, 0.0, 512, 0)            
            if ptrQMC.val:
                ptrSamplesQMC = Slider("Samples  ", updateguievent, 40, 334, 180, 15, ptrSamplesQMC.val, 1, 512, 0, "PathTracing Samples")  
            else:  
                ptrSamplesSquare = ptrSamples.val*ptrSamples.val                
                ptrSamples = Number("", updateguievent, 40, 334, 99, 15, ptrSamples.val, 1, 16, "PathTracing Samples controller")               
                glRasterPos2i(142, 338)
                Text("%d Samples" %ptrSamplesSquare) 
            ptrQMC  =   Toggle("QMC",  updateguievent,40,318,89,15,ptrQMC.val, "use Quasi Montecarlo Sampling")
            PathCache = Toggle("Cache",updateguievent,40,302,89,15,PathCache.val,"use cached pathlight")
            if PathCache.val:
                cacheSize = Number("Size: ",updateguievent,40,286,70,15,cacheSize.val,     0.0,1.0,"Cache Size")
                angleThreshold =  Number("Angle: ", updateguievent,110,286,80,15,angleThreshold.val,0.0,1.0,"Angle Threshold")
                shadowThreshold = Number("Shdw: ",updateguievent,190,286,80,15,shadowThreshold.val,0.0,1.0,"Shadow Threshold")
            
        #bground = Toggle("Back Ground",            updateguievent,  280, 408, 180, 28, bground.val,\
        #               "Add Bg Color to the scene")
        #if (bground.val == 1):
        
        constbg = Toggle("Const BkGd", constbgevt, 280, 418, 89, 15, constbg.val, "Add a Constant Background")
        sunbg = Toggle("SunSky BkGd", sunbgevt, 371, 418, 89, 15, sunbg.val, "Add a SunSky Background")
        hdribg = Toggle("HDRI BkGd", hdribgevt, 280, 402, 89, 15, hdribg.val, "Add HDRI Background for HEMI or PATH light -- NON hdri for others")
        normbg = Toggle("Normal BkGd", normbgevt, 371, 402, 89, 15, normbg.val, "Add Normal Image Background")      
        if constbg.val:
            bgroundR = Slider("Red     " ,  updateguievent, 280, 374, 160, 15, bgroundR.val, 0.0, 1.0,0)
            bgroundG = Slider("Green   " ,  updateguievent, 280, 358, 160, 15, bgroundG.val, 0.0, 1.0,0)
            bgroundB = Slider("Blu     " ,  updateguievent, 280, 342, 160, 15, bgroundB.val, 0.0, 1.0,0)
            drawcolorbox(442, 389, 460, 342, bgroundR.val, bgroundG.val, bgroundB.val)  
        if sunbg.val:
            setsun = Toggle("Set Sun Pos", updateguievent, 311, 374, 79, 15, setsun.val,\
                            "Sun Position...Select an object (or an EMPTY) in the 3D window and confirm.")
            addsun = Toggle("+Sun", updateguievent, 280, 374, 30, 15, addsun.val,\
                            "Add a \"Real\" Sun Light at the position")
            if setsun.val:
                Button ("Confirm", selsunevt, 392, 374, 70, 15, "Confirm Sun Position")
            
            turb    = Slider("Turbid: ", updateguievent, 280, 357, 180, 15,turb.val, 1, 15, 0,\
                    "Turbidity controls atmosphere density,4 is an almost clear sky")
            if addsun.val:
                sunbgsunpower = Number("Power:", updateguievent, 280, 341, 180, 15, sunbgsunpower.val, 0, 5, \
                            "Scales Sun Brightness: <1 darker, >1 brighter")
            sunbgavar = Number("BrightH:", updateguievent, 280, 325, 89, 15, sunbgavar.val, -255, 255,\
                            "Brightness of the Horizon colors")
            sunbgbvar = Number("SpreadH:", updateguievent, 371, 325, 89, 15, sunbgbvar.val, -255, 255,\
                            "Spread of the Horizon color")
            sunbgcvar = Number("BSGlow:", updateguievent, 280, 309, 89, 15, sunbgcvar.val, -255, 255,\
                            "Brightness of the Sun Glow: <0 black blob")
            sunbgdvar = Number("SSGlow:", updateguievent, 371, 309, 89, 15, sunbgdvar.val, -255, 255,\
                            "Apparent Size of the Sun Glow : higher = smaller glow")
            sunbgevar = Number("BackScattLight:", updateguievent, 280, 293, 180, 15, sunbgevar.val, -255, 255,\
                            "Amount of back-scattered light (red around sun): <0 remove, >0 add")
        if hdribg.val:
            hdriexp = Number("Exposure Adjust : ", updateguievent, 280, 374, 180, 15, hdriexp.val, -5, +5, "- darker , + brighter")
            hdriprobe = String("Probe Name: ", hdriprobeevt, 280, 358, 180, 15, hdriprobe.val, 160, "Only namefile with extension (hdr)")               
        if normbg.val:
            normbgpower = Number("Power : ", updateguievent, 280, 374, 180, 15, normbgpower.val, -5.0, +5.0, "1:default, <1:darker , >1:brighter")
            normbgimg = String("Image Name: ", normbgimgevt, 280, 358, 180, 15, normbgimg.val, 160, "Only namefile with extension")

        fog = Slider("Fog: ",updateguievent, 280, 271, 180, 15, fog.val, 0, 0.5, 0,\
                            "Fog Density, 0 means No Fog")  
        fogr    =   Toggle("Red", fogrevt, 280, 255, 58, 15, fogr.val)
        if fogr.val:
            fogR = Slider("R: ",updateguievent, 280, 239, 160, 15, fogR.val, 0, 1, 0)

        fogg    =   Toggle("Green", foggevt, 341, 255, 58, 15, fogg.val)
        if fogg.val:
            fogG = Slider("G: ",updateguievent, 280, 239, 160, 15, fogG.val, 0, 1, 0)

        fogb    =   Toggle("Blu", fogbevt, 402, 255, 58, 15, fogb.val)
        if fogb.val:
            fogB = Slider("B: ",updateguievent, 280, 239, 160, 15, fogB.val, 0, 1, 0)
        drawcolorbox(440, 254, 460, 239, fogR.val, fogG.val, fogB.val)

        FilterDOF = Toggle("Depth of Field", updateguievent, 40, 255, 180, 15, FilterDOF.val, "Add DOF Filter")
        if FilterDOF.val:
            setfocus = Toggle("Set Focus", updateguievent, 40, 235,90,15, setfocus.val, \
                    "Select an object in the 3D window and confirm")

            if setfocus.val:
                Button ("Confirm", selfocusevt,132, 235, 90, 15)
            nearblur = Slider("Near Blur: ",    updateguievent, 40, 215, 180, 14,nearblur.val, 0.1, 50, 0)
            farblur  = Slider("Far Blur: ", updateguievent, 40, 200, 180, 14,farblur.val, 0.1, 50, 0)
            scale    = Slider("Scale: ",    updateguievent, 40, 185, 180, 14, scale.val, 0.1, 50, 0)

        FilterANoise = Toggle("Anti Noise" ,    updateguievent, 40, 165, 180, 15, FilterANoise.val,\
                        "Add Anti Noise Filter")
        if FilterANoise.val:
            #E: radius range 1-25, max_delta 0-1
            anoise_radius = Slider("Radius: ", updateguievent, 40, 150, 180, 14, anoise_radius.val, 1, 25, 0)
            anoise_maxdelta = Slider("Max Delta: ", updateguievent, 40, 135, 180, 14,anoise_maxdelta.val, 0, 1, 0)
        #Camera Stuff
        #AAsamples = CamSamples.val*CamSamples.val
        glRasterPos2i(280, 102)
        Text("AntiAlias Settings")
        #CamSamples     = Number("AA Quality ", updateguievent, 280, 82, 180, 15, CamSamples.val,1, 16, \
        #       "AA Quality")
        #tolerance  = Slider("Tol.: ",  updateguievent, 280, 62, 180, 15, tolerance.val, 0.001, 1, 0,\
        #   "tolerance: Minimum distance betwen a pixel and its neighbours to allow resampling. ")          
        AAPasses    =   Number("AAP ", updateguievent, 280, 82, 89, 15, AAPasses.val,0, 32, "AAPasses: if zero NO AA")
        AAMinSamples = Number("AAMS ", updateguievent, 371, 82, 89, 15, AAMinSamples.val,1, 32, "AAMinSamples: n of samples per pass")
        AAPixelWidth = Number("AAPW ", updateguievent, 280, 66, 89, 15, AAPixelWidth.val,1.00 ,2.00, "AAPixelWidth - 1.0 OLD method")
        AAThreshold = Number("AAT ", updateguievent, 371, 66, 89, 15, AAThreshold.val,0.00, 1.00, "AAThreshold: 1 NO AA - 0 all pixels") 
                
        CamRayDepth     = Number("Ray Depth  ", updateguievent, 280, 46, 89, 15, CamRayDepth.val,1,20)
        cpu = Number("Cpu", updateguievent,371, 46, 89, 15, cpu.val, 1,16,\
            "Specfify the number of CPU to use")
        CamBias     = Number("Bias: ",  updateguievent, 280, 30, 89, 15, CamBias.val,0.0,5.0)
        CamIndSamplesSquare = CamIndSamples.val*CamIndSamples.val
        CamIndSamples = Number("", updateguievent, 371, 30, 89, 15, CamIndSamples.val, 0, 16, "Indirect Samples controller")                
        glRasterPos2i(373, 20)
        Text("%d IndSamples" %CamIndSamplesSquare)      

        Button("No", noexpevt, 280,196,88, 15, "No Exposure Calculations")
        Button("Default", defexpevt, 370, 196, 90, 15, "Set to sqrt(2)")
        exposure = Slider("expos: ", updateguievent, 280, 180, 180, 15, exposure.val, 0,5,0,\
        "Controls the brightness curve,the default is sqrt(2),lower is darker, higher is brighter")

        Button("No", nogammaevt, 280, 160, 15, 15, "Don't use Gamma Correction")
        gamma = Slider("Gamma", updateguievent, 297, 160, 163,15, gamma.val, 0, 5, 0,\
                "'Gamma' controls the gamma correction, 1.0 is no correction")

        if OutPicMenu.val<9:
            glRasterPos2i(302, 130)
            Text("Resolution -- %s"%(OutSizeList[OutPicMenu.val-1]))
        else:
            glRasterPos2i(280, 152)
            Text("Custom Resolution")

        OutPicMenu  = Menu(OutSize,     updateguievent, 280, 120, 20, 25, OutPicMenu.val,\
                        "Choose or set Your Output Pic Size")
        if OutPicMenu.val == 9:
            #Set Custom Size
            CustSizeX = Number("ResX: ",    updateguievent, 302, 120, 80, 25, CustSizeX.val,\
                    4, 2048, "X Resolution")

            CustSizeY =Number("ResY: ", updateguievent, 382, 120, 78, 25, CustSizeY.val,\
                    4, 2048, "Y Resolution")

        fname       = String("Filename:  ", updateguievent, 116, 50, 140, 19, fname.val, 200, \
                            "Name of the XML file to be produced")  
        image       = String("Image: ",     updateguievent, 116, 30, 140, 19, image.val, 200, \
                            "Name of the rendered image (no extension)")
        alphatga = Toggle("alphaTGA", updateguievent, 116, 10, 70, 19, alphatga.val, \
                            "Save a rendered targa image with the alpha channel")

        Button("Layers", layersetevt, 5,88, 50, 15, "Choose the layers to export")

        anim = Toggle("Anim", updateguievent, 56, 88, 200, 15,anim.val, "Export all frames")
        fullpath = Toggle("Path", updateguievent, 5,72,33,15, fullpath.val, \
                "Export the file using full path in the include tag")

        DISABLE = Toggle("!INC" ,updateguievent, 5, 30, 33, 40, DISABLE.val,\
                        "Disable Include tag: you will get a single and monolithic xml as output")

        gzipit = Toggle(".GZ",          updateguievent, 40, 72, 53, 15,  gzipit.val,\
                "If OFF, disable gzip. Use it if your gzip gives you problems")
        finname = Toggle("Fr",          updateguievent, 95, 72, 53, 15,  finname.val, \
                        "Include frame number in xml and tga file name")

        update = Toggle("Update Meshes",    updateguievent, 148, 72, 107, 15, update.val, \
                        "Update mesh description.set to off to speed up exporting")

        Button("Export",            exportevent, 40, 50, 75, 20, "Start the export")
        Button("Render",            renderevent, 40, 30, 75, 20, "Export the scene and render it")
        appview = Toggle("View Output" ,updateguievent, 40, 10, 75, 20, appview.val,"View with VIEWERAPP after render")
        timer = Toggle("Time", updateguievent, 5,10,33,20,timer.val,"Time the yafray render")
        Button("Exit",              exitevent,  400, 480, 50, 20, "Exit the application")

    # Shader editor settings
    if (shadereditor.val == 1):
        #Seleziono un oggetto e ne definisco lo shader
        glRasterPos2i(200, 35)
        Text('Name: ' + selectedname)
        glRasterPos2i(200, 25)
        Text(selectedtype)
        Button("Get Selected",   50, 10, 30, 180,  20)

        if (selectedtype == 'Type: Mesh'):

            glRasterPos2i(40, 440)
            Text("Shader Type")
            surfshader = Menu(shadertype,           updateguievent,  120, 430, 100, 25,  surfshader.val)

            if(surfshader.val == 1):
                #GenericShader
                glRasterPos2i(40, 400)
                Text("Diffuse Color")
                Button("Bl",        getblcol, 6,350,28,45, "Get it from Blender")
                GenR     = Slider("Red      ", updateguievent, 40, 380, 180, 14, GenR.val, 0.0, 1.0, 0)
                GenG     = Slider("Green    ", updateguievent, 40, 365, 180, 14, GenG.val, 0.0, 1.0, 0)
                GenB     = Slider("Blu      ", updateguievent, 40, 350, 180, 14, GenB.val, 0.0, 1.0, 0)
                drawcolorbox(222, 395,252,350,GenR.val, GenG.val,GenB.val)  

                addSpec = Toggle("Add Specular Color", updateguievent, 40, 331, 180, 16,addSpec.val,\
                            "Add Specular color to the shader")
                if addSpec.val:

                    Button("Bl",        getblspec, 6,285,28,45, "Get it from Blender")
                    GenSpecR = Slider("Red      ", updateguievent   , 40, 315, 180, 14, GenSpecR.val, 0.0, 1.0, 0)
                    GenSpecG = Slider("Green    ", updateguievent   , 40, 300, 180, 14, GenSpecG.val, 0.0, 1.0, 0)
                    GenSpecB = Slider("Blu      ", updateguievent   , 40, 285, 180, 14, GenSpecB.val, 0.0, 1.0, 0)
                    drawcolorbox(222, 330,252,285,GenSpecR.val, GenSpecG.val,GenSpecB.val)  

                #addRef = Toggle("Add Reflection Color", updateguievent, 40, 266, 180, 16,addRef.val,\
                #                   "Add Reflection color to the shader")
                #if addRef.val:

                #   Button("Bl",        getblref, 6, 220, 28,45, "Get it from Blender")
                #   GenRefR  = Slider("Red      ", updateguievent, 40, 250, 180, 14, GenRefR.val, 0.0, 1.0, 0)
                #   GenRefG  = Slider("Green    ", updateguievent, 40, 235, 180, 14, GenRefG.val, 0.0, 1.0, 0)
                #   GenRefB  = Slider("Blu      ", updateguievent, 40, 220, 180, 14, GenRefB.val, 0.0, 1.0, 0)
                #   drawcolorbox(222, 265, 252, 220,GenRefR.val, GenRefG.val,GenRefB.val)
                                
                addRef = Toggle("Refl 1", updateguievent, 40, 266, 89, 16,addRef.val,\
                                    "Add Reflection color 1 to the shader")             
                if addRef.val:
                    addRef2 = Toggle("Refl 2", updateguievent, 131, 266, 89, 16,addRef2.val,\
                                    "Add Reflection color 2 to the shader")
                    Button("Bl",        getblref, 6, 266, 28, 16, "Get it from Blender")
                    GenRefR  = Slider("R ", updateguievent, 30, 250, 99, 14, GenRefR.val, 0.0, 1.0, 0)
                    GenRefG  = Slider("G ", updateguievent, 30, 235, 99, 14, GenRefG.val, 0.0, 1.0, 0)
                    GenRefB  = Slider("B ", updateguievent, 30, 220, 99, 14, GenRefB.val, 0.0, 1.0, 0)
                    drawcolorbox(8, 265, 28, 220, GenRefR.val, GenRefG.val, GenRefB.val)                    
                    if addRef2.val:
                        GenRef2R  = Slider("R ", updateguievent, 131, 250, 99, 14, GenRef2R.val, 0.0, 1.0, 0)
                        GenRef2G  = Slider("G ", updateguievent, 131, 235, 99, 14, GenRef2G.val, 0.0, 1.0, 0)
                        GenRef2B  = Slider("B ", updateguievent, 131, 220, 99, 14, GenRef2B.val, 0.0, 1.0, 0)
                        drawcolorbox(232, 265, 252, 220, GenRef2R.val, GenRef2G.val, GenRef2B.val)
                
                                
                #addTra = Toggle("Add Transmitted Color", updateguievent, 40, 201, 180, 16,addTra.val,\
                #                   "Add Transmitted color to the shader")
                
                #if addTra.val:

                #   GenTraR  = Slider("Red      ",  updateguievent, 40, 185, 180, 14, GenTraR.val, 0.0, 1.0, 0)
                #   GenTraG  = Slider("Green    ",  updateguievent, 40, 170, 180, 14, GenTraG.val, 0.0, 1.0, 0)
                #   GenTraB  = Slider("Blu      ",  updateguievent, 40, 155, 180, 14, GenTraB.val, 0.0, 1.0, 0)
                #   drawcolorbox(222, 200, 252, 155, GenTraR.val, GenTraG.val,GenTraB.val)

                addTra = Toggle("Transm 1", updateguievent, 40, 201, 89, 16,addTra.val,\
                                    "Add Transmitted color 1 to the shader")
                if addTra.val:
                    addTra2 = Toggle("Transm 2", updateguievent, 131, 201, 89, 16,addTra2.val,\
                                    "Add Transmitted color 2 to the shader")
                    GenTraR  = Slider("R ",     updateguievent, 30, 185, 99, 14, GenTraR.val, 0.0, 1.0, 0)
                    GenTraG  = Slider("G ",     updateguievent, 30, 170, 99, 14, GenTraG.val, 0.0, 1.0, 0)
                    GenTraB  = Slider("B ",     updateguievent, 30, 155, 99, 14, GenTraB.val, 0.0, 1.0, 0)
                    drawcolorbox(8, 200, 28, 155, GenTraR.val, GenTraG.val, GenTraB.val)
                    if addTra2.val:
                        GenTra2R  = Slider("R ",    updateguievent, 131, 185, 99, 14, GenTra2R.val, 0.0, 1.0, 0)
                        GenTra2G  = Slider("G ",    updateguievent, 131, 170, 99, 14, GenTra2G.val, 0.0, 1.0, 0)
                        GenTra2B  = Slider("B ",    updateguievent, 131, 155, 99, 14, GenTra2B.val, 0.0, 1.0, 0)
                        drawcolorbox(232, 200, 252, 155, GenTra2R.val, GenTra2G.val, GenTra2B.val)
                                
                GenHard   = Slider ("Hard   ",    updateguievent, 40, 125, 180, 14, GenHard.val, 0.0, 500, 0)
                addIOR    = Toggle  ("Add IOR",   updateguievent, 40, 110, 180, 14, addIOR.val,\
                                    "Click to add IOR parameter")
                if addIOR.val:
                    GenIOR    = Slider ("IOR    ",    updateguievent, 40, 95, 180, 14, GenIOR.val, 0.0, 50., 0)
                GenMinRef = Slider ("Min Ref  ",  updateguievent, 40, 80, 180, 14, GenMinRef.val, 0.0, 1, 0)

                GenFastFresnel = Toggle("Fast Fresnel", updateguievent, 40, 65, 89, 14, GenFastFresnel.val, "Alternative Fresnel Implementation")
                
                additional=Toggle("Object Attributes" ,updateguievent, 280, 430, 180, 20, additional.val)
                if (additional.val == 1):
                    castsh = Toggle("Cast Shadows", updateguievent, 280, 405, 180, 15, castsh.val,\
                            "If on, the object cast Shadows")
                    addCausIOR = Toggle("Caustic IOR", updateguievent, 280,380,180,19, addCausIOR.val)
                    if addCausIOR.val:
                        CausIOR = Slider("CausIOR:", updateguievent, 280, 360,180, 19,CausIOR.val,0.0, 50.,0)
                    recrad = Toggle("Receive Radio", updateguievent, 280, 340,89,19,recrad.val)
                    emirad = Toggle("Emit Radio"    ,updateguievent, 371, 340, 89, 19, emirad.val)

                    addCausTCol=Toggle("Caustic TColor",updateguievent, 280, 316,180,16, addCausTCol.val)
                    if addCausTCol.val:
                        CausTR = Slider("Red      ", updateguievent , 280, 300, 170, 14, CausTR.val, 0.0, 1.0, 0)
                        CausTG = Slider("Green    ", updateguievent , 280, 285, 170, 14, CausTG.val, 0.0, 1.0, 0)
                        CausTB = Slider("Blu      ", updateguievent , 280, 270, 170, 14, CausTB.val, 0.0, 1.0, 0)
                        drawcolorbox(452, 315,470,270,CausTR.val,CausTG.val,CausTB.val)

                    addCausRCol=Toggle("Caustic RColor",updateguievent, 280, 251,180,16, addCausRCol.val)
                    if addCausRCol.val:
                        CausRR = Slider("Red      ", updateguievent , 280, 235, 170, 14, CausRR.val, 0.0, 1.0, 0)
                        CausRG = Slider("Green    ", updateguievent , 280, 220, 170, 14, CausRG.val, 0.0, 1.0, 0)
                        CausRB = Slider("Blu      ", updateguievent , 280, 204, 170, 14, CausRB.val, 0.0, 1.0, 0)
                        drawcolorbox(452, 250,470,204,CausRR.val,CausRG.val,CausRB.val)

                Button("MAT Preview", previewmatevt , 280, 90 , 109, 16, "Preview Mat with yafray") 
                phlprv = Toggle("phlightprv", updateguievent , 391, 90, 69, 16, phlprv.val, "Use Photon light in preview")
                
            elif (surfshader.val == 2): 
                #Constant Shader
                glRasterPos2i(40, 340)
                Text("Color")
                Button("Bl",        getblcol, 6, 290, 28,45, "Get it from Blender")
                ConstR   = Slider("Red      ",  updateguievent, 40, 320, 180, 14, ConstR.val, 0.0, 15.0, 0)
                ConstG   = Slider("Green    ",  updateguievent, 40, 305, 180, 14, ConstG.val, 0.0, 15.0, 0)
                ConstB   = Slider("Blu      ",  updateguievent, 40, 290, 180, 14, ConstB.val, 0.0, 15.0, 0)
                drawcolorbox(222, 335, 252, 290, ConstR.val, ConstG.val,ConstB.val)

            elif (surfshader.val == 4): 
                #Adhoc Shader
                
                glRasterPos2i(40, 340)
                Text("Load from Crafter Preview File:"+CRAFTERFILE)
                #cft_Name = String("Mat Name:",updateguievent,13,290,200,16,cft_Name.val,25,"Crafter Material Name")



            if not (loadmenu or savemenu):

                Button("Load Material",     loadmatevt   , 280, 165, 88, 20, "Load Shader to Archive")
                Button("Save Material",     savematevt   , 370, 165, 88, 20, "Save Shader from Archive")
            else:

                if(loadmenu):
                    if (len(shadlist)):
                        shadernames = Menu (shadlist,  updateguievent,  280, 140, 180, 20,  shadernames.val)
                        Button("OK",    okload   , 280, 165, 88, 20, "Load!")

                    else:
                        glColor3f(1,0,0)
                        glRasterPos2i(280, 190)
                        Text("No shaders Available!")
                        glColor3f(1,1,1)
                if(savemenu):

                    shname      = String("Name :  ",    updateguievent, 280, 140, 180, 20, shname.val, 200, "Name of the shader")   
                    Button("OK",    oksave   , 280, 165, 88, 20, "Save!")


                Button("Cancel",        cancel   , 370, 165, 88, 20, "Do nothing")

            AutoSmooth=Toggle("AutoSmooth",     updateguievent, 280, 110,78,15,AutoSmooth.val)

            if AutoSmooth.val:
                Degr=Slider("Degr:",        updateguievent, 360, 110, 98,15, Degr.val, 1, 90, 0)

            if not (assigned or loadmenu or savemenu):

                Button("Modulators", gototexevt, 280, 65, 180, 20, "Go to the texture window")
                Button("Assign",    assignmeshevt, 370, 15, 90, 30, "Assign Shader to the Selected Mesh")

                #JWM - MultiAssign
                Button("Assign All",    assignallmeshevt, 300, 15, 70, 30, "Assign Shader to all the Selected Meshes")

    #Light editor
    if (lighteditor.val == 1):
        #glClearColor(0.4,0.48,0.57, 0.0)   # blue background   
        #glClear(GL_COLOR_BUFFER_BIT)
        glColor3f(0, 0, 0)          # main black back
        #glRectf(2, 2, 514, 520)
        glRectf(2, 2, 530, 520)
        glColor3f(0.4, 0.48, 0.57)      # main blue back
        glRectf(4, 4, 528, 480)
        glColor3f(0.27, 0.3, 0.35)      # blue top
        glRectf(4, 475,528, 510)

        #Seleziono una Lamp  e ne definisco la caratteristica
        glRasterPos2i(200, 35)
        Text('Name: ' + selectedname)       
        glRasterPos2i(200, 15)
        #Text(selectedtype)
        Button("Get Selected", 50, 10, 30, 180,  20)
        if (selectedtype == 'Type: Lamp'):

            lampObj = Blender.Object.Get(selectedname)
            if lampObj==None: return    # DELETED LAMPS RETURN NONE

            glColor3f(1,1,1)
            glRasterPos2i(25, 450)
            Text("DIRECT Light Type")
            lightshader = Menu(lighttype, updateguievent,  30, 420, 100, 20,  lightshader.val)
            glColor3f(1,1,1)
            glRasterPos2i(250, 450)
            Text("INDIRECT and CAUSTIC Settings")
            addphldiff = Toggle("Diffuse", updateguievent, 200, 420, 100, 20, addphldiff.val, "Activate Photons for Indirect Light")
            addphlcaus = Toggle("Caustic", updateguievent, 370, 420, 100, 20, addphlcaus.val, "Activate Photons for Caustics")
            
            
            if (lightshader.val == 1):
                #Point light - Draw Stuff Here
                plpow =     Number("Power: ", updateguievent, 30, 400, 100, 15, plpow.val, 0.0, 500)
                plR =       Number("Red      ", linklightevt, 30, 384, 100, 15, plR.val, 0.0,1.0)
                plG =       Number("Green    ", linklightevt, 30, 368, 100, 15, plG.val, 0.0,1.0)
                plB =       Number("Blu     ", linklightevt, 30, 352, 100, 15, plB.val, 0.0,1.0)
                drawcolorbox    (132, 399, 152, 352, plR.val, plG.val,plB.val)
                plshad =    Toggle("Cast Shadow",   updateguievent, 30, 336, 100, 15, plshad.val)
                linktargetdiffdir.val = 0
                linktargetcausdir.val = 0
            elif (lightshader.val == 2):
                #Sun Light- Draw Stuff Here
                slpow =     Number("Power: ",   updateguievent, 30, 400, 100, 15, slpow.val, 0.0, 50)
                slR =       Number("Red      ",     linklightevt, 30, 384, 100, 15, slR.val,0.0,1.0)
                slG =       Number("Green    ",     linklightevt, 30, 368, 100, 15, slG.val,0.0,1.0)
                slB =       Number("Blu      ",     linklightevt, 30, 352, 100, 15, slB.val,0.0,1.0)
                drawcolorbox    (132, 399, 152, 352, slR.val, slG.val,slB.val)
                slshad =    Toggle("Cast Shadow",   updateguievent, 30, 336, 100, 15, slshad.val)
                linktargetdiffdir.val = 0
                linktargetcausdir.val = 0               
            elif (lightshader.val == 3):
                #Soft Light- Draw Stuff Here
                sflpow =    Number("Power: ",   updateguievent, 30, 400, 100, 15, sflpow.val, 0.0, 10000)               
                sflR =      Number("Red      ",     linklightevt, 30, 384, 100, 15, sflR.val, 0.0,1.0)
                sflG =      Number("Green    ",     linklightevt, 30, 368, 100, 15, sflG.val, 0.0,1.0)
                sflB =      Number("Blu      ",     linklightevt, 30, 352, 100, 15, sflB.val, 0.0,1.0)
                drawcolorbox    (132, 399, 152, 352, sflR.val, sflG.val,sflB.val)
                sflres =    Number("Resol:    ",    updateguievent, 30, 336, 100, 15, sflres.val, 0, 4096)
                sflrad =  Number("Radius:   ",      updateguievent, 30, 320, 100, 15, sflrad.val, 0, 120)
                sflbias=    Number("Bias:     ",    updateguievent, 30, 304, 100, 15, sflbias.val,0.0,5.0)
                linktargetdiffdir.val = 0
                linktargetcausdir.val = 0                               
            elif (lightshader.val == 4):
                #AreaLight              
                alpow   = Number("Power: ",  updateguievent, 30, 400, 100, 15, alpow.val, 0.0, 1000)
                alR = Number("Red      ",    linklightevt, 30, 384, 100, 15, alR.val, 0.0,1.0)
                alG     = Number("Green    ",    linklightevt, 30, 368, 100, 15, alG.val, 0.0,1.0)
                alB     = Number("Blu      ",    linklightevt, 30, 352, 100, 15, alB.val, 0.0,1.0)
                drawcolorbox    (132, 399, 152, 352, alR.val, alG.val,alB.val)
                alSamp  = Number("Samples:   ",   updateguievent, 30, 336, 100, 15, alSamp.val, 0.0, 1024)
                alPSamp = Number("PSamples:   ",  updateguievent, 30, 320, 100, 15, alPSamp.val, 0.0, 1024,\
                                            "Penumbra prediction")
                alLato  = Number("Side:", updateguievent, 30, 304, 100, 15, alLato.val, 1.0, 250,\
                                        "Height/Width of the Quad")
                SelTargB= Toggle("Select Target",updateguievent, 30, 116, 100, 15, SelTargB.val,"Default: Origin")
                if SelTargB.val:
                    Button("Confirm Target", seltargetevt,  30, 100, 100, 15)
            
            elif (lightshader.val == 5):
                #FIX ... Blender Creator223 compatibility
                #A "Spot" lamp Selected 
                if BL_VERSION<=223:
                    lampData=None
                else:
                    lampData=lampObj.getData()
                    
                """
                if lampData:
                    if lampData.getType() == "Spot":
                """    
## jms : fix for Blender 2.28   3/n --  start                     
                if lampData:
                    if lampData.getType() == "2":
                        #It is a Spot...
                        Button ("Bl", getspcolevt, 5, 352, 20, 47, \
                                "Get color from Blender")
                        Button("Bl", getspsievt, 5, 336, 20, 15, \
                                "Get Size from Blender" )
                        Button("Bl", getspblevt, 5, 320, 20, 15,\
                                "Get Blend from Blender!")
## jms : fix for Blender 2.28   3/n --  end
                        
                splpow  =   Number("Power: ",   updateguievent, 30, 400, 100, 15, splpow.val, 0.1, 5000)
                splR =      Number("Red     ", linklightevt, 30, 384, 100, 15, splR.val, 0.0, 1.0)
                splG =      Number("Green   ", linklightevt, 30, 368, 100, 15, splG.val, 0.0, 1.0)
                splB =      Number("Blu     ", linklightevt, 30, 352, 100, 15, splB.val, 0.0, 1.0)
                drawcolorbox(132, 399, 152, 352, splR.val, splG.val, splB.val)
                splsize=    Number("Width: ",   updateguievent, 30, 336, 100, 15, splsize.val, 1,180,\
                            "The width of the lightcone in degrees (SpotSi)")
                splbl=      Number("Blend: ",   updateguievent, 30, 320, 100, 15, splbl.val, 0, 50,\
                        "Transition width of the lightcone, it controls the softness of the edge(SpotBl)")
                splbeoff=   Number("Falloff:",  updateguievent, 30, 304, 100, 15, splbeoff.val, 0, 15,\
                        "Control the light distribution fallof inside the lightcone. The higher, the softer")
                splshadow = Toggle("Cast Shadow", updateguievent, 30, 288, 100, 15, splshadow.val,"Cast Shadows if also shadow object attribute is on")
                splhaloADD = Toggle("Halo",updateguievent, 30, 272, 100, 15, splhaloADD.val,"Add halo volumetric effect to spotlight")
                if splhaloADD.val:
                    splhalofogR = Number("Red     ", updateguievent, 30, 256, 100, 15, splhalofogR.val, 0.0, 1.0,"Red component color of fog")
                    splhalofogG = Number("Green     ", updateguievent, 30, 240, 100, 15, splhalofogG.val, 0.0, 1.0,"Green component color of fog")
                    splhalofogB = Number("Blu     ", updateguievent, 30, 224, 100, 15, splhalofogB.val, 0.0, 1.0,"Blue component color of fog")
                    drawcolorbox(132, 271, 152, 224, splhalofogR.val, splhalofogG.val, splhalofogB.val)
                    splhalofogdensity = Number("Density:", updateguievent, 30, 208, 100, 15, splhalofogdensity.val, 0.0, 1.0,"Amount of fog in the Halo")
                    splhaloblur = Number("Blur: ", updateguievent, 30, 192, 100, 15, splhaloblur.val, 0.0, 1.0,"Blur applied to volumetric shadow")
                    splhalores = Number("Res: ", updateguievent, 30, 176, 100, 15, splhalores.val, 0, 512,"Resolution of the shadowmap")
                    splhalosamples = Number("Samples:", updateguievent, 30, 160, 100, 15, splhalosamples.val, 0, 512,"Number of samples for halo rendering")
                SelTargB= Toggle("Select Target",updateguievent, 30, 116, 100, 15, SelTargB.val,"Default: Origin")
                if SelTargB.val:
                    Button("Confirm Target", seltargetevt,  30, 100, 100, 15)
            
            ## Photons Zone ##
            if (addphldiff.val):
                phlDpow = Number("Power: ", linklightevt, 200, 400, 100, 15, phlDpow.val, 0.1, 10000)
                phlDR = Number("Red     ",      linklightevt, 200, 384, 100, 15, phlDR.val, 0.0, 1.0)
                phlDG = Number("Green   ",      linklightevt, 200, 368, 100, 15, phlDG.val, 0.0, 1.0)
                phlDB = Number("Blu     ",      linklightevt, 200, 352, 100, 15, phlDB.val, 0.0, 1.0)
                drawcolorbox(302, 399, 322, 352, phlDR.val, phlDG.val, phlDB.val)
                phlDph  =   Number("Photons:",  linklightevt, 200, 336, 100, 15, phlDph.val, 100,1000000)
                phlDdep =   Number("Depth:   ",     linklightevt, 200, 320, 100, 15, phlDdep.val, 1, 20)
                phlDsearch =    Number("Search:  ", linklightevt, 200, 304, 100, 15, phlDsearch.val, 1, 800)
                phlDang =   Number("Angle:  ",  linklightevt,   200, 288, 100, 15, phlDang.val, 0, 180)
                phlDfix = Number("Fixed:  ", linklightevt, 200, 272, 100, 15, phlDfix.val, 0.0, 10.0)
                phlDcluster = Number("Cluster:  ", linklightevt, 200, 256, 100, 15, phlDcluster.val, 0.0, phlDfix.val)
                phlDQMC = Toggle("QMC", linklightevt, 200, 240, 100, 15, phlDQMC.val,"Use Quasi Montecarlo Sampling")
                if not linktargetdiffdir.val:
                    SelTargphlDB= Toggle("Select Target",updateguievent, 200, 116, 100, 15, SelTargphlDB.val,"Default: Origin")
                if SelTargphlDB.val:
                    Button("Confirm Target", seltargetphlDevt, 200, 100, 100, 15)
                if (lightshader.val):
                    linkcolordiffdir = Toggle("->", linklightevt, 169, 368, 30, 15, linkcolordiffdir.val, "link diffuse-direct colors")
                    if (lightshader.val == 4 or lightshader.val == 5):
                        linktargetdiffdir = Toggle("->", linklightevt, 169, 116, 30, 15, linktargetdiffdir.val, "link diffuse-direct targets")
            else:
                linkcolordiffdir.val = 0
                linktargetdiffdir.val = 0
                
            if (addphlcaus.val):
                phlCpow = Number("Power: ", linklightevt, 370, 400, 100, 15, phlCpow.val, 0.1, 10000)
                phlCR = Number("Red     ",      linklightevt, 370, 384, 100, 15, phlCR.val, 0.0, 1.0)
                phlCG = Number("Green   ",      linklightevt, 370, 368, 100, 15, phlCG.val, 0.0, 1.0)
                phlCB = Number("Blu     ",      linklightevt, 370, 352, 100, 15, phlCB.val, 0.0, 1.0)
                drawcolorbox(472, 399, 492, 352, phlCR.val, phlCG.val, phlCB.val)
                phlCph  =   Number("Photons:",  linklightevt, 370, 336, 100, 15, phlCph.val, 100,1000000)
                phlCdep =   Number("Depth:   ",     linklightevt, 370, 320, 100, 15, phlCdep.val, 1, 20)
                phlCsearch =    Number("Search:  ", linklightevt, 370, 304, 100, 15, phlCsearch.val, 1, 200)
                phlCang =   Number("Angle:  ",  linklightevt,   370, 288, 100, 15, phlCang.val, 0, 180)
                phlCfix = Number("Fixed:  ", linklightevt, 370, 272, 100, 15, phlCfix.val, 0.0, 10.0)
                phlCcluster = Number("Cluster:  ", linklightevt, 370, 256, 100, 15, phlCcluster.val, 0.0, phlCfix.val)
                phlCQMC = Toggle("QMC", linklightevt, 370, 240, 100, 15, phlCQMC.val,"Use Quasi Montecarlo Sampling")
                if not (linktargetcausdiff.val or linktargetcausdir.val):
                    SelTargphlCB= Toggle("Select Target",updateguievent, 370, 116, 100, 15, SelTargphlCB.val,"Default: Origin")             
                if SelTargphlCB.val:
                    Button("Confirm Target", seltargetphlCevt, 370, 100, 100, 15)
                if (addphldiff.val):
                    linkcolorcausdiff = Toggle("->", linklightevt, 339, 368, 30, 15, linkcolorcausdiff.val, "link caustic-diffuse colors")
                    linkpowercausdiff = Toggle("->", linklightevt, 339, 400, 30, 15, linkpowercausdiff.val, "link caustic-diffuse powers")
                    linkphotonscausdiff = Toggle("->", linklightevt, 339, 336, 30, 15, linkphotonscausdiff.val, "link caustic-diffuse photons")
                    linkdepthcausdiff = Toggle("->", linklightevt, 339, 320, 30, 15, linkdepthcausdiff.val, "link caustic-diffuse depths")
                    linksearchcausdiff = Toggle("->", linklightevt, 339, 304, 30, 15, linksearchcausdiff.val, "link caustic-diffuse search")
                    linkanglecausdiff = Toggle("->", linklightevt, 339, 288, 30, 15, linkanglecausdiff.val, "link caustic-diffuse angles")
                    linkfixedcausdiff = Toggle("->", linklightevt, 339, 272, 30, 15, linkfixedcausdiff.val, "link caustic-diffuse fixed")
                    linkclustercausdiff = Toggle("->", linklightevt, 339, 256, 30, 15, linkclustercausdiff.val, "link caustic-diffuse clusters")
                    linkqmccausdiff = Toggle("->", linklightevt, 339, 240, 30, 15, linkqmccausdiff.val, "link caustic-diffuse qmcs")    
                    linktargetcausdiff = Toggle("->", linklightevt, 339, 116, 30, 15, linktargetcausdiff.val, "link caustic-diffuse targets")
                if lightshader.val:
                    linkcolorcausdir = Toggle("<-", linklightevt, 493, 368, 30, 15, linkcolorcausdir.val, "link caustic-direct colors")
                    if (lightshader.val == 4 or lightshader.val == 5):
                        linktargetcausdir = Toggle("<-", linklightevt, 493, 116, 30, 15, linktargetcausdir.val, "link caustic-direct targets")
            else:
                linkcolorcausdiff.val = 0
                linkpowercausdiff.val = 0
                linkphotonscausdiff.val = 0
                linkdepthcausdiff.val = 0
                linksearchcausdiff.val = 0
                linkanglecausdiff.val = 0
                linkfixedcausdiff.val = 0
                linkclustercausdiff.val = 0
                linkqmccausdiff.val = 0
                linktargetcausdiff.val = 0              

            if not (assigned or SelTargB.val or SelTargphlDB.val or SelTargphlCB.val):
                Button("Assign",        assignlightevt, 280, 30, 180, 30, \
                            "Assign Shader to the Selected Light")
                                #JWM - MultiAssign
                Button("Assign All",    assignalllightevt, 280, 60, 180, 30, "Assign Shader to all the Selected Lights")



def event(evt, val):
    if (evt == ESCKEY and not val): 
        Exit()


def bevent(evt):

    global meshmat, shadlist, loadmenu, savemenu, assigned, targetok, SelTargB, SelTargphlDB, SelTargphlCB
    global setfocus, BL_VERSION
    global sunbg, hdribg, normbg
    global fullpath, constbg, hdribg, normbg, constbg, sunbg
    global normbg, hdriprobe, constbg, sunbg, hdribg, normbgimg, normbgimg, pathtrace, hemi
    global hdriprobe, allLoopSelected,  allLoopSelected, shname, topos,  toposphlD, toposphlC
    global fogg, fogb, fogr, sunpos, sunname, to_focus, GenR, GenG, GenB, ConstR, ConstG, ConstB
    global GenSpecR, GenSpecG, GenSpecB, GenRefR, GenRefG, GenRefB, splR, splG, splB, splsize
    global splbl, texmenu, gamma, exposure


    if (evt == exportevent):    # Export button
        if anim.val:
            #oldpth=fullpath.val
            #fullpath.val=1
            oldfr=finname.val
            finname.val=1
            sta = Blender.Get('staframe')
            end = Blender.Get('endframe')
            cur = Blender.Get("curframe")
            if Blender.Get("version") > 223  :
                curscene = Blender.Scene.getCurrent()
                for frame in range(sta, end+ 1):
                    #curscene.currentFrame(frame)
                    #curscene.update()

## jms : modif for 228
                    if BL_VERSION <228:
                       curscene.frameSettings(curscene.startFrame(), curscene.endFrame(), frame)
                    else:
                       Blender.Set('curframe',frame)
## jms : modif for 228

                    #Blender.Window.Redraw("View")
                    #Blender.Window.Redraw("Button")
                    Blender.Window.RedrawAll()
                    export()

## jms : modif for 228
                if BL_VERSION <228:
                   curscene = Blender.Scene.getCurrent()
                   curscene.frameSettings(curscene.startFrame(), curscene.endFrame(), cur)
                   #curscene.currentFrame(cur)
                   curscene.update()
                else:
                       Blender.Set('curframe',frame)
## jms : modif for 228

                Blender.Window.QRedrawAll()
            else:
                for frame in range(sta, end+1):
                    Blender.Set("curframe", frame)
                    Blender.Redraw()
                    export()
                Blender.Set("curframe", cur)
                Blender.Redraw()
            #fullpath.val=oldpth
            finname.val=oldfr
        else:
            export()

    if (evt == renderevent):# Render button
        
        oldfp=fullpath.val
        fullpath.val = 1
        filename = export()
        fullpath.val = oldfp
        print filename
        cpus = "-c%d"%cpu.val
        print "CPU", cpus
        start = 0
        end = 0
        try:
            #JWM Run render in P_WAIT
            if timer.val or appview.val:
                if POSIX:
                    #os.spawnvp(1, "yafray", tuple(("", cpus, filename)))
                    os.spawnlp(os.P_WAIT, "yafray", "", cpus, filename )
                else:
                    start = time.time()
                    #os.spawnl(1, "yafray","", cpus, filename)
                    os.spawnl(os.P_WAIT, 'C:\yafray\yafray.exe','C:\yafray\yafray.exe', cpus, filename)
                    end = time.time()
            else:
                if POSIX:
                    #os.spawnvp(1, "yafray", tuple(("", cpus, filename)))
                    os.spawnlp(os.P_NOWAIT, "yafray", "", cpus, filename )
                else:
                    #os.spawnl(1, "yafray","", cpus, filename)
                    os.spawnl(os.P_NOWAIT, 'C:\yafray\yafray.exe','C:\yafray\yafray.exe', cpus, filename)
            
        except OSError:
            print "Cannot launch renderer!!"

        #JWM                    
        if timer.val:
            tme = end - start
            hrs = int( tme / 3600 )
            min = int( (tme - hrs * 3600) / 60)
            sec = round(((tme - hrs*3600) - (min * 60)),2)
            if min < 10:
                min = "0"+str(min)
            if sec < 10:
                sec = "0"+str(min)
            print str(hrs)+":"+str(min)+":"+str(sec)+" to Render"

        if appview.val:
            try:
                os.spawnl(os.P_NOWAIT, VIEWERAPP,VIEWERAPP,appviewfilename)
            except OSError:
                print "Cannot Launch Viewer!!"

    if (evt == layersetevt):
        print "Layer evt"
        getlayers()
        Register(layergui, layerevent, layerbevent)

    if (evt == exitevent):  # Exit button
        cooksettings()
        Exit()

    if (evt == updateguievent):# Update gui
        Register(gui, event, bevent)

    if(evt == constbgevt):            # con booleani
        if constbg.val:
            if sunbg.val or hdribg.val or normbg.val:
                
                sunbg.val = 0
                hdribg.val = 0
                normbg.val = 0
        Register(gui, event, bevent)

    if(evt == sunbgevt):              # con due if
        if sunbg.val:
            if constbg.val:
                
                constbg.val = 0
            if hdribg.val:


                hdribg.val = 0
            if normbg.val:

                normbg.val = 0
        Register(gui, event, bevent)

    if(evt == hdribgevt):              # con due if
        if hdribg.val:
            if constbg.val:
                constbg.val = 0
            if sunbg.val:
                sunbg.val = 0
            if normbg.val:
                normbg.val = 0
        Register(gui, event, bevent)
    
    if(evt == hdriprobeevt):
        if not (hdriprobe.val == ""):
            hdriprobe.val = hdriprobe.val
        else:
           
            hdriprobe.val = "EMPTY"
        Register(gui, event, bevent)
    
    if(evt == normbgevt):              # con due if
        if normbg.val:
            if constbg.val:
                constbg.val = 0
            if sunbg.val:
                sunbg.val = 0
            if hdribg.val:
                hdribg.val = 0
        Register(gui, event, bevent)
    
    if(evt == normbgimgevt):
        if not (normbgimg.val == ""):

            normbgimg.val = normbgimg.val
        else:
            normbgimg.val = "EMPTY"
        Register(gui, event, bevent)
    
    if(evt == hemievt):
        if hemi.val:
            if pathtrace.val:
                pathtrace.val = 0
        Register(gui, event, bevent)

    if(evt == ptrevt):
        if pathtrace.val:
            if hemi.val:

                hemi.val = 0
        Register(gui, event, bevent)

    if (evt == assignmeshevt):  #Assign Shader      
        assigned = 1
        if (surfshader.val == 3):
            #Default Shader
            #BUG!! non esporto AutoSmooth
            if meshmat.has_key(selectedname):
                if not(meshmat[selectedname] == "Default"):
                    elimina(matdir, selectedname+".mat")
            meshmat[selectedname] = "Default"
        elif surfshader.val == 4:
                assigncrafttomesh()
        else:
            assigntomesh()
        Register(gui, event, bevent)

    #JWM MultiAssign
    if (evt == assignallmeshevt):   #Assign Shader  
        
        print "Doing Multi Assign"
        All = Blender.Object.getSelected()
        for allLoopSelected in All:
            getSelectedObjects()
            print selectedname,"run"
            assigned = 1
            if (surfshader.val == 3):
                #Default Shader
                #BUG!! non esporto AutoSmooth
                if meshmat.has_key(selectedname):
                    if not(meshmat[selectedname] == "Default"):
                        elimina(matdir, selectedname+".mat")
                meshmat[selectedname] = "Default"
            elif surfshader.val == 4:
                assigncrafttomesh()
            else:
                assigntomesh()
        Register(gui, event, bevent)

    if (evt == savematevt):
        savemenu=1
        Register(gui, event, bevent)

    if (evt == loadmatevt):
        loadmenu=1
        shadlist=scanarchive()
        Register(gui, event, bevent)

    if (evt== okload):
        lista = string.split(shadlist,"|")
        matf = os.path.join(archdir,lista[shadernames.val-1])
        matf +=".mat"
        loadmat(matf)       
        optattr = scanlocal(archdir, ".objattr")
        if optattr.has_key(lista[shadernames.val-1]):
            attrf = os.path.join(archdir,lista[shadernames.val-1])
            attrf +=".objattr"      
            print("okload-->loadattr")
            loadattr(attrf)
        else:
            additional.val = 0      
        loadmenu=0
        Register(gui, event, bevent)

    if (evt == oksave):
        
        if not (surfshader.val == 3):
            savemat()
            savemenu = 0
            shname.val = "fooname"
        Register(gui, event, bevent)

    if (evt == cancel):
        loadmenu = 0
        savemenu = 0
        Register(gui, event, bevent)

    if (evt == assignlightevt):     
        assigned = 1
        assigntolight()
        Register(gui, event, bevent)

    if (evt == assignalllightevt):

        
        
        print "Doing Multi Light Assign"
        All = Blender.Object.getSelected()
        for allLoopSelected in All:
            getSelectedObjects()        
            assigned = 1
            assigntolight()
        Register(gui, event, bevent)

    if (evt == seltargetevt):
        dummyname,topos = getTargetPos()
        SelTargB.val = 0
        if linktargetdiffdir.val:           
            toposphlD[0] = topos[0]
            toposphlD[1] = topos[1]
            toposphlD[2] = topos[2]             
        if linktargetcausdiff.val:
            toposphlC[0] = toposphlD[0]
            toposphlC[1] = toposphlD[1]
            toposphlC[2] = toposphlD[2]
        if linktargetcausdir.val:
            toposphlC[0] = topos[0]
            toposphlC[1] = topos[1]
            toposphlC[2] = topos[2]
        Register(gui, event, bevent)

    if (evt == seltargetphlDevt):
        dummyname,toposphlD = getTargetPos()
        SelTargphlDB.val = 0
        if linktargetcausdiff.val:
            toposphlC[0] = toposphlD[0]
            toposphlC[1] = toposphlD[1]
            toposphlC[2] = toposphlD[2]
        Register(gui, event, bevent)

    if (evt == seltargetphlCevt):
        dummyname,toposphlC = getTargetPos()
        SelTargphlCB.val = 0        
        Register(gui, event, bevent)


    if (evt == selsunevt):
        setsun.val = 0
        sunname, sunpos = getTargetPos()
        Register(gui, event, bevent)

    if (evt == fogrevt):
 
        if fogr.val:
            fogg.val = 0
            fogb.val = 0
        else:
            fogr.val = 1
        Register (gui, event, bevent)

    if (evt == foggevt):
        if fogg.val:
            fogr.val = 0
            fogb.val = 0
        else:
            fogg.val =  1
        Register (gui, event, bevent)

    if (evt == fogbevt):
        if fogb.val:
            fogr.val = 0
            fogg.val = 0
        else:
            fogb.val = 1
        Register (gui, event, bevent)

    if (evt == selfocusevt):
        setfocus.val = 0
        dummyname, to_focus = getTargetPos()
        Register(gui, event, bevent)

    if (evt == getblcol):
        if blcolors:
            if surfshader.val == 1:
                GenR.val = blcolors[0][0]
                GenG.val = blcolors[0][1]
                GenB.val = blcolors[0][2]

            elif surfshader.val == 2:
                ConstR.val=blcolors[0][0]
                ConstG.val=blcolors[0][1]
                ConstB.val=blcolors[0][2]
        Register(gui, event, bevent)

    if (evt == getblspec):
        if blcolors:
            GenSpecR.val = blcolors[1][0]
            GenSpecG.val = blcolors[1][1]
            GenSpecB.val = blcolors[1][2]
        Register(gui, event, bevent)
        
    if (evt == getblref):

        if blcolors:
            GenRefR.val = blcolors[2][0]
            GenRefG.val = blcolors[2][1]
            GenRefB.val = blcolors[2][2]
        Register(gui, event, bevent)

    if evt==getspcolevt:
        splR.val,splG.val,splB.val = lampdat[0]
        Register(gui, event, bevent)

    if evt == getspsievt:
        splsize.val=lampdat[1]
        Register(gui, event, bevent)
        
    if evt == getspblevt:
        splbl.val = lampdat[2]
        Register(gui, event, bevent)

    if (evt == gototexevt):
        retrieveUV()
        texmenu = retrieve_modulators()
        #print texmenu
        Register(texgui, texevent, texbevent)

    if (evt == glset):
        shadereditor.val = 0
        lighteditor.val = 0
        Register(gui, event, bevent)

    if (evt== shset):
        globalsettings.val = 0
        lighteditor.val = 0
        Register(gui, event, bevent)

    if (evt== luzset):
        globalsettings.val = 0
        shadereditor.val = 0
        Register(gui, event, bevent)

    if evt == nogammaevt:
        gamma.val = 1.0
        Register(gui, event, bevent)

    if evt == noexpevt:
 
        exposure.val = 0
        Register(gui, event, bevent)

    if evt == defexpevt:
       
        exposure.val = sqrt(2)
        Register(gui, event, bevent)

    if (evt == 50):
        worked = getSelectedObject()
        if worked:
            recalldata()
            Register (gui, event, bevent)
        Register(gui, event, bevent)

    if (evt == 80):
        Register(gui, event, bevent)

    if (evt == previewmatevt):
        matpreview()

    if (evt == linklightevt):
        Register(gui, event, bevent)
        linklightitems()
        Register(gui, event, bevent)

#AUXILIAR GUI. POPS UP WHEN GLOBAL SETTINGS ARE NOT  FOUND
def startgui():
    
    global TEXROOT
    glClearColor(0.4,0.48,0.57, 0.0)    # blue background   
    glClear(GL_COLOR_BUFFER_BIT)                           
    #glColor3f(1, 0, 0)         # main black back
    #glRectf(2, 2, 514, 520)
    #glColor3f(0.4, 0.48, 0.7)      # main blue back
    #glRectf(4, 4, 512, 480)
    #glColor3f(0.3, 0.3, 0.35)      # blue top
    #glRectf(4, 475,512, 510)
    glColor3f(1,1,1)
    TEXROOT = String("Texture dir: ", ignore_evt, 40, 250, 420, 25, TEXROOT.val, 160, \
        "Main texture directory, must be defined before exporting UV textures!")
    Button("Start YaBle",  exitevent,  200, 200, 110, 30, "Start YaBle")

def startevent(evt, val):
    if (evt == ESCKEY and not val): 
        Exit()

def startbevent(evt):
    global TEXROOT
    if (evt == exitevent):  
        TEXROOT = Create(TEXROOT.val)
        Register (gui, event, bevent)

"""
Buttons for the texture Gui

"""
#UV texture as in Blender
mixmode     =   Create(0)
mulmode     =   Create(1)
addmode     =   Create(0)

kcmode      =   Create(1)
ksmode      =   Create(0)
krmode      =   Create(0)
khmode      =   Create(0)
ktmode      =   Create(0)

kcvalue     =   Create(1.00)
ksvalue     =   Create(1.00)
krvalue     =   Create(1.00)
khvalue     =   Create(1.00)
ktvalue     =   Create(1.00)
knormvalue = Create(0.0)
sizex       =   Create(1.00)
sizey       =   Create(1.00)
sizez       =   Create(1.00)
uvsize      =   Create(1.0)

#Other Textures
oth_mixmode     =   Create(0)
oth_mulmode =   Create(1)
oth_addmode =   Create(0)

oth_kcmode  =   Create(1)
oth_ksmode  =   Create(0)
oth_krmode  =   Create(0)
oth_khmode  =   Create(0)
oth_ktmode  =   Create(0)

oth_kcvalue =   Create(1.00)
oth_ksvalue =   Create(1.00)
oth_krvalue =   Create(1.00)
oth_khvalue =   Create(1.00)
oth_ktvalue =   Create(1.00)
oth_knormvalue = Create(0.0)
oth_sizex   =   Create(1.00)
oth_sizey   =   Create(1.00)
oth_sizez   =   Create(1.00)
oth_size    =   Create(1.0)

txmenu      =   Create(1)

txname      =   Create("GIVE_ME_A_NAME")
#Strange, without this the String button doesn't show any string
#The first time you call it in the gui
imgpath     =   Create("")

#The Clouds Texture
cldepth     =   Create(2)
selcol1     =   Create(1)
selcol2     = Create(0)
clcol1R     =   Create(0.8)
clcol1G     =   Create(0.8)
clcol1B     =   Create(0.8)
clcol2R     =   Create(0.8)
clcol2G     =   Create(0.8)
clcol2B     =   Create(0.8)

#The Marble Texture
marbledepth = Create(3)
marbleselcol1 = Create(1)
marbleselcol2 = Create(0)
marblecol1R = Create(0.8)
marblecol1G = Create(0.8)
marblecol1B = Create(0.8)
marblecol2R = Create(0.1)
marblecol2G = Create(0.1)
marblecol2B = Create(0.1)
marblehard = Create(0)
marbleturbulence = Create(10)
marblesharpness = Create(1)



def retrieveUV():
    global uvsize, kcvalue, ksvalue, krvalue,\
        mixmode, mulmode, addmode, kcmode, ksmode, krmode,\
        khmode, ktmode, khvalue, ktvalue, knormvalue

    if uvlist.has_key(selectedname):
        uv=UVmodulator(selectedname)
        uv.load()
        uvsize.val = uv.size
        if uv.kc <0:
            kcmode.val=0
            kcvalue.val = 1
        else:
            kcmode.val=1
            kcvalue.val = uv.kc

        if uv.ks <0:
            ksmode.val=0
            ksvalue.val = 1
        else:
            ksmode.val=1
            ksvalue.val = uv.ks

        if uv.kr <0:
            krmode.val=0
            krvalue.val = 1
        else:
            krmode.val=1
            krvalue.val = uv.kr

        if uv.kh <0:
            khmode.val=0
            khvalue.val = 1
        else:
            khmode.val=1
            khvalue.val = uv.kh

        if uv.kt <0:
            ktmode.val=0
            ktvalue.val = 1
        else:
            ktmode.val=1
            ktvalue.val = uv.kt

        knormvalue.val = uv.knorm
        
        if uv.mode == "mix":
            mixmode.val=1
            mulmode.val=0
            addmode.val=0
        elif uv.mode == "mul":
            mixmode.val=0
            mulmode.val=1
            addmode.val=0
        else:
            mixmode.val=0
            mulmode.val=0
            addmode.val=1
        del uv
    else:
        pass

def reset_mod(m):
    global txname,oth_mixmode,oth_mulmode,oth_addmode,\
        oth_kcmode, oth_ksmode,oth_krmode,oth_kcvalue,\
        oth_ksvalue, oth_krvalue, oth_size, imgpath,\
        oth_sizex,oth_sizey,oth_sizez
    global cldepth, clcol1R, clcol1G, clcol1B ,\
            clcol2R, clcol2G, clcol2B 
    global marbledepth, marblecol1R, marblecol1G, marblecol1B, marblecol2R, marblecol2G, marblecol2B, marblehard,\
                    marbleturbulence, marblesharpness

    txname.val = "GIVE_ME_A_NAME" 
    oth_mixmode.val = 0
    oth_mulmode.val = 1
    oth_addmode.val = 0
    oth_kcmode.val  = 1
    oth_ksmode.val  = 0
    oth_krmode.val  = 0
    oth_kcvalue.val = 1
    oth_ksvalue.val = 1
    oth_krvalue.val = 1
    oth_khvalue.val = 1
    oth_ktvalue.val = 1
    oth_knormvalue.val = 0
    oth_size.val    = 1
    oth_sizex.val   = 1
    oth_sizey.val   = 1
    oth_sizez.val   = 1

    if m == "image":
        imgpath.val = ""

    elif m == "clouds":
        print "Resetting Clouds Mask"
        clcol1R.val =   0.8
        clcol1G.val =   0.8
        clcol1B.val =   0.8

        clcol2R.val =   0.8
        clcol2G.val =   0.8
        clcol2B.val =   0.8

    else:
        #marble
        print "Resetting Marble Mask"
        marbledepth.val = 3
        marblecol1R.val = 0.8
        marblecol1G.val = 0.8
        marblecol1B.val = 0.8
        marblecol2R.val = 0.1
        marblecol2G.val = 0.1
        marblecol2B.val = 0.1
        marblehard.val = 0
        marbleturbulence.val = 10
        marblesharpness.val = 1

def recall_mod():
    global  txname, oth_kcmode, oth_kcvalue, oth_ksmode,\
        oth_ksvalue, oth_krmode, oth_krvalue, oth_size,\
        oth_khmode, oth_ktmode, oth_khvalue, othktvalue, oth_knormvalue, \
        oth_sizex, oth_sizey, oth_sizez
    global imgpath
    global cldepth, clcol1R, clcol1G, clcol1B ,\
        clcol2R, clcol2G, clcol2B
    global marbledepth, marblecol1R, marblecol1G, marblecol1B, marblecol2R, marblecol2G, marblecol2B, marblehard,\
                    marbleturbulence, marblesharpness

    mask = mod_data[0].type
    txname.val  = mod_data[0].name
    if mod_data[0].kc == -1:
        oth_kcmode.val = 0
        oth_kcvalue.val = 1
    else:
        oth_kcmode.val = 1
        oth_kcvalue.val= mod_data[0].kc

    if mod_data[0].ks == -1:
        oth_ksmode.val = 0
        oth_ksvalue.val = 1
    else:
        oth_ksmode.val = 1
        oth_ksvalue.val = mod_data[0].ks

    if mod_data[0].kr == -1:
        oth_krmode.val = 0
        oth_krvalue.val= 1 
    else:
        oth_krmode.val = 1
        oth_krvalue.val = mod_data[0].kr
        
    if mod_data[0].kh == -1:
        oth_khmode.val = 0
        oth_khvalue.val= 1 
    else:
        oth_khmode.val = 1
        oth_khvalue.val = mod_data[0].kh
        
    if mod_data[0].kt == -1:
        oth_ktmode.val = 0
        oth_ktvalue.val= 1 
    else:
        oth_ktmode.val = 1
        oth_ktvalue.val = mod_data[0].kt

    oth_knormvalue.val = mod_data[0].knorm

    oth_size.val =  mod_data[0].size
    oth_sizex.val =  mod_data[0].sizex
    oth_sizey.val =  mod_data[0].sizey
    oth_sizex.val =  mod_data[0].sizez

    if mask == "image":
        
        print "recalling data image"
        imgpath = Create(mod_data[1].texpath)

    elif mask == "clouds":
        print "recalling data clouds"
        cldepth.val   = mod_data[1].depth
        clcol1R.val   = mod_data[1].color1.R
        clcol1G.val   = mod_data[1].color1.G
        clcol1B.val   = mod_data[1].color1.B
        clcol2R.val   = mod_data[1].color2.R
        clcol2G.val   = mod_data[1].color2.G
        clcol2B.val   = mod_data[1].color2.B

    else:
        #marble
        print "recalling data marble"
        marbledepth.val = mod_data[1].depth
        marblecol1R.val = mod_data[1].color1.R
        marblecol1G.val = mod_data[1].color1.G
        marblecol1B.val = mod_data[1].color1.B
        marblecol2R.val = mod_data[1].color2.R
        marblecol2G.val = mod_data[1].color2.G
        marblecol2B.val = mod_data[1].color2.B
        marblehard.val = mod_data[1].hard
        marbleturbulence.val = mod_data[1].turbulence
        marblesharpness.val = mod_data[1].sharpness
        


def delmod(num):
    global modulatorlist, modullist, texmenu
    modlis = string.split(texmenu, "|")
    m=Modulators(selectedname)
    m.load()
    if len(m.modulators) == 1:
        elimina(texdir, selectedname+"__Modulators")
        del modulatorlist[selectedname]
    else:
        for i in range(len(m.modulators)):
            if m.modulators[i] == modlis[num-1]:
                del m.modulators[i]
                break
        m.save()
    del m
    del modullist[modlis[num-1]]
    elimina(texdir,modlis[num-1]+"__Modul")
    elimina(texdir,modlis[num-1]+"__Texture")

def StoreUV():
    uvmod = UVmodulator(selectedname)
    uvmod.size = uvsize.val
    uvmod.sizex = sizex.val
    uvmod.sizey = sizey.val
    uvmod.sizez = sizez.val
    if mixmode.val:
        uvmod.mode="mix"
    elif mulmode.val:
        uvmod.mode="mul"
    else:
        uvmod.mode="add"

    if(kcmode.val):
        uvmod.kc = kcvalue.val
    else:
        uvmod.kc = -1

    if(ksmode.val):
        uvmod.ks = ksvalue.val
    else:
        uvmod.ks = -1

    if(krmode.val):
        uvmod.kr = krvalue.val
    else:
        uvmod.kr = -1

    if(khmode.val):
        uvmod.kh = khvalue.val
    else:
        uvmod.kh = -1

    if(ktmode.val):
        uvmod.kt = ktvalue.val
    else:
        uvmod.kt = -1

    uvmod.knorm = knormvalue.val

    uvmod.save()
    del uvmod

def Storetex():
    global modulatorlist, modullist
    if txmenu.val>4:
        type = mod_data[0].type
    elif txmenu.val == 2:
        type = "image"
    elif txmenu.val == 3:
        type = "clouds"
    else:
        type = "marble"
    print "Storing Texture: ", type, "\n"
    tname = txname.val
    mods = Modulators(selectedname)
    if modulatorlist.has_key(selectedname):
        mods.load()

    if (not(modullist.has_key(tname)) or (tname in mods.modulators)):
        #Ok, name is not used by another......
        if not (tname in mods.modulators):
            mods.modulators.append(tname)
        mods.save()
        del mods

        mod=Modulator(tname)
        mod.type=type
        if oth_mixmode.val:
            mod.mode="mix"
        elif oth_mulmode.val:
            mod.mode="mul"
        else:
            mod.mode="add"
        mod.size = oth_size.val
        mod.sizex = oth_sizex.val
        mod.sizey = oth_sizey.val
        mod.sizez = oth_sizez.val
        if(oth_kcmode.val):
            mod.kc = oth_kcvalue.val
        else:
            mod.kc = -1
            
        if(oth_ksmode.val):
            mod.ks = oth_ksvalue.val
        else:
            mod.ks = -1

        if(oth_krmode.val):
            mod.kr = oth_krvalue.val
        else:
            mod.kr = -1

        if(oth_khmode.val):
            mod.kh = oth_khvalue.val
        else:
            mod.kh = -1

        if(oth_ktmode.val):
            mod.kt = oth_ktvalue.val
        else:
            mod.kt = -1
        
        mod.knorm = oth_knormvalue.val
        
        #Redundant
        mod.texture = txname.val
        mod.save()
        del mod
        modullist[tname]=tname+"__Modul"

        if type == "image":
            imt=imageTexture(txname.val)
            imt.texpath=imgpath.val
            imt.save()
            del imt

        elif type == "clouds":
            clt=cloudsTexture(txname.val)
            clt.depth = cldepth.val
            col1=Color()
            col1.setColorf(clcol1R.val, clcol1G.val, clcol1B.val)
            clt.color1=col1
            col2=Color()
            col2.setColorf(clcol2R.val, clcol2G.val, clcol2B.val)
            clt.color2=col2
            clt.save()
            del clt, col1, col2

        elif type == "marble":
            mrblt = marbleTexture(txname.val)
            mrblt.depth = marbledepth.val
            col1 = Color()
            col1.setColorf(marblecol1R.val, marblecol1G.val, marblecol1B.val)
            mrblt.color1 = col1
            col2 = Color()
            col2.setColorf(marblecol2R.val, marblecol2G.val, marblecol2B.val)
            mrblt.color2 = col2
            mrblt.hard = marblehard.val
            mrblt.turbulence = marbleturbulence.val
            mrblt.sharpness = marblesharpness.val
            mrblt.save()
            del mrblt, col1, col2
        
        modulatorlist[selectedname] =selectedname+".modulators"
        return 1
    else:
        print ("DEBUG: nome in uso")
        #Ops!, name is already in use!!
        del mods
        return 0
        
#Texture related GUI
def texgui():

    global  kcvalue, ksvalue, krvalue, txmenu,\
        cldepth, clcol1R,clcol1G,\
        clcol1B , clcol2R, clcol2G, clcol2B, txname, imgpath,\
        marbledepth, marblecol1R, marblecol1G, marblecol1B, marblecol2R, marblecol2G, marblecol2B,\
        marblehard, marbleturbulence, marblesharpness,\
        mixmode, mulmode, addmode, kcmode, ksmode, krmode,\
        oth_mixmode, oth_mulmode, oth_addmode, oth_kcmode, oth_ksmode, oth_krmode,\
        oth_kcvalue, oth_ksvalue, oth_krvalue, oth_size,\
        selcol1, selcol2, marbleselcol1, marbleselcol2, uvsize, khmode, khvalue, ktmode, ktvalue, knormvalue, \
        oth_khmode, oth_khvalue, oth_ktmode, oth_ktvalue, oth_knormvalue, sizex, sizey, sizez,\
        oth_sizex, oth_sizey, oth_sizez

    #print selectedname

    glClearColor(0.4,0.48,0.57, 0.0) 
    glClear(GL_COLOR_BUFFER_BIT)

    glColor3f(1,1,1)

    glRasterPos2i(250,480)
    Text("UV Texture Settings")
    glRasterPos2i(250,465)
    Text("These values will affect")
    glRasterPos2i(250,450)
    Text("only the Blender Texture!")

    mixmode = Toggle("Mix", mixmodeevt,80, 440, 48, 18, mixmode.val, \
            "The texture blends the values or color")
    mulmode = Toggle("Mul", mulmodeevt,130, 440, 48, 18, mulmode.val, \
            "The texture multiplies the values or color" )
    addmode = Toggle("Add", addmodeevt,180, 440, 48, 18 , addmode.val, \
            "The texture adds the values or color")
    kcmode =  Toggle("Col", updateguievent , 40, 422, 38, 16, kcmode.val, \
            "Let the texture affect the basic colour of material")
    ksmode =  Toggle("Spec", updateguievent, 40, 404, 38, 16, ksmode.val, \
            "Let the texture affect the colour of specularity")
    krmode =  Toggle("Ref", updateguievent , 40, 386, 38, 16 , krmode.val, \
            "Let the texture affect the colour of reflectivity")
    khmode = Toggle("Hard", updateguievent, 40, 368, 38, 16, khmode.val, \
            "Let the texture affect the hardness")
    ktmode = Toggle("Trans", updateguievent, 40, 350, 38, 16, ktmode.val, \
            "Let the texture affect the transmission")

    kcvalue=  Slider("color: ", updateguievent, 80, 422, 150, 16, kcvalue.val,\
            0, 1, 0,"Set the amount the texture affects colour")
    ksvalue=  Slider("Spec: ", updateguievent,  80, 404, 150, 16, ksvalue.val,\
             0, 1, 0, "Set the amount the texture affects specular colour")
    krvalue=  Slider("Ref: ",updateguievent,  80, 386, 150, 16, krvalue.val,\
            0, 1, 0, "Set the amount the texture affects reflections colour")
    khvalue = Slider("Hard: ", updateguievent, 80, 368, 150, 16, khvalue.val,\
            0,1,0, "Set the amount the texture affect hardness")
    ktvalue = Slider("Trans: ", updateguievent, 80, 350, 150, 16, ktvalue.val,\
            0,1,0, "Set the amount the texture affect transmission")
    knormvalue = Number("Normal: ", updateguievent, 80, 332, 150, 16, knormvalue.val,\
            -5, 5, "Bump mapping: <0 rise dark zone , >0 rise bright zone")
    sizex   = Slider("SizeX: ", updateguievent, 250, 404, 150, 16, sizex.val,\
                 0, 1, 0, "Set the scale factor along X")
    sizey   = Slider("SizeY: ", updateguievent, 250, 386, 150, 16, sizey.val,\
                 0, 1, 0, "Set the scale factor along Y")
    sizez   = Slider("SizeZ: ", updateguievent, 250, 368, 150, 16, sizez.val,\
                 0, 1, 0,"Set the scale factor along Z")
    uvsize = Slider("Size: ",updateguievent, 250, 350, 150, 16, uvsize.val, 0.1, 100, 0,\
            "Set the scale factor (xyz)")


    #Add other textures
    glRasterPos2i(4,330)
    Text("Others:")
    txmenu = Menu(texmenu, texbrowserevt, 40, 300, 20, 20, txmenu.val)

    if txmenu.val == 2:
        #Add Image
        mask = "image"
        print "ADD IMAGE TEXTURE"

    if txmenu.val == 3:
        #Add CLouds
        mask = "clouds"
        print "ADD CLOUDS TEXTURE"

    if txmenu.val == 4:
        #Add Marble
        mask = "marble"
        print "ADD MARBLE TEXTURE"
    
    if txmenu.val > 4:
        mask = mod_data[0].type
        deltex = Button("Del", deltexevt, 180, 20, 50, 20,\
                "Remove this texture from shader")
    if not txmenu.val == 1:
        txname = String("Name: ", updateguievent, 62, 300, 158, 20, txname.val,160) 
        if name_in_use:
            glColor3f(1,0.1,0.1)
            glRasterPos2i(235,310)
            Text("ATTENTION!! Name in use, choose another one .....")
            glColor3f(1, 1, 1)

        oth_mixmode = Toggle("Mix", oth_mixmodeevt, 80, 270, 48, 18,oth_mixmode.val, \
            "The texture blends the values or color")
        oth_mulmode = Toggle("Mul", oth_mulmodeevt, 130, 270, 48, 18, oth_mulmode.val, \
            "The texture multiplies the values or color" )
        oth_addmode = Toggle("Add", oth_addmodeevt,180, 270, 48, 18 , oth_addmode.val, \
            "The texture adds the values or color")

        oth_kcmode =  Toggle("Col", updateguievent , 40, 252, 38, 16, oth_kcmode.val, \
            "Let the texture affect the basic colour of material")
        oth_ksmode =  Toggle("Spec", updateguievent, 40, 235, 38, 16, oth_ksmode.val, \
            "Let the texture affect the colour of specularity")
        oth_krmode =  Toggle("Ref", updateguievent , 40, 218, 38, 16 , oth_krmode.val, \
            "Let the texture affect the colour of reflectivity")
        oth_khmode = Toggle("Hard", updateguievent , 40, 201, 38, 16, oth_khmode.val, \
            "Let the texture affect the hardness")
        oth_ktmode = Toggle("Trans", updateguievent , 40, 184, 38, 16, oth_ktmode.val, \
            "Let the texture affect the transmission")

        oth_kcvalue=  Slider("color: ", updateguievent, 80, 252, 150, 16, oth_kcvalue.val,\
            0, 1, 0,"Set the amount the texture affects colour")
        oth_ksvalue=  Slider("Spec: ", updateguievent,  80, 235, 150, 16, oth_ksvalue.val,\
             0, 1, 0, "Set the amount the texture affects specular colour")
        oth_krvalue=  Slider("Ref: ",updateguievent,  80, 218, 150, 16, oth_krvalue.val,\
            0, 1, 0, "Set the amount the texture affects reflections colour")
        oth_khvalue=  Slider("Hard: ",updateguievent,  80, 201, 150, 16, oth_khvalue.val,\
            0, 1, 0, "Set the amount the texture affects hardness")
        oth_ktvalue=  Slider("Trans: ",updateguievent,  80, 184, 150, 16, oth_ktvalue.val,\
            0, 1, 0, "Set the amount the texture affects transmission colour")
        oth_knormvalue= Number("Normal: ", updateguievent, 80, 167, 150,16, oth_knormvalue.val, \
            -5, 5, "Bump mapping: <0 rise dark zone , >0 rise bright zone")
        oth_sizex   = Slider("SizeX: ", updateguievent, 250, 235, 150, 16, oth_sizex.val,\
                 0, 1, 0, "Set the scale factor along X")
        oth_sizey   = Slider("SizeY: ", updateguievent, 250, 218, 150, 16, oth_sizey.val,\
                 0, 1, 0, "Set the scale factor along Y")
        oth_sizez   = Slider("SizeZ: ", updateguievent, 250, 201, 150, 16, oth_sizez.val,\
                 0, 1, 0,"Set the scale factor along Z")
        oth_size = Slider("Size :", updateguievent,  250, 184, 150, 16, oth_size.val,\
            0.1, 100, 0, "Set the scale factor (xyz)")

        if mask == "image":
            print "Image GUI"
            imgpath = String("Name: ", updateguievent, 40, 130, 190, 20, imgpath.val, 50,\
            "Write ONLY the texture name WITH extension. Ex: wallbricks.jpeg or pippo.tga")
        elif mask == "clouds":
            print "Clouds GUI"
            cldepth = Slider("Depth", updateguievent, 80, 150, 150, 16, cldepth.val,\
                1, 20, 0,"Set the depth of the cloud calculation")

            #Blender-like color selector
            selcol1 = Toggle("Col1", selcol1evt, 231, 99, 40, 16, selcol1.val,\
                    "Select Color1 component")
            drawcolorbox(231, 149, 271,116, clcol1R.val, clcol1G.val, clcol1B.val)
            selcol2 = Toggle("Col2", selcol2evt, 271, 99, 40, 16, selcol2.val,\
                    "Select Color2 component")
            drawcolorbox(271, 149, 311, 116, clcol2R.val, clcol2G.val, clcol2B.val)
            if selcol1.val:
                clcol1R = Slider("R :", updateguievent, 80, 133, 150, 16, clcol1R.val,\
                        0, 1.0, 0)
                clcol1G = Slider("G :", updateguievent, 80, 116, 150, 16, clcol1G.val,\
                        0, 1.0, 0)
                clcol1B = Slider("B: ", updateguievent, 80, 99, 150, 16, clcol1B.val,\
                        0, 1.0, 0)
            else:
                clcol2R = Slider("R :", updateguievent, 80, 133, 150, 16, clcol2R.val,\
                        0, 1.0, 0)
                clcol2G = Slider("G :", updateguievent, 80, 116, 150, 16, clcol2G.val,\
                        0, 1.0, 0)
                clcol2B = Slider("B: ", updateguievent, 80, 99, 150, 16, clcol2B.val,\
                        0, 1.0, 0)

        else:
            print "Marble GUI"
            marbledepth = Slider("Depth", updateguievent, 80, 150, 150, 16, marbledepth.val,\
                1, 20, 0,"Set the nuber of noises frequencies")

            #Blender-like color selector
            marbleselcol1 = Toggle("Col1", marbleselcol1evt, 231, 99, 40, 16, marbleselcol1.val,\
                    "Select Color1 component")
            drawcolorbox(231, 149, 271,116, marblecol1R.val, marblecol1G.val, marblecol1B.val)
            marbleselcol2 = Toggle("Col2", marbleselcol2evt, 271, 99, 40, 16, marbleselcol2.val,\
                    "Select Color2 component")
            drawcolorbox(271, 149, 311, 116, marblecol2R.val, marblecol2G.val, marblecol2B.val)
            if marbleselcol1.val:
                marblecol1R = Slider("R :", updateguievent, 80, 133, 150, 16, marblecol1R.val,\
                        0, 1.0, 0)
                marblecol1G = Slider("G :", updateguievent, 80, 116, 150, 16, marblecol1G.val,\
                        0, 1.0, 0)
                marblecol1B = Slider("B: ", updateguievent, 80, 99, 150, 16, marblecol1B.val,\
                        0, 1.0, 0)
            else:
                marblecol2R = Slider("R :", updateguievent, 80, 133, 150, 16, marblecol2R.val,\
                        0, 1.0, 0)
                marblecol2G = Slider("G :", updateguievent, 80, 116, 150, 16, marblecol2G.val,\
                        0, 1.0, 0)
                marblecol2B = Slider("B: ", updateguievent, 80, 99, 150, 16, marblecol2B.val,\
                        0, 1.0, 0)
            marblehard = Toggle("Hard", updateguievent, 80, 82, 150, 16, marblehard.val,\
                    "Noise type : off = the noise varies smoothly")
            marbleturbulence = Number("Turbulence", updateguievent, 80, 65, 150, 16, marbleturbulence.val, 0, 100,\
                    "Amount of noise turbulence when mixing the two colors")
            marblesharpness = Number("Sharpness", updateguievent, 80, 48, 150, 16, marblesharpness.val, 1, 100,\
                    "Sharpness of color 1 compared to color 2, the higher this value, the thinner the color band of color1")
        
        
        
        oktex = Button("Ok!", oktexevt, 40, 20, 50, 20,\
                "Add this texture to the shader")
        canctex= Button("Canc", canctexevt, 92, 20, 50, 20,\
                "Discard Changes")

    Button("Back",  exitevent, 340, 30, 60, 30, "Back to Shader Settings")
#A flag 
name_in_use = 0 

def texevent(evt, val):
    if (evt == ESCKEY and not val): 
        Exit()

def texbevent(evt):
    global mod_data
    global txmenu, texmenu, name_in_use
    global mulmode, addmode, mixmode
    global addmode, mulmode
    global oth_mulmode, oth_addmode, oth_mixmode
    global selcol2, selcol1
    global marbleselcol2, marbleselcol1
        
    if (evt == exitevent):  
        StoreUV()
        Register (gui, event, bevent)

    if (evt == oktexevt):
        
        toutbien = Storetex()
        if toutbien:
            name_in_use = 0
            txmenu.val = 1
            texmenu = retrieve_modulators()
        else:
            print "Name in use, choose another....."
            name_in_use = 1

        Register (texgui, texevent, texbevent)

    if (evt == canctexevt):
        
        txmenu.val = 1
        Register (texgui, texevent, texbevent)

    if (evt == deltexevt):  
        
        delmod(txmenu.val)
        txmenu.val = 1
        texmenu = retrieve_modulators()
        Register (texgui, texevent, texbevent)

    if (evt == mixmodeevt):
        
        if mixmode.val:
            mulmode.val = 0
            addmode.val = 0
        else:
            mixmode.val = 1
        Register (texgui, texevent, texbevent)

    if (evt == mulmodeevt):
        
        if mulmode.val:
            mixmode.val = 0
            addmode.val = 0
        else:
            mulmode.val =  1
        Register (texgui, texevent, texbevent)

    if (evt == addmodeevt):
        
        if addmode.val:
            mixmode.val = 0
            mulmode.val = 0
        else:
            addmode.val = 1
        Register (texgui, texevent, texbevent)

    if (evt == oth_mixmodeevt):
        if oth_mixmode.val:
            oth_mulmode.val = 0
            oth_addmode.val = 0
        else:
            oth_mixmode.val = 1
        Register (texgui, texevent, texbevent)

    if (evt == oth_mulmodeevt):
        if oth_mulmode.val:
            oth_mixmode.val = 0
            oth_addmode.val = 0
        else:
            oth_mulmode.val = 1
        Register (texgui, texevent, texbevent)

    if (evt == oth_addmodeevt):

        if oth_addmode.val:
            oth_mixmode.val = 0
            oth_mulmode.val = 0
        else:
            oth_addmode.val = 1
        Register (texgui, texevent, texbevent)

    if (evt == selcol1evt):
        if not selcol2.val:
            selcol2.val = 0
            selcol1.val = 1
        else:
            selcol2.val = 0
        Register (texgui, texevent, texbevent)

    if (evt == selcol2evt):
        if not selcol1.val:
            selcol1.val = 0
            selcol2.val = 1
        else:
            selcol1.val = 0
        Register (texgui, texevent, texbevent)

    if (evt == marbleselcol1evt):
        if not marbleselcol2.val:
            marbleselcol2.val = 0
            marbleselcol1.val = 1
        else:
            marbleselcol2.val = 0
        Register (texgui, texevent, texbevent)

    if (evt == marbleselcol2evt):
        if not marbleselcol1.val:
            marbleselcol1.val = 0
            marbleselcol2.val = 1
        else:
            marbleselcol1.val = 0
        Register (texgui, texevent, texbevent)


    if (evt == texbrowserevt):

        if txmenu.val > 4:
            mod_data = retrieve_mod_data(txmenu.val)
            recall_mod()

        elif txmenu.val == 2:
            reset_mod("image")

        elif txmenu.val == 3:
            reset_mod("clouds")

        elif txmenu.val == 4:
            reset_mod("marble")

        Register (texgui, texevent, texbevent)

    if (evt == updateguievent):
        Register (texgui, texevent, texbevent)


##################################################
#LAYER MASK GUI                                                                         #
##################################################

Layersw=[]
LAYER_ERR = 0

#Default  Init: All layers.
for i in range(20):
    Layersw.append(Create(1))

def getlayers():
    global Layersw
    for i in range(20):
        bit = (1<<i)
        if (bit & bmask):
            Layersw[i] =Create(1)
        else:
            Layersw[i] = Create(0)

def dobitmask():
    bm = 0
    for i in range(len(Layersw)):

        if Layersw[i].val:
            
            bm |= (1<<i)
    return bm

def layergui():
    global bmask, Layersw
    print "LayerGUI"

    glClearColor(0.4,0.48,0.57, 0.0)
    glClear(GL_COLOR_BUFFER_BIT)
    glColor3f(1,1,1)

    wd = 35
    ht = 20
    #starting coord
    xs = 65 
    ys = 320
    for i in range(2): #Two rows
        for j in range(10): #Ten columns

            laynum = (j+1)+(i*10)
            st = "%2d"%laynum
            Layersw[laynum-1] = Toggle(st, updateguievent, xs+(j*(wd+2)), ys-(i*(ht)), wd, ht, Layersw[laynum-1].val)

    # Message when trying to return to settings screen with all layer buttons off
    if LAYER_ERR:
        glColor3f(0, 0, 0)
        glRecti(10,59 ,380 ,39 )
        glColor3f(1, 0.5, 0)
        glRasterPos2i(15, 45)
        Text("YOU HAVE TO ENABLE AT LEAST ONE LAYER !")

    # All buttons off
    Button("All Off", alloffevt, 90,250,150 , ht, "Set all Layer buttons to off")   
    # All buttons on
    Button("All On", allonevt, 260, 250,150 , ht, "Set all Layer buttons to on")
    # return to settings screen Layersw
    Button("Back", okbackevt, 170, 200, 150,20, "Back to settings screen")

def layerbevent(evt):
    global LAYER_ERR, bmask, Layersw
    if (evt== okbackevt):
        bmask=dobitmask()
        print "layerbevent ",bmask
        if not bmask:
            LAYER_ERR = 1
            Blender.Redraw()
            return
        LAYER_ERR = 0
        Register(gui, event, bevent)

    if (evt == updateguievent):
        Register(layergui, layerevent, layerbevent)

    if evt==allonevt:
        #global Layersw
        for i in range(20):
            Layersw[i].val = 1
        Register(layergui, layerevent, layerbevent)
    if evt==alloffevt:
        #global Layersw
        for i in range(20):
            Layersw[i].val = 0
        Register(layergui, layerevent, layerbevent)

def layerevent(evt, val):
    if (evt == ESCKEY and not val): 
        Exit()


#CHECK FOR THE EXISTENCE OF A CONFIG FILE
#IF FOUND START THE YABLE GUI. 
#IF NOT A PRELIMINARY  GUI IS SHOWN
if startup():
    print "startGUI"
    Register (startgui, startevent, startbevent)
else:
    print "WorkGUI"
    eatsettings()
    Register (gui, event, bevent)



def export():
    print "Clicked Export"
    filename = exporttoyafray()
    return filename 


def assigntomesh():

    global meshmat, meshattr
    meshmat[selectedname] =selectedname+".mat" 
    meshattr[selectedname] = selectedname+".objattr"
    saveoptparams(matdir,selectedname)
    writematdesc(matdir,selectedname)

def assigncrafttomesh():

    global meshmat, meshattr
    meshmat[selectedname] =selectedname+".mat" 
    #meshattr[selectedname] = selectedname+".objattr"
    #Write out crafter code here
    fd = open(CRAFTERFILE)
    crafter = fd.read()
    fd.close
    crafter = string.replace(crafter,"preview.mat",meshmat[selectedname])
    attrfilename = os.path.join(matdir, meshmat[selectedname] )
    attrfile = open(attrfilename,'w')   
    attrfile.write("Crafter\n")
    #AutoSmooth Angle
    attrfile.write("AutoSmooth : %f\n"%AutoSmooth.val)
    if AutoSmooth.val:
        attrfile.write("Degr : %f\n"%Degr.val)
    attrfile.write(crafter)
    attrfile.close()    

def saveoptparams(dir,name):

    if additional.val:
        mname = name + ".objattr"
        meshattr[name] = mname

        attrfilename = os.path.join(dir, mname)
        
        attrfile = open(attrfilename,'w')
        attrfile.write("%f\n"%addCausIOR.val)
        if addCausIOR.val:
            attrfile.write("%f\n"%CausIOR.val)
        attrfile.write("%f\n"%recrad.val)
        attrfile.write("%f\n"%emirad.val)
        attrfile.write("%f\n"%addCausTCol.val)
        if addCausTCol.val:
            attrfile.write("%f\n"%CausTR.val)
            attrfile.write("%f\n"%CausTG.val)
            attrfile.write("%f\n"%CausTB.val)

        attrfile.write("%f\n"%addCausRCol.val)
        if addCausRCol.val:
            attrfile.write("%f\n"%CausRR.val)
            attrfile.write("%f\n"%CausRG.val)
            attrfile.write("%f\n"%CausRB.val)
        if castsh.val:
            attrfile.write("on\n")
        else:
            attrfile.write("off\n")

        attrfile.close()
    else:
        #Devo rumuovere il file .objattr dalla directory Materials
        #e dal dizionario
        if meshattr.has_key(name):
            del meshattr[name]
            optattr = scanlocal(matdir, ".objattr")
            if optattr.has_key(name):
                print("saveoptparams-->elimina in Materials")
                elimina(matdir,name+".objattr")
            optattr2 = scanlocal(archdir, ".objattr")
            if optattr2.has_key(name):
                elimina(archdir,name+".objattr")

PI = pi
PI_2 = pi/2

# Arcotangente a quattro quadranti
def Atan(y, x):
    if x==0:
        if y==0:
            return 0
        elif y>0:
            return PI_2
        else:
            return -PI_2
    elif y == 0:
        if x>0:
            return 0
        else:
            return PI
    else:
        ret = atan2(y,x)
        return ret

#Build a Quad which normal is parallel to the direction
#given by his position and the topos variable
#E: This function could be very much simplified, but as it works, no need to change anything really,
#   except for a slight offset to make the plane positioned behind the arealight, could lead to shading errors otherwise.
def setQuad():

    global corners, topos, verso
    corner = [[0,0,0], [0,0,0],[0,0,0],[0,0,0]]
    quadObj = Blender.Object.Get(selectedname)
    centre = quadObj.loc
    ToDir = [0,0,0]
    hlen = alLato.val
    hlen /= 2

    #Quad centered in the Origin , parallel to XY plane.
    # Widht & Height given by hlen
    corner[0] = [-hlen, -hlen, 0]
    corner[1] = [-hlen, hlen, 0]
    corner[2] = [hlen, hlen, 0]
    corner[3] = [hlen, -hlen, 0]
    #The To Direction
    ToDir[0] = -centre[0] + topos[0]
    ToDir[1] = -centre[1] + topos[1]
    ToDir[2] = -centre[2] + topos[2]

    ToDir_n=normalize(ToDir)

    if not (ToDir_n[0] == 0 and ToDir_n[1] == 0 and ToDir_n[2] == 0):


        if  (abs(ToDir_n[0]) < 0.00001):
                #Componente su X  nulla-> ruoto solo intorno x
                yzang = Atan(ToDir_n[2], ToDir_n[1])- PI_2

                Tm = [[1, 0, 0], [0, cos(yzang), -sin(yzang)], [0, sin(yzang), cos(yzang)]]

        elif    (abs(ToDir_n[1]) < 0.00001):
                #Componente suY nulla-> ruoto solo intorno y
                xzang = - Atan(ToDir_n[2], ToDir_n[0]) + PI_2

                Tm = [[cos(xzang), 0, sin(xzang)], [0, 1, 0], [-sin(xzang), 0, cos(xzang)]]

        elif    (abs(ToDir_n[2]) < 0.00001):
                #Componente su Z nulla-> ruoto solo intorno z
                xyang = Atan(ToDir_n[1],ToDir_n[0])- PI_2
                Tm = [[cos(xyang), -sin(xyang), 0], [sin(xyang), cos(xyang), 0], [0, 0, 1]]
        else:

            yzang = Atan(ToDir_n[2], ToDir_n[1])
            xyang = Atan(ToDir_n[1],ToDir_n[0])
            if yzang > abs(PI_2):
                xyang = xyang + PI_2
            else:
                if xyang>0:

                    xyang = -PI_2 + xyang  
                else:

                    xyang = xyang + PI_2
            yzang = yzang-PI_2

            Tx = [[1, 0, 0], [0, cos(yzang), -sin(yzang)], [0, sin(yzang), cos(yzang)]]
            Tz = [[cos(xyang), -sin(xyang), 0], [sin(xyang), cos(xyang), 0], [0, 0, 1]]
            Tm = multiply3x3(Tz,Tx)

        for i in range(4):
            corners[i] = multiply3x1(Tm, corner[i])

    quadnorm = calcnorm(corners)
    verso = scalar(quadnorm, ToDir_n)

    #E: center offset to put the areaquad behind the actual arealight to prevent shading errors
    for i in range(4):
        for j in range(3):
            corners[i][j] += centre[j] - ToDir_n[j]*0.1

    cookAQuad(corners, "DELETEME")


def cookAQuad(corners, quadname):

    me = NMesh.GetRaw()

    for i in range(4):
        v=NMesh.Vert(corners[i][0],corners[i][1],corners[i][2]) 
        me.verts.append(v)


    f=NMesh.Face()
    f.v.append(me.verts[0]) 
    f.v.append(me.verts[1]) 
    f.v.append(me.verts[2]) 
    f.v.append(me.verts[3]) 

    me.faces.append(f)
    NMesh.PutRaw(me, quadname, 1)

    Blender.Redraw()



def assigntolight():
    global lightlist
    if lightshader.val == 4:
        setQuad()

    #if lightshader.val == 6:
    #   writephlightdesc(matdir,selectedname)
    #else:
    writelightdesc(matdir,selectedname)


#def assigntophlight():
#   global lightlist
#   writephlightdesc(matdir,selectedname)


def savemat():
    saveoptparams(archdir,shname.val)
    if not additional.val:
        optattr2 = scanlocal(archdir, ".objattr")
        if optattr2.has_key(shname.val):
            print("savemat-->elimina in Archivio")
            elimina(archdir,shname.val+".objattr")
    writematdesc(archdir,shname.val)

def resetshaderval():
    global GenR, GenG, GenB, GenSpecR, GenSpecG, GenSpecB, GenRefR, GenRefG, GenRefB, GenRef2R, GenRef2G, GenRef2B, \
         GenTraR, GenTraG, GenTraB, GenTra2R, GenTra2G, GenTra2B
    global  GenHard, GenIOR, GenMinRef, GenFastFresnel, ConstR, ConstG, ConstB, addSpec, addIOR, addRef, addRef2, addTra, addTra2
    global  AutoSmooth, Degr
    global  additional, addCausIOR, recrad, emirad, addCausTCol,CausIOR, CausTR, CausTG,CausTB
    global  addCausRCol, CausRR, CausRG,CausRB
    global  phlprv

    #print "ReseT Val"
    GenR.val= 0.8
    GenG.val= 0.8
    GenB.val= 0.8
    GenSpecR.val= 1.0
    GenSpecG.val= 1.0
    GenSpecB.val= 1.0   
    GenRefR.val= 0.0
    GenRefG.val= 0.0
    GenRefB.val= 0.0
    GenRef2R.val= 0.0
    GenRef2G.val= 0.0
    GenRef2B.val= 0.0
    GenTraR.val= 0.0
    GenTraG.val= 0.0
    GenTraB.val= 0.0
    GenTra2R.val= 0.0
    GenTra2G.val= 0.0
    GenTra2B.val= 0.0
    GenHard.val= 25
    GenIOR.val= 1.00
    GenMinRef.val= 0.2
    GenFastFresnel.val = 0

    ConstR.val= 0.80
    ConstG.val= 0.80
    ConstB.val= 0.80
    addIOR.val= 0
    addSpec.val= 0
    addRef.val= 0
    addRef2.val= 0
    addTra.val= 0
    addTra2.val= 0
    AutoSmooth.val=0
    Degr.val=30

    additional.val= 0
    addCausIOR.val= 0
    recrad.val= 0
    emirad.val= 0
    addCausTCol.val= 0
    CausIOR.val= 1.5
    CausTR.val= 1.0
    CausTG.val= 1.0
    CausTB.val= 1.0
    CausRR.val= 1.0
    CausRG.val= 1.0
    CausRB.val= 1.0
    addCausRCol.val= 0
    
    phlprv.val = 0

def resetlightval():
    global  lightshader
    global  plpow, plshad, plR, plG, plB, slpow, slshad, slR, slG, \
        slB, sflR, sflG, sflB, sflpow, sflres, sflbias, sflrad
    global  addphldiff, addphlcaus, \
                    phlDpow, phlDR, phlDG, phlDB, phlDdep, phlDsearch, phlDmode, phlDang,  phlDfix, phlDph, phlDcluster, phlDQMC, \
                    phlCpow, phlCR, phlCG, phlCB, phlCdep, phlCsearch, phlCmode, phlCang,  phlCfix, phlCph, phlCcluster, phlCQMC
    global  alpow, alR, alG, alB, alSamp, alPSamp, alLato
    global  splpow, splR, splG, splB, splsize, splbl, splbeoff, splshadow, \
                    splhaloADD, splhalofogR, splhalofogG, splhalofogB, splhalofogdensity, splhaloblur, splhalores, splhalosamples

    lightshader.val = 1
    #Point Light
    plpow.val   =   1.0
    plshad.val  =   0
    plR.val     =   1.00
    plG.val     =   1.00
    plB.val     =   1.00

    #Soft Light
    slpow.val   =   1.0
    slshad.val  =   0
    slR.val     =   1.00
    slG.val     =   1.00
    slB.val     =   1.00
    sflR.val    =   1.00
    sflG.val    =   1.00
    sflB.val    =   1.00
    sflpow.val  =   1.0
    sflres.val  =   200
    sflbias.val =   0.3
    sflrad.val  =   5

    #Photon Light
    addphldiff.val = 0
    addphlcaus.val = 0
    phlDpow.val =   100.0
    phlDR.val   =   1.0
    phlDG.val   =   1.0
    phlDB.val   =   1.0
    phlDdep.val =   3
    phlDsearch.val  =   200
    phlDang.val =   15.0
    phlDcluster.val =   0.5
    phlDfix.val =   1.0
    phlDph.val  =   50000
    phlDQMC.val =   0

    phlCpow.val =   100.0
    phlCR.val   =   1.0
    phlCG.val   =   1.0
    phlCB.val   =   1.0
    phlCdep.val =   3
    phlCsearch.val  =   100
    phlCang.val =   15.0
    phlCcluster.val =   0.01
    phlCfix.val =   0.1
    phlCph.val  =   50000
    phlCQMC.val =   0
    
    
    #Spot
    splpow.val      =   100
    splR.val    =   1.0
    splG.val    =   1.0
    splB.val    =   1.0
    splsize.val     =   45
    splbl.val   =   20
    splbeoff.val    =   2
    splshadow.val = 0
    splhaloADD.val = 0
    splhalofogR.val = 0.8
    splhalofogG.val = 0.8
    splhalofogB.val = 0.8
    splhalofogdensity.val = 0.2
    splhaloblur.val = 0.0
    splhalores.val = 64
    splhalosamples.val = 64

    #Area Light
    alpow.val   =   25
    alR.val     =   1.00
    alG.val     =   1.00
    alB.val     =   1.00
    alSamp.val  =   64
    alPSamp.val =   64
    alLato.val  =   5.0

def recalldata():
    global surfshader, additional

    if (selectedtype == 'Type: Mesh'):
        resetshaderval()
        if (meshmat.has_key(selectedname)):
            if not (meshmat[selectedname] == "Default"):
                matf = os.path.join(matdir,meshmat[selectedname])
                loadmat(matf)
                optattr = scanlocal(matdir, ".objattr")
                if optattr.has_key(selectedname):
                    optf = os.path.join(matdir,optattr[selectedname])
                    print("recalldata-->loadattr")
                    loadattr(optf)
                else:
                    additional.val = 0
            else:
                surfshader.val = 3
        else:

            surfshader.val = 3

            print "No Assigned Shaders for:  ", selectedname
    elif (selectedtype == 'Type: Lamp'):
        resetlightval()
        if (lightlist.has_key(selectedname)):
            luzf = os.path.join(matdir,lightlist[selectedname])
            loadlight(luzf)

def retrieve_colors(obname):
#retrieve color info from blender
#if any is given a medium grey is returtned.
    obcolor = [[0.8,0.8,0.8], [1.0,1.0,1.0],[1.0,1.0,1.0]]


    ob = Blender.Object.Get(obname)
    if BL_VERSION<=223:
        dt = ob.data
    else:
        dt = ob.getData()

## jms : fix for Blender 2.28   5/n --  start                     
    if dt:
        if BL_VERSION<228:
            if dt.block_type=='NMesh':
                mt = None
                if BL_VERSION<=223:
                    me = ob.data
                    materials = me.mats
                    if len(materials) and (materials[0]!=None):
                        mt = Blender.Material.Get(materials[0])
                else:
                    me = ob.getData()
                    materials = me.getMaterials()
                    if len(materials)and (materials[0]!=None):
                        mt = materials[0]
                if mt: 
                    obcolor[0]=[mt.R, mt.G, mt.B]
                    
                    if BL_VERSION<=223:
                        obcolor[1]=[mt.SpecR, mt.SpecG, mt.SpecB]
                        obcolor[2]=[mt.MirR, mt.MirG, mt.MirB]
                    else:
                        obcolor[1]=[mt.specR, mt.specG, mt.specB]
                        obcolor[1]=[mt.mirR, mt.mirG, mt.mirB]
        else:                
            if ob.getType()=='Mesh':
                mt = None
                materials = dt.materials
                if len(materials)and (materials[0]!=None):
                        mt = materials[0]
                if mt: 
                    obcolor[0]=[mt.R, mt.G, mt.B]
                    obcolor[1]=[mt.specCol[0], mt.specCol[1], mt.specCol[2]]
                    obcolor[1]=[mt.mirCol[0], mt.mirCol[1], mt.mirCol[2]]
            return obcolor
    else:
        return None
## jms : fix for Blender 2.28   5/n --  end                         

    """    
    if dt:
        if dt.block_type=='NMesh':

            mt = None
            if BL_VERSION<=223:
                me = ob.data
                materials = me.mats
                if len(materials) and (materials[0]!=None):
                    mt = Blender.Material.Get(materials[0])
            else:
                me = ob.getData()
                materials = me.getMaterials()
                if len(materials)and (materials[0]!=None):
                    mt = materials[0]
            if mt: 

                obcolor[0]=[mt.R, mt.G, mt.B]
                
                if BL_VERSION<=223:

                    obcolor[1]=[mt.SpecR, mt.SpecG, mt.SpecB]
                    obcolor[2]=[mt.MirR, mt.MirG, mt.MirB]
                else:

                    obcolor[1]=[mt.specR, mt.specG, mt.specB]
                    obcolor[1]=[mt.mirR, mt.mirG, mt.mirB]

        return obcolor
    else:
        return None
    """
    
def retrievelampdata(name):
    lmpret = [[1.0, 1.0, 1.0], 50, 2]
    lmp = Blender.Object.Get(name)
    if BL_VERSION<=223:
        dt = lmp.data
    else:
        dt = lmp.getData()
    if dt:
        if BL_VERSION<=223:
            tipo = "foo"
        else:
            tipo = dt.getType()
        """
        if tipo == "Spot":
        """
        
## jms : fix for Blender 2.28   4/n --  start                     
        if tipo == "2":
            lmpret[0][0]=dt.R
            lmpret[0][1]=dt.G
            lmpret[0][2]=dt.B
            lmpret[1] = dt.spotSize
            lmpret[2] = dt.spotBlend
## jms : fix for Blender 2.28   4/n --  start
            
    return lmpret

def getsunpos():
    if not sunname == "NO_SUN_SEL":

        sunob = Blender.Object.Get(sunname)
        return sunob.loc
    else:
        #a value must be returned
        return [20,20,20]

def loadmat(matf):
    global  surfshader
    global  GenR, GenG, GenB, GenSpecR, GenSpecG, GenSpecB, GenRefR, GenRefG, GenRefB, GenRef2R, GenRef2G, GenRef2B, \
         GenTraR, GenTraG, GenTraB, GenTra2R, GenTra2G, GenTra2B
    global  GenHard, GenIOR, GenMinRef, GenFastFresnel, ConstR, ConstG, ConstB, addIOR, addSpec, addRef, addRef2, addTra, addTra2
    global  AutoSmooth, Degr
    global  ConstR, ConstG, ConstB

    fn = open(matf,'r')
    mline = readstr(fn)
    #mline = fn.readline()
    print "shader type ", mline
    wasCrafter = 0
    if (mline == "Generic"):
        surfshader.val=1
        GenR.val = float(string.split(fn.readline(),':')[1])
        GenG.val = float(string.split(fn.readline(),':')[1])
        GenB.val = float(string.split(fn.readline(),':')[1])

        addSpec.val = float(string.split(fn.readline(),':')[1])
        if addSpec.val:
            GenSpecR.val     = float(string.split(fn.readline(),':')[1])
            GenSpecG.val     = float(string.split(fn.readline(),':')[1])
            GenSpecB.val     = float(string.split(fn.readline(),':')[1])

        addRef.val = float(string.split(fn.readline(),':')[1])
        if addRef.val:
            GenRefR.val =   float(string.split(fn.readline(),':')[1])
            GenRefG.val = float(string.split(fn.readline(),':')[1])
            GenRefB.val =   float(string.split(fn.readline(),':')[1])
            addRef2.val = float(string.split(fn.readline(),':')[1])
            if addRef2.val:
                GenRef2R.val =  float(string.split(fn.readline(),':')[1])
                GenRef2G.val =  float(string.split(fn.readline(),':')[1])
                GenRef2B.val =  float(string.split(fn.readline(),':')[1])

        addTra.val = float(string.split(fn.readline(),':')[1])
        if addTra.val:
            GenTraR.val =   float(string.split(fn.readline(),':')[1])
            GenTraG.val =   float(string.split(fn.readline(),':')[1])
            GenTraB.val =   float(string.split(fn.readline(),':')[1])
            addTra2.val =   float(string.split(fn.readline(),':')[1])
            if addTra2.val:
                GenTra2R.val=   float(string.split(fn.readline(),':')[1])
                GenTra2G.val=   float(string.split(fn.readline(),':')[1])
                GenTra2B.val=   float(string.split(fn.readline(),':')[1])

        GenHard.val = float(string.split(fn.readline(),':')[1])
        
        addIOR.val = float(string.split(fn.readline(),':')[1])
        if addIOR.val:
            GenIOR.val = float(string.split(fn.readline(),':')[1])

        GenMinRef.val = float(string.split(fn.readline(),':')[1])
        GenFastFresnel.val = float(string.split(fn.readline(),':')[1])
    elif (mline == "Crafter"):
        surfshader.val = 4
        #wasCrafter = 1

    else:
        #Constant
        surfshader.val = 2
        r = float(fn.readline())
        g = float(fn.readline())
        b = float(fn.readline())
        ConstR.val = r
        ConstG.val = g
        ConstB.val =b
    if wasCrafter == 0:
        autosm = float(string.split(fn.readline(),':')[1])
        if autosm:
            Degr.val = float(string.split(fn.readline(),':')[1])
        AutoSmooth.val = autosm
    fn.close()

def loadattr(opt):
    global  additional, addCausIOR, recrad, emirad, addCausTCol,CausIOR, CausTR, CausTG, CausTB
    global  addCausRCol, CausRR, CausRG,CausRB, castsh
    print("loadattr")
    fn = open(opt,'r')
    additional.val = 1
    addCausIOR.val = float (fn.readline())
    if addCausIOR.val:
        CausIOR.val= float (fn.readline())
    recrad.val =  float (fn.readline())
    emirad.val =  float (fn.readline())
    addCausTCol.val =  float (fn.readline())
    if addCausTCol.val:
        CausTR.val =  float (fn.readline())
        CausTG.val =  float (fn.readline())
        CausTB.val =  float (fn.readline())

    addCausRCol.val =  float (fn.readline())
    if addCausRCol.val:
        CausRR.val =  float (fn.readline())
        CausRG.val =  float (fn.readline())
        CausRB.val =  float (fn.readline())
    yesno=readstr(fn)

    if yesno == "off":
        castsh.val = 0
    else:
        castsh.val = 1

    fn.close()

def loadopt(infile):

        optparams = {}
        infilepath = os.path.join(matdir, infile)
        attr=open(infilepath,'r')

        if float(attr.readline()):
            optparams["CausIOR"] = float(attr.readline())

        if float(attr.readline()):
            optparams["recrad"] = "on"
        else: 
            optparams["recrad"] = "off"

        if float(attr.readline()):
            optparams["emirad"] = "on"
        else: 
            optparams["emirad"] = "off"

        if float(attr.readline()):
            optparams["caus_tcolor"] = 1
            optparams["tR"] = float(attr.readline())
            optparams["tG"] = float(attr.readline()) 
            optparams["tB"] = float(attr.readline())
        else :
            optparams["caus_tcolor"] = 0

        if float(attr.readline()):
            optparams["caus_rcolor"] = 1
            optparams["rR"] = float(attr.readline())
            optparams["rG"] = float(attr.readline())
            optparams["rB"] = float(attr.readline())
        else :
            optparams["caus_rcolor"] = 0

        optparams["cast"] = readstr(attr)
        print "Cast Shadow", optparams["cast"]
        attr.close()
        return optparams

def loadlight(luzf):

    global  lightshader, dirlightshader
    global  plpow, plshad, plR, plG, plB, slpow, slshad, \
        slR, slG, slB,sflR, sflG, sflB, sflpow, sflres, sflbias, sflrad
    global  addphldiff, addphlcaus, \
                    phlDpow, phlDR, phlDG, phlDB, phlDdep, phlDsearch, phlDang, phlDfix, phlDph, phlDcluster, phlDQMC, \
                    phlCpow, phlCR, phlCG, phlCB, phlCdep, phlCsearch, phlCang, phlCfix, phlCph, phlCcluster, phlCQMC
    global  topos, pos, toposphlD, toposphlC
    global  splpow, splR, splG, splB, splsize, splbl, splbeoff, splshadow, \
                    splhaloADD, splhalofogR, splhalofogG, splhalofogB, splhalofogdensity, splhaloblur, splhalores, splhalosamples
    global  alR, alG, alB, alpow, alSamp, alPSamp, alLato, corners
    
    infile = open(luzf,'r')

    lline = readstr(infile)
    
    if (lline == "AreaLight"):
        #AreaLight
        lightshader.val = 4
        alR.val    =    float(string.split(infile.readline(), ':')[1])
        alG.val    =    float(string.split(infile.readline(), ':')[1])
        alB.val    =    float(string.split(infile.readline(), ':')[1])
        alpow.val  =    float(string.split(infile.readline(), ':')[1])
        alSamp.val =    float(string.split(infile.readline(), ':')[1])
        alPSamp.val = float(string.split(infile.readline(), ':')[1])
        alLato.val =    float(string.split(infile.readline(), ':')[1])
        for i in range(4):
            for j in range(3):
                corners[i][j] = float(string.split(infile.readline(), ':')[1])

    else:
        if (lline == "PointLight"):
            lightshader.val = 1
            plR.val = float(string.split(infile.readline(), ':')[1])
            plG.val = float(string.split(infile.readline(), ':')[1])
            plB.val = float(string.split(infile.readline(), ':')[1])
            plpow.val = float(string.split(infile.readline(), ':')[1])
            plshad.val = float(string.split(infile.readline(), ':')[1])     

        elif (lline == "SpotLight"):
            lightshader.val = 5
            splR.val = float(string.split(infile.readline(), ':')[1])
            splG.val = float(string.split(infile.readline(), ':')[1])
            splB.val = float(string.split(infile.readline(), ':')[1])
            splpow.val = float(string.split(infile.readline(), ':')[1])
            splsize.val = float(string.split(infile.readline(), ':')[1])
            splbl.val = float(string.split(infile.readline(), ':')[1])
            splbeoff.val = float(string.split(infile.readline(), ':')[1])
            splshadow.val = float(string.split(infile.readline(), ':')[1])
            splhaloADD.val = float(string.split(infile.readline(), ':')[1])
            if splhaloADD.val:
                splhalofogR.val = float(string.split(infile.readline(), ':')[1])
                splhalofogG.val = float(string.split(infile.readline(), ':')[1])
                splhalofogB.val = float(string.split(infile.readline(), ':')[1])
                splhalofogdensity.val = float(string.split(infile.readline(), ':')[1])
                splhaloblur.val = float(string.split(infile.readline(), ':')[1])
                splhalores.val = float(string.split(infile.readline(), ':')[1])
                splhalosamples.val = float(string.split(infile.readline(), ':')[1])
            topos[0] = float(string.split(infile.readline(), ':')[1])
            topos[1] = float(string.split(infile.readline(), ':')[1])
            topos[2] = float(string.split(infile.readline(), ':')[1])

        elif (lline == "SunLight"):
            lightshader.val= 2
            slR.val   =  float(string.split(infile.readline(), ':')[1])
            slG.val   =  float(string.split(infile.readline(), ':')[1])
            slB.val   =  float(string.split(infile.readline(), ':')[1])
            slpow.val =  float(string.split(infile.readline(), ':')[1])
            slshad.val=  float(string.split(infile.readline(), ':')[1])

        elif (lline == "SoftLight"):
            lightshader.val= 3
            sflR.val    = float(string.split(infile.readline(), ':')[1])
            sflG.val    = float(string.split(infile.readline(), ':')[1])
            sflB.val    = float(string.split(infile.readline(), ':')[1])
            sflpow.val  = float(string.split(infile.readline(), ':')[1])
            sflres.val  = float(string.split(infile.readline(), ':')[1])
            sflrad.val  = float(string.split(infile.readline(), ':')[1])
            sflbias.val = float(string.split(infile.readline(), ':')[1])


        pos[0] = float(string.split(infile.readline(), ':')[1])
        pos[1] = float(string.split(infile.readline(), ':')[1])
        pos[2] = float(string.split(infile.readline(), ':')[1])


    addphldiff.val = float(string.split(infile.readline(), ':')[1])
    if  addphldiff.val:
        phlDpow.val = float(string.split(infile.readline(), ':')[1])
        phlDR.val = float(string.split(infile.readline(), ':')[1])
        phlDG.val = float(string.split(infile.readline(), ':')[1])
        phlDB.val = float(string.split(infile.readline(), ':')[1])
        phlDph.val = float(string.split(infile.readline(), ':')[1])
        phlDdep.val = float(string.split(infile.readline(), ':')[1])
        phlDsearch.val = float(string.split(infile.readline(), ':')[1])
        phlDang.val = float(string.split(infile.readline(), ':')[1])
        phlDfix.val = float(string.split(infile.readline(), ':')[1])
        phlDcluster.val = float(string.split(infile.readline(), ':')[1])
        phlDQMC.val = float(string.split(infile.readline(), ':')[1])
        toposphlD[0] = float(string.split(infile.readline(), ':')[1])
        toposphlD[1] = float(string.split(infile.readline(), ':')[1])
        toposphlD[2] = float(string.split(infile.readline(), ':')[1])

    addphlcaus.val = float(string.split(infile.readline(), ':')[1])
    if  addphlcaus.val:
        phlCpow.val = float(string.split(infile.readline(), ':')[1])
        phlCR.val = float(string.split(infile.readline(), ':')[1])
        phlCG.val = float(string.split(infile.readline(), ':')[1])
        phlCB.val = float(string.split(infile.readline(), ':')[1])      
        phlCph.val = float(string.split(infile.readline(), ':')[1])
        phlCdep.val = float(string.split(infile.readline(), ':')[1])
        phlCsearch.val = float(string.split(infile.readline(), ':')[1])
        phlCang.val = float(string.split(infile.readline(), ':')[1])
        phlCfix.val = float(string.split(infile.readline(), ':')[1])
        phlCcluster.val = float(string.split(infile.readline(), ':')[1])
        phlCQMC.val = float(string.split(infile.readline(), ':')[1])
        toposphlC[0] = float(string.split(infile.readline(), ':')[1])
        toposphlC[1] = float(string.split(infile.readline(), ':')[1])
        toposphlC[2] = float(string.split(infile.readline(), ':')[1])
    
    infile.close()


def scanarchive():

    goodones = []
    names=os.listdir(archdir)
    if len(names):
        for name in names:
            root,ext = os.path.splitext(name)
            if ext == ".mat":
                goodones.append(root) 
                names = string.join(goodones, "|")
    return names



def elimina(dir, name):
    os.remove(os.path.join(dir, name))

def getTargetPos():
    try:
        selectedTO = Blender.Object.GetSelected()[0]
    except:
        return [0,0,0]

    toname = selectedTO.name
    selname = toname
    toObj=Blender.Object.Get(toname)
    top = toObj.loc
    return [toname, [top[0], top[1], top[2]]]
    print "TO Position ", top



def getSelectedObject():
    global selectedname, selectedtype, assigned, blcolors, lampdat
    try:
        selected = Blender.Object.GetSelected()[0]
        selectedname = selected.name
        assigned = 0

## jms : fix for Blender 2.28   6/n --  start                     
        if BL_VERSION<228:
            if BL_VERSION<=223:
                dt = selected.data
            else:
                dt = selected.getData()
            if dt:
                if dt.block_type=='NMesh':
                    selectedtype = 'Type: Mesh'
                    blcolors = retrieve_colors(selectedname)
                elif dt.block_type=='Lamp':
                    selectedtype = 'Type: Lamp'
                    lampdat = retrievelampdata(selectedname)
                else:
                    selectedtype = 'Type: Other'
            return 1
        else:
            dt = selected.getData()
            if dt:
                if selected.getType()=='Mesh':
                    selectedtype = 'Type: Mesh'
                    blcolors = retrieve_colors(selectedname)
                elif selected.getType()=='Lamp':
                    selectedtype = 'Type: Lamp'
                    lampdat = retrievelampdata(selectedname)
                else:
                    selectedtype = 'Type: Other'
            return 1
## jms : fix for Blender 2.28   6/n --  end                     
            
        """
        if BL_VERSION<=223:
            dt = selected.data
        else:
            dt = selected.getData()
        if dt:
            if dt.block_type=='NMesh':
                selectedtype = 'Type: Mesh'
                blcolors = retrieve_colors(selectedname)
            elif dt.block_type=='Lamp':
                selectedtype = 'Type: Lamp'
                lampdat = retrievelampdata(selectedname)
            else:
                selectedtype = 'Type: Other'
        return 1
        """
        
    except:
        return 0

def getSelectedObjects():
    global selectedname, selectedtype, assigned, blcolors, lampdat
    try:
        print "yup"
        selected = allLoopSelected
        selectedname = selected.name
        assigned = 0
## jms : fix for Blender 2.28   7/n --  start                     
        if BL_VERSION<228:
            if BL_VERSION<=223:
                dt = selected.data
            else:
                dt = selected.getData()
            if dt:
                if dt.block_type=='NMesh':
                    selectedtype = 'Type: Mesh'
                    blcolors = retrieve_colors(selectedname)
                elif dt.block_type=='Lamp':
                    selectedtype = 'Type: Lamp'
                    lampdat = retrievelampdata(selectedname)
                else:
                    selectedtype = 'Type: Other'
            return 1
        else:
            dt = selected.getData()
            if dt:
                if selected.getType()=='Mesh':
                    selectedtype = 'Type: Mesh'
                    blcolors = retrieve_colors(selectedname)
                elif selected.getType()=='Lamp':
                    selectedtype = 'Type: Lamp'
                    lampdat = retrievelampdata(selectedname)
                else:
                    selectedtype = 'Type: Other'
            return 1
## jms : fix for Blender 2.28   7/n --  end                     

        """
        if BL_VERSION<=223:
            dt = selected.data
        else:
            dt = selected.getData()
        if dt:
            if dt.block_type=='NMesh':
                selectedtype = 'Type: Mesh'
                blcolors = retrieve_colors(selectedname)
            elif dt.block_type=='Lamp':
                selectedtype = 'Type: Lamp'
                lampdat = retrievelampdata(selectedname)
            else:
                selectedtype = 'Type: Other'
        return 1
        """
        
    except:
        return 0

def drawcolorbox(xs,ys,xd,yd,R,G,B):
        glColor3f(0,0,0)
        glRectf(xs,ys,xd,yd)
        glColor3f( R, G, B)  
        glRectf(xs+2,ys-2,xd-2,yd+2)
        glColor3f(1,1,1)

def writematdesc(dir,name):
    #Scrivo una descrizione dei materiali nella directory
    mname = name + ".mat"
    matfilename = os.path.join(dir, mname)
    matfile = open(matfilename,'w')


    if (surfshader.val==1):
        #Generic

        matfile.write("Generic\n")
        matfile.write("GenR : %f\n"%GenR.val)
        matfile.write("GenB : %f\n"%GenG.val)
        matfile.write("GenB : %f\n"%GenB.val)
        matfile.write("addSpec : %f\n"%addSpec.val)
        if addSpec.val:
            matfile.write("GenSpecR : %f\n"%GenSpecR.val)
            matfile.write("GenSpecG : %f\n"%GenSpecG.val)
            matfile.write("GenSpecB : %f\n"%GenSpecB.val)
                    
        matfile.write("addRef : %f\n"%addRef.val)
        if addRef.val:
            matfile.write("GenRefR : %f\n"%GenRefR.val)
            matfile.write("GenRefG : %f\n"%GenRefG.val)
            matfile.write("GenRefB : %f\n"%GenRefB.val)         
            matfile.write("addRef2 : %f\n"%addRef2.val)
            if addRef2.val:
                matfile.write("GenRef2R : %f\n"%GenRef2R.val)
                matfile.write("GenRef2G : %f\n"%GenRef2G.val)
                matfile.write("GenRef2B : %f\n"%GenRef2B.val)
            
        matfile.write("addTra : %f\n"%addTra.val)
        if addTra.val:
            matfile.write("GenTraR : %f\n"%GenTraR.val)
            matfile.write("GenTraG : %f\n"%GenTraG.val)
            matfile.write("GenTraB : %f\n"%GenTraB.val)
            matfile.write("addTra2 : %f\n"%addTra2.val)
            if addTra2.val:
                matfile.write("GenTra2R : %f\n"%GenTra2R.val)
                matfile.write("GenTra2G : %f\n"%GenTra2G.val)
                matfile.write("GenTra2B : %f\n"%GenTra2B.val)
            
        matfile.write("GenHard : %f\n"%GenHard.val)


        matfile.write("addIOR : %f\n"%addIOR.val)
        if addIOR.val:
            matfile.write("GenIOR : %f\n"%GenIOR.val)

        #if addIOR.val:
        #   matfile.write("Ior\n")
        #   matfile.write("%f\n"%GenIOR.val)
        #else:
        #   matfile.write("noIor\n")
                    
        matfile.write("GenMinRef : %f\n"%GenMinRef.val)
        matfile.write("GenFastFresnel : %f\n"%GenFastFresnel.val)


    elif (surfshader.val==2):
        #Constant
        matfile.write("Constant\n")
        matfile.write("%f\n"%ConstR.val)
        matfile.write("%f\n"%ConstG.val)
        matfile.write("%f\n"%ConstB.val)
    #AutoSmooth Angle
    matfile.write("AutoSmooth : %f\n"%AutoSmooth.val)
    if AutoSmooth.val:
        matfile.write("Degr : %f\n"%Degr.val)
    matfile.close()



def writelightdesc(dir,name):
    #Scrivo una descrizione della Lamp nella directory

    mname = name + ".luz"
    lightlist[selectedname] = mname
    lightfilename = os.path.join(dir, mname)
    lightfile = open(lightfilename,'w')
    lampObj = Blender.Object.Get(name)

    if (lightshader.val== 4):
        #AreaLight
        lightfile.write("AreaLight\n")
        lightfile.write("alR : %f\n"%alR.val)
        lightfile.write("alG : %f\n"%alG.val)
        lightfile.write("alB : %f\n"%alB.val)
        lightfile.write("alpow : %f\n"%alpow.val)
        lightfile.write("alSamp : %f\n"%alSamp.val)
        lightfile.write("alPSamp : %f\n"%alPSamp.val)
        lightfile.write("alLato : %f\n"%alLato.val)
        for i in range(4):
            for j in range(3):
                lightfile.write("corner : %f\n"%corners[i][j])

    else:

        pos = lampObj.loc

        if (lightshader.val==1):
            #Point
            lightfile.write("PointLight\n")
            lightfile.write("plR : %f\n"%plR.val)
            lightfile.write("plG : %f\n"%plG.val)
            lightfile.write("plB : %f\n"%plB.val)
            lightfile.write("plpow : %f\n"%plpow.val)
            lightfile.write("plshad : %f\n"%plshad.val)

        elif (lightshader.val==2):
            #Sun
            lightfile.write("SunLight\n")
            lightfile.write("slR : %f\n"%slR.val)
            lightfile.write("slG : %f\n"%slG.val)
            lightfile.write("slB : %f\n"%slB.val)
            lightfile.write("slpow : %f\n"%slpow.val)
            lightfile.write("slshad : %f\n"%slshad.val)
    
            
        elif  (lightshader.val==3):
            #Soft
            lightfile.write("SoftLight\n")
            lightfile.write("sflR : %f\n"%sflR.val)
            lightfile.write("sflG : %f\n"%sflG.val)
            lightfile.write("sflB : %f\n"%sflB.val)
            lightfile.write("sflpow : %f\n"%sflpow.val)
            lightfile.write("sflres : %f\n"%sflres.val)
            lightfile.write("sflrad : %f\n"%sflrad.val)
            lightfile.write("sflbias : %f\n"%sflbias.val)
            
        elif (lightshader.val == 5):
            #Spot
            lightfile.write("SpotLight\n")
            lightfile.write("splR : %f\n"%splR.val)
            lightfile.write("splG : %f\n"%splG.val)
            lightfile.write("splB : %f\n"%splB.val)
            lightfile.write("splpow : %f\n"%splpow.val)
            lightfile.write("splsize : %f\n"%splsize.val)
            lightfile.write("splbl : %f\n"%splbl.val)
            lightfile.write("splbeoff : %f\n"%splbeoff.val)
            lightfile.write("splshadow : %f\n"%splshadow.val)
            lightfile.write("splhaloADD : %f\n"%splhaloADD.val)
            if splhaloADD.val:
                lightfile.write("splhalofogR : %f\n"%splhalofogR.val)
                lightfile.write("splhalofogG : %f\n"%splhalofogG.val)
                lightfile.write("splhalofogB : %f\n"%splhalofogB.val)
                lightfile.write("splhalofogdensity : %f\n"%splhalofogdensity.val)
                lightfile.write("splhaloblur : %f\n"%splhaloblur.val)
                lightfile.write("splhalores : %f\n"%splhalores.val)
                lightfile.write("splhalosamples : %f\n"%splhalosamples.val)
            lightfile.write("topos0 : %f\n"%topos[0])
            lightfile.write("topos1 : %f\n"%topos[1])
            lightfile.write("topos2 : %f\n"%topos[2])

        lightfile.write("pos0 : %f\n"%pos[0])
        lightfile.write("pos1 : %f\n"%pos[1])
        lightfile.write("pos2 : %f\n"%pos[2])
        
    lightfile.write("addphldiff : %f\n"%addphldiff.val)     
        
    if (addphldiff.val):
        #lightfile.write("DIFFUSE\n")
        lightfile.write("phlDpow : %f\n"%phlDpow.val)
        lightfile.write("phlDR : %f\n"%phlDR.val)
        lightfile.write("phlDG : %f\n"%phlDG.val)
        lightfile.write("phlDB : %f\n"%phlDB.val)
        lightfile.write("phlDph : %f\n"%phlDph.val)
        lightfile.write("phlDdep : %f\n"%phlDdep.val)
        lightfile.write("phlDsearch : %f\n"%phlDsearch.val)
        lightfile.write("phlDang : %f\n"%phlDang.val)
        lightfile.write("phlDfix : %f\n"%phlDfix.val)
        lightfile.write("phlDcluster : %f\n"%phlDcluster.val)
        lightfile.write("phlDQMC : %f\n"%phlDQMC.val)
        lightfile.write("toposphlD0 : %f\n"%toposphlD[0])
        lightfile.write("toposphlD1 : %f\n"%toposphlD[1])
        lightfile.write("toposphlD2 : %f\n"%toposphlD[2])

    lightfile.write("addphlcaus : %f\n"%addphlcaus.val)

    if (addphlcaus.val):
        #lightfile.write("CAUSTIC\n")
        lightfile.write("phlCpow : %f\n"%phlCpow.val)
        lightfile.write("phlCR : %f\n"%phlCR.val)
        lightfile.write("phlCG : %f\n"%phlCG.val)
        lightfile.write("phlCB : %f\n"%phlCB.val)
        lightfile.write("phlCph : %f\n"%phlCph.val)
        lightfile.write("phlCdep : %f\n"%phlCdep.val)
        lightfile.write("phlCsearch : %f\n"%phlCsearch.val)
        lightfile.write("phlCang : %f\n"%phlCang.val)
        lightfile.write("phlCfix : %f\n"%phlCfix.val)
        lightfile.write("phlCcluster : %f\n"%phlCcluster.val)
        lightfile.write("phlCQMC : %f\n"%phlCQMC.val)
        lightfile.write("toposphlC0 : %f\n"%toposphlC[0])
        lightfile.write("toposphlC1 : %f\n"%toposphlC[1])
        lightfile.write("toposphlC2 : %f\n"%toposphlC[2])

    lightfile.close()


## gestione dei links nella finestra delle luci ##

def linklightitems():
    if linkcolordiffdir.val:
        if (lightshader.val == 1):
            phlDR.val = plR.val
            phlDG.val = plG.val
            phlDB.val = plB.val
        elif (lightshader.val == 2):
            phlDR.val = slR.val
            phlDG.val = slG.val
            phlDB.val = slB.val
        elif (lightshader.val == 3):
            phlDR.val = sflR.val
            phlDG.val = sflG.val
            phlDB.val = sflB.val
        elif (lightshader.val == 4):
            phlDR.val = alR.val
            phlDG.val = alG.val
            phlDB.val = alB.val
        elif (lightshader.val == 5):
            phlDR.val = splR.val
            phlDG.val = splG.val
            phlDB.val = splB.val

    if linkcolorcausdiff.val:
        linkcolorcausdir.val = 0
        phlCR.val = phlDR.val
        phlCG.val = phlDG.val
        phlCB.val = phlDB.val

    if linkcolorcausdir.val:
        linkcolorcausdiff.val = 0
        if (lightshader.val == 1):
            phlCR.val = plR.val
            phlCG.val = plG.val
            phlCB.val = plB.val
        elif (lightshader.val == 2):
            phlCR.val = slR.val
            phlCG.val = slG.val
            phlCB.val = slB.val
        elif (lightshader.val == 3):
            phlCR.val = sflR.val
            phlCG.val = sflG.val
            phlCB.val = sflB.val
        elif (lightshader.val == 4):
            phlCR.val = alR.val
            phlCG.val = alG.val
            phlCB.val = alB.val
        elif (lightshader.val == 5):
            phlCR.val = splR.val
            phlCG.val = splG.val
            phlCB.val = splB.val

    if linkpowercausdiff.val:
        phlCpow.val = phlDpow.val
    if linkphotonscausdiff.val:
        phlCph.val = phlDph.val
    if linkdepthcausdiff.val:
        phlCdep.val = phlDdep.val
    if linksearchcausdiff.val:
        phlCsearch.val = phlDsearch.val
    if linkanglecausdiff.val:
        phlCang.val = phlDang.val
    if linkfixedcausdiff.val:
        phlCfix.val = phlDfix.val
    if linkclustercausdiff.val:
        phlCcluster.val = phlDcluster.val
    if linkqmccausdiff.val:
        phlCQMC.val = phlDQMC.val

    if linktargetdiffdir.val:
        toposphlD[0] = topos[0]
        toposphlD[1] = topos[1]
        toposphlD[2] = topos[2]
    
    if linktargetcausdiff.val:
        linktargetcausdir.val = 0
        toposphlC[0] = toposphlD[0]
        toposphlC[1] = toposphlD[1]
        toposphlC[2] = toposphlD[2]

    if linktargetcausdir.val:       
        linktargetcausdiff.val = 0
        toposphlC[0] = topos[0]
        toposphlC[1] = topos[1]
        toposphlC[2] = topos[2]



def multiply3x3(A, B):

    C = [[0, 0, 0], [0, 0, 0], [0, 0, 0]]
    for i in range(3):
        for j in range(3):
            for k in range(3):
                C[i][j] = C[i][j] + A[i][k]*B[k][j]
    return C

def multiply3x1(A, B):
    C = [0,0,0]
    for i in range(3):
        for j in range(3):
            C[i] = C[i] + A[i][j]*B[j]
    return C

#Some More Math Stuff
def veclen(v):
    return sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2])

def scalar(v1, v2):
    return v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2]

def somma(a,b):
    return [a[0]+b[0],a[1]+b[1],a[2]+b[2]]

def diff(a,b):
    return [a[0]-b[0],a[1]-b[1],a[2]-b[2]]

def normalize(vec):
    mod = veclen(vec)
    if (mod == 0.0): return [0, 0, 0]
    return [vec[0]/mod, vec[1]/mod, vec[2]/mod]

def normcrossprod(p,q):

    out = [0,0,0]
    out[0] = p[1]*q[2] - p[2]*q[1];
    out[1] = p[2]*q[0] - p[0]*q[2];
    out[2] = p[0]*q[1] - p[1]*q[0];
    return normalize(out)

def calcnorm(quad):
    a=quad[0]
    b=quad[1]
    c=quad[2]
    #p=b-a
    p = diff(b,a)
    #q=c-a
    q = diff(b,a)
    return normcrossprod(p,q)

#Angolo tra due vettori (in Gradi)
def angle(v1,v2):

    r=scalar(normalize(v1),normalize(v2))
    rr=acos(r)
    return radtodeg(rr)

#Data Una direzione ne restituisce il Coseno Direttore
def rotan(a):
    res=[0,0,0]

    res[0]=angle(a,[1,0,0])
    res[1]=angle(a,[0,1,0])
    res[2]=angle(a,[0,0,1])
    return res

def radtodeg(a):
    return a*57.295779513082323

def parallelVector(rotX, rotY, rotZ, oAxis):
    rotX = deg2rad(rotX)
    rotY = deg2rad(rotY) 
    rotZ = deg2rad(rotZ) 
    Tx = [[1, 0, 0], [0, cos(rotX), -sin(rotX)], [0, sin(rotX), cos(rotX)]]
    Ty = [[cos(rotY), 0, sin(rotY)], [0, 1, 0], [-sin(rotY), 0, cos(rotY)]]
    Tz = [[cos(rotZ), -sin(rotZ), 0], [sin(rotZ), cos(rotZ), 0], [0, 0, 1]]
    Tm = multiply3x3(multiply3x3(Tz, Ty), Tx)
    return multiply3x1(Tm, oAxis)

# returns matrix identity
def Identity():
    # return 4x4 identity
    return [    [1.0, 0.0, 0.0, 0.0],
            [0.0, 1.0, 0.0, 0.0],
            [0.0, 0.0, 1.0, 0.0],
            [0.0, 0.0, 0.0, 1.0]]
def nullmat():
    #return 4x4 null matrix
    return [[0,0,0,0], [0,0,0,0], [0,0,0,0], [0,0,0,0]]

def transpose(m):

    tra= nullmat()
    for row in range(4):
        for col in range(4):
            tra[row][col]=m[col][row]

    return  tra
    
def writeMatrix (exportFile, matrix):
    exportFile.write("<transform\n" )
    for row in range(4):
        for col in range(4):

            exportFile.write ("m%d%d = \"%f\" " % (row, col , matrix[row][col]))

        exportFile.write("\n")

    exportFile.write(">\n\n")

def deg2rad(a):
    b=a*0.017453292519943295
    return b


def deg2rad_v(a):

    r=[0,0,0]
    r[0]=deg2rad(a[0])
    r[1]=deg2rad(a[1])
    r[2]=deg2rad(a[2])
    return r
"""
def buildmat(obj):

    return transpose(obj.matrix)
"""
def buildmat(obname):

    return transpose(Blender.Object.Get(obname).matrix)

def addhemi(file):
    print ("Power: %f\n") %HPow.val
    file.write("<light type=\"hemilight\" name=\"sky\" power= \"%f\" "%HPow.val)
    if (HQMC.val):
        print ("Samples: %d\n QMC: on ") %HSamplesQMC.val
        file.write("samples =\"%d\" "  %HSamplesQMC.val) 
        file.write("use_QMC = \"on\"")
    else:
        print ("Samples: %d\n QMC: off ") %(HSamples.val*HSamples.val)
        file.write("samples = \"%d\" "  %(HSamples.val*HSamples.val))
        file.write("use_QMC = \"off\"")
    file.write(">\n")   
    if (not (HRed.val == 0 and HGreen.val == 0 and HBlu.val == 0) or not hdribg.val):
        if not(usebground.val):
            file.write("\t<color r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(HRed.val, HGreen.val, HBlu.val))
        else:
            file.write("\t<color r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(bgroundR.val,bgroundG.val,bgroundB.val,))
    file.write("</light>\n\n")

def addpathtrace(ofile):
    print ("Power: %f") %ptrPow.val
    ofile.write("<light type=\"pathlight\" name=\"path\" power= \"%f\" depth =\"%d\" "\
                            %(ptrPow.val, ptrDepth.val))
    if (ptrQMC.val):
        print ("Samples: %d\n QMC: on ") %ptrSamplesQMC.val
        ofile.write("samples = \"%d\" " %ptrSamplesQMC.val)
        ofile.write("use_QMC = \"on\"")
    else:
        print ("Samples: %d\n QMC: off ") %(ptrSamples.val*ptrSamples.val)
        ofile.write("samples =\"%d\" " %(ptrSamples.val*ptrSamples.val))
        ofile.write("use_QMC = \"off\"")    
    if (PathCache.val):
        ofile.write(" cache=\"on\" ")
        ofile.write(" cache_size=\"%f\" "  %cacheSize.val)
        ofile.write(" angle_threshold=\"%f\" "  %angleThreshold.val) 
        ofile.write(" shadow_threshold=\"%f\" " %shadowThreshold.val)
    ofile.write(">\n")
    ofile.write("</light>\n\n")

def commento(s):
    return "<!-- %s -->\n"%s

def writetextures(ob, ofile):
    global WRONGTEX
    if modulatorlist.has_key(ob):
        #Ok,...
        obmods=Modulators(ob)
        obmods.load()
        for i in range(len(obmods.modulators)):
            obmod = Modulator(obmods.modulators[i])
            obmod.load()
            type = obmod.type
            if type == "image":
                #Image
                obtex = imageTexture(obmods.modulators[i])
                obtex.load()
                #TODO .... do a search
                texpath=GetTexturePath(0,None,obtex.texpath)
                if texpath:
                    # replace win32 '\' with '/', otherwise it is written with a single '\' which will be interpreted as an escape char
                    texpath = texpath.replace('\\', '/')
                    st = '\n<texture name="%s" type="image">\n' % obtex.name
                    st += '\t<filename value="%s" />\n' %texpath
                    st += '</texture>\n\n'
                    ofile.write(st)
                else:
                    #Catched an error...Tag the modulatorlist
                    print "Catched an eror ",obtex.name 
                    WRONGTEX.append(obtex.name)
            elif type == "clouds":
                #clouds
                obtex = cloudsTexture(obmods.modulators[i])
                obtex.load()
                st = '\n<texture name="%s" type="clouds">\n' % obtex.name
                st +="\t<depth value=\"%d\" />\n"% obtex.depth
                st +="\t<color1 r=\"%f\" g=\"%f\" b=\"%f\" />\n"\
                    %(obtex.color1.R, obtex.color1.G, obtex.color1.B)
                st +="\t<color2 r=\"%f\" g=\"%f\" b=\"%f\" />\n"\
                    %(obtex.color2.R, obtex.color2.G, obtex.color2.B)
                st += '</texture>\n\n'
                ofile.write(st)
            else:
                #marble
                obtex = marbleTexture(obmods.modulators[i])
                obtex.load()
                st = '\n<texture name="%s" type="marble" depth=\"%d\" ' % (obtex.name, obtex.depth)
                if obtex.hard:
                    st += "hard=\"on\" "
                else:
                    st += "hard=\"off\" "
                st += "turbulence=\"%d\" sharpness=\"%d\" >\n" % (obtex.turbulence, obtex.sharpness)
                st += "\t<color1 r=\"%f\" g=\"%f\" b=\"%f\" />\n"\
                    %(obtex.color1.R, obtex.color1.G, obtex.color1.B)
                st +="\t<color2 r=\"%f\" g=\"%f\" b=\"%f\" />\n"\
                    %(obtex.color2.R, obtex.color2.G, obtex.color2.B)
                st += '</texture>\n\n'
                ofile.write(st)
            
            del obtex
            del obmod
        del obmods

def writemodulators(ob, ofile):
    if modulatorlist.has_key(ob):
        #Ok,...
        obmods=Modulators(ob)
        obmods.load()
        for i in range(len(obmods.modulators)):
            obmod = Modulator(obmods.modulators[i])
            obmod.load()
            if not obmod.texture in WRONGTEX:
                st = '\t<modulator texname="%s" mode="%s" '\
                    %(obmod.texture, obmod.mode)
                st += 'size="%f" \n' % obmod.size
                st += '\t\tsizex="%f" sizey="%f" sizez="%f" '\
                    %(obmod.sizex, obmod.sizey, obmod.sizez) 
                st += ">\n"
                if obmod.kc > 0:
                    st += '\t\t<color value="%f" />\n' % obmod.kc
                if  obmod.ks> 0:
                    st += '\t\t<specular value="%f" />\n' % obmod.ks
                if  obmod.kr> 0:
                    st += '\t\t<reflection value="%f" />\n' % obmod.kr
                if  obmod.kh> 0:
                    st += '\t\t<hard value="%f" />\n' % obmod.kh
                if  obmod.kt> 0:
                    st += '\t\t<transmission value="%f" />\n' % obmod.kt
                if  obmod.knorm:
                    st += '\t\t<normal value="%f" />\n' % obmod.knorm
                st += '\t</modulator>\n'
                ofile.write(st)
            else:
                print "\n\tTEXTURE ERROR: WRONG TEXTURE NAME FOR ", ob
                print "\tTHIS TEXTURE FILE DOES NOT EXIST IN THE TEXTURE PATH :", obmod.texture, "\n"
                
            del obmod
        del obmods


#def writeshader(type, infile, ofile, name, tex_modulator):
def writeshader(type, infile, ofile, name, texpath):
    #E: tex_written, list of texture names written to xml file
    global tex_written
    wasCrafter = 0
    if (type == 'Generic\n'):

        #E: writes texture, only when not written yet
        # tex_modulator = (texpath, texmode, texsize, tex_colorval)
        #if tex_modulator:

        #Name of the object the shader is linked to...
        obname = os.path.splitext(name)[0]
        print "UVLIST * OBNAME", uvlist , obname
        if texpath:

            if (uvlist.has_key(obname)):
                #texpath, texmode, texsize, tex_colorval = tex_modulator
                print "Carico UV settings"
                uvob=UVmodulator(obname)
                uvob.load()
                texmode = uvob.mode
                texsize = uvob.size
                texsizexyz = (uvob.sizex, uvob.sizey, uvob.sizez)
                tex_colorval = uvob.kc
                tex_reflval = uvob.kr
                tex_specval = uvob.ks
                tex_hardval = uvob.kh
                tex_transval=uvob.kt
                del uvob
            else:
                print "Carico UV default"
                #Default settings
                texmode = "mul"
                texsize = 1
                texsizexyz= (1,1,1)
                tex_colorval = 1.0
                tex_reflval = 0
                tex_specval = 0
                tex_hardval =0
                tex_transval=0
            # use image name without extension as name
            texname = os.path.splitext(os.path.split(texpath)[1])[0]
            #  check if in tex_written dict., is so, do nothing
            if texname not in tex_written:
                st = '\n<texture name="%s" type="image">\n' % texname
                st += '\t\t<filename value="%s" />\n' % texpath
                st += '</texture>\n\n'
                ofile.write(st)
                tex_written.append(texname)


        #Write additional textures
        writetextures(obname, ofile)

        ofile.write("<shader type = \"generic\" name = \"%s\">\n" %name)
        ofile.write("\t<attributes>\n")
        r = float(string.split(infile.readline(),':')[1])
        g = float(string.split(infile.readline(),':')[1])
        b = float(string.split(infile.readline(),':')[1])

        ofile.write("\t\t<color r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))
        
        add = float(string.split(infile.readline(),':')[1])
        if add:
            r = float(string.split(infile.readline(),':')[1])
            g = float(string.split(infile.readline(),':')[1])
            b = float(string.split(infile.readline(),':')[1])
            ofile.write("\t\t<specular r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))
                
        add = float(string.split(infile.readline(),':')[1])
        if add: 
            r = float(string.split(infile.readline(),':')[1])
            g = float(string.split(infile.readline(),':')[1])
            b = float(string.split(infile.readline(),':')[1])
            ofile.write("\t\t<reflected r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))
            add = float(string.split(infile.readline(),':')[1])
            if add: 
                r = float(string.split(infile.readline(),':')[1])
                g = float(string.split(infile.readline(),':')[1])
                b = float(string.split(infile.readline(),':')[1])
                ofile.write("\t\t<reflected2 r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))

        add = float(string.split(infile.readline(),':')[1])
        if add:
            r = float(string.split(infile.readline(),':')[1])
            g = float(string.split(infile.readline(),':')[1])
            b = float(string.split(infile.readline(),':')[1])
            ofile.write("\t\t<transmitted r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))
            add = float(string.split(infile.readline(),':')[1])
            if add:
                r = float(string.split(infile.readline(),':')[1])
                g = float(string.split(infile.readline(),':')[1])
                b = float(string.split(infile.readline(),':')[1])
                ofile.write("\t\t<transmitted2 r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))

        h = float(string.split(infile.readline(),':')[1])
        ofile.write("\t\t<hard value = \"%f\"/>\n" %h)

        setIor = float(string.split(infile.readline(),':')[1])
        #if setIor == "Ior\n":
        if setIor:
            i = float(string.split(infile.readline(),':')[1])
            ofile.write("\t\t<IOR value = \"%f\"/>\n" %i)

        mr = float(string.split(infile.readline(),':')[1])
        ofile.write("\t\t<min_refle value = \"%f\"/>\n" %mr)
        
        genfastfresn = float(string.split(infile.readline(),':')[1])
        if genfastfresn:
            ofile.write("\t\t<fast_fresnel value = \"on\"/>\n")
        else:
            ofile.write("\t\t<fast_fresnel value = \"off\"/>\n")
        ofile.write("\t</attributes>\n")

        #E: write texture modulator if used
        #if tex_modulator:
        if texpath:
            st = '\t<modulator texname="%s" mode="%s" ' % (texname, texmode)
            #if len(texsize)==1:
            st += 'size="%f"\n ' % texsize
            st += '\t\tsizex="%f" sizey="%f" sizez="%f" ' %texsizexyz
            st += ">\n"
            if tex_colorval > 0:
                st += '\t\t<color value="%f" />\n' % tex_colorval
            if tex_specval > 0:
                st += '\t\t<specular value="%f" />\n' % tex_specval
            if tex_reflval > 0:
                st += '\t\t<reflection value="%f" />\n' % tex_reflval
            if tex_hardval > 0:
                st += '\t\t<hard value="%f" />\n' % tex_hardval
            if tex_transval>0:
                st += '\t\t<transmission value="%f" />\n' % tex_transval
            st += '\t</modulator>\n'
            ofile.write(st)

        #Write additional modulators
        writemodulators(obname, ofile)
    elif (type == 'Crafter\n'):
        print "Write Shader -> Crafter"
        xml = infile.read()
        ofile.write(xml)
        wasCrafter = 1
    
    else:

        ofile.write("<shader type = \"constant\" name = \"%s\">\n" %name)
        ofile.write("\t<attributes>\n") 
        r = float(infile.readline())
        g = float(infile.readline())
        b = float(infile.readline())
        ofile.write("\t\t<color r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))
        ofile.write("\t</attributes>\n")
    if wasCrafter == 0:
        ofile.write("</shader>\n\n")

def writedefault(ofile):

    ofile.write("<shader type = \"generic\" name = \"Default\">\n")
    ofile.write("\t<attributes>\n")
    r=0.75
    g=0.75
    b=0.80
    ofile.write("\t\t<color r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))
    r=0
    g=0
    b=0
    ofile.write("\t\t<specular r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))
    ofile.write("\t\t<reflected r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))
    ofile.write("\t\t<transmitted r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))
    ofile.write("\t</attributes>\n")
    ofile.write("</shader>\n")

def writelight(type, infile, ofile, name, fr):
    print type
    onoff = ""
    if(type == "PointLight"):
        r=float(string.split(infile.readline(), ':')[1])
        g=float(string.split(infile.readline(), ':')[1])
        b=float(string.split(infile.readline(), ':')[1])
        p=float(string.split(infile.readline(), ':')[1])
        sh=float(string.split(infile.readline(), ':')[1])       
        if(sh):
            onoff="on"
        else:
            onoff="off"
        head = "<light type = \"pointlight\" name= \"%s\" " %name
        head +="power= \"%f\" "%p
        head +="cast_shadows= \"%s\" "%onoff
        head +=">\n"
        ofile.write(head)

        x = float(string.split(infile.readline(), ':')[1])
        y = float(string.split(infile.readline(), ':')[1])
        z = float(string.split(infile.readline(), ':')[1])
        ofile.write("\t<from x=\"%f\" y=\"%f\" z=\"%f\" />\n"%(fr[0],fr[1],fr[2]))
        ofile.write("\t<color r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))

    elif (type == "SunLight"):
        r=float(string.split(infile.readline(), ':')[1])
        g=float(string.split(infile.readline(), ':')[1])
        b=float(string.split(infile.readline(), ':')[1])
        p=float(string.split(infile.readline(), ':')[1])
        sh=float(string.split(infile.readline(), ':')[1])
        if(sh):
            onoff="on"
        else:
            onoff="off"
        ofile.write("<light type =\"sunlight\" name=\"%s\" power=\"%f\" cast_shadows=\"%s\">\n"%(name,p,onoff))
        x = float(string.split(infile.readline(), ':')[1])
        y = float(string.split(infile.readline(), ':')[1])
        z = float(string.split(infile.readline(), ':')[1])
        ofile.write("\t<from x=\"%f\" y=\"%f\" z=\"%f\" />\n"%(fr[0],fr[1],fr[2]))
        ofile.write("\t<color r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))

    elif (type == "SoftLight"):
        #SoftLight
        r=float(string.split(infile.readline(), ':')[1])
        g=float(string.split(infile.readline(), ':')[1])
        b=float(string.split(infile.readline(), ':')[1])
        p=float(string.split(infile.readline(), ':')[1])
        re=float(string.split(infile.readline(), ':')[1])
        ra=float(string.split(infile.readline(), ':')[1])
        bi=float(string.split(infile.readline(), ':')[1])

        ofile.write("<light type =\"softlight\" name=\"%s\" power=\"%f\" res=\"%d\" radius=\"%f\" bias=\"%f\" >\n"\
                            %(name,p,re,ra,bi))
        #dummy read
        x = float(string.split(infile.readline(), ':')[1])
        y = float(string.split(infile.readline(), ':')[1])
        z = float(string.split(infile.readline(), ':')[1])
        ofile.write("\t<from x=\"%f\" y=\"%f\" z=\"%f\" />\n"%(fr[0],fr[1],fr[2]))
        ofile.write("\t<color r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))

    elif (type == "SpotLight"):
        r=float(string.split(infile.readline(), ':')[1])
        g=float(string.split(infile.readline(), ':')[1])
        b=float(string.split(infile.readline(), ':')[1])

        p=float(string.split(infile.readline(), ':')[1])
        s=float(string.split(infile.readline(), ':')[1])
        bl=float(string.split(infile.readline(), ':')[1])
        bof=float(string.split(infile.readline(), ':')[1])
        ofile.write("<light type =\"spotlight\" name=\"%s\" power=\"%d\" size=\"%f\" \n\tblend=\"%f\" beam_falloff=\"%f\" "\
            %(name, p, s, bl, bof))
        shadow=float(string.split(infile.readline(), ':')[1])
        if shadow:
            ofile.write("cast_shadows=\"on\" ")
        else:
            ofile.write("cast_shadows=\"off\" ")
        haloadd = float(string.split(infile.readline(), ':')[1])
        if haloadd:
            halofogr = float(string.split(infile.readline(), ':')[1])
            halofogg = float(string.split(infile.readline(), ':')[1])
            halofogb = float(string.split(infile.readline(), ':')[1])
            halofogdensity = float(string.split(infile.readline(), ':')[1])
            haloblur = float(string.split(infile.readline(), ':')[1])
            halores = float(string.split(infile.readline(), ':')[1])
            halosamples = float(string.split(infile.readline(), ':')[1])
            ofile.write("\n\thalo=\"on\"  fog_density=\"%f\" blur=\"%f\" res=\"%d\" samples=\"%d\" "\
                %(halofogdensity, haloblur, halores, halosamples)) 
        ofile.write(">\n")
        ofile.write("\t<color r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))
        if haloadd:
            ofile.write("\t<fog r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(halofogr, halofogg, halofogb)) 
            
        xt = float(string.split(infile.readline(), ':')[1])
        yt = float(string.split(infile.readline(), ':')[1])
        zt = float(string.split(infile.readline(), ':')[1])
        x = float(string.split(infile.readline(), ':')[1])
        y = float(string.split(infile.readline(), ':')[1])
        z = float(string.split(infile.readline(), ':')[1])
        ofile.write("\t<from x=\"%f\" y=\"%f\" z=\"%f\" />\n"%(fr[0],fr[1],fr[2]))
        ofile.write("\t<to x=\"%f\" y=\"%f\" z=\"%f\" />\n"%(xt,yt,zt))

    else:
        #AreaLight
        r=float(string.split(infile.readline(), ':')[1])
        g=float(string.split(infile.readline(), ':')[1])
        b=float(string.split(infile.readline(), ':')[1])
        p=float(string.split(infile.readline(), ':')[1])
        sa=float(string.split(infile.readline(), ':')[1])
        psa=float(string.split(infile.readline(), ':')[1])
        tmp=float(string.split(infile.readline(), ':')[1])#don't need...bu necessary to skip lato info.
        ofile.write("<light type =\"arealight\" name=\"%s\" power=\"%f\" samples=\"%f\" psamples=\"%f\">\n"\
                %(name,p,sa, psa))

        #La normale deve 'Vedere' il punto verso cui e' diretta la luce
        if verso>0:
            corn = ["a","b","c","d"]
        else:
            corn = ["d","c","b","a"]

        for i in range(4):
            x = float(string.split(infile.readline(), ':')[1])
            y = float(string.split(infile.readline(), ':')[1])
            z = float(string.split(infile.readline(), ':')[1])
            ofile.write("\t<%s  x = \"%f\"  y = \"%f\"  z = \"%f\" />\n"%(corn[i],x, y, z))
        ofile.write("\t<color r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))

    ofile.write("</light>\n\n")
    
    
    ## Detect Diffuse Photon light properties and save as a new lightsource  ##
        
    adddiff=float(string.split(infile.readline(), ':')[1])
    if adddiff:
        nameD = name + "diff"
        p=float(string.split(infile.readline(), ':')[1])
        r=float(string.split(infile.readline(), ':')[1])
        g=float(string.split(infile.readline(), ':')[1])
        b=float(string.split(infile.readline(), ':')[1])    
        ph=float(string.split(infile.readline(), ':')[1])
        de=float(string.split(infile.readline(), ':')[1])
        se=float(string.split(infile.readline(), ':')[1])
    
    #   m=float(string.split(infile.readline(), ':')[1])
    #if m:
    #   mo="caustic"
    #else:
    #   mo="diffuse"
        
        an=float(string.split(infile.readline(), ':')[1])

        head = "<light type= \"photonlight\" name=\"%s\" " %nameD
        head += "power =\"%f\" photons =\"%d\" depth =\"%d\" "%(p, ph, de)
        head += "search = \"%d\" angle =\"%f\"\n " %(se,an)
        head += "\tmode = \"diffuse\" "

        frad=float(string.split(infile.readline(), ':')[1])
        head += "fixedradius = \"%f\" " %frad

        cl=float(string.split(infile.readline(), ':')[1])
        head += "cluster =\"%f\" " % cl

        qm=float(string.split(infile.readline(), ':')[1])
        if qm:
            onoff="on"
        else:
            onoff="off"
        head += "use_QMC =\"%s\" "%onoff

        head += ">\n"
        ofile.write(head)
    
        #dummy read
        #x = float(string.split(infile.readline(), ':')[1])
        #y = float(string.split(infile.readline(), ':')[1])
        #z = float(string.split(infile.readline(), ':')[1])
        ofile.write("\t<from x=\"%f\" y=\"%f\" z=\"%f\" />\n"%(fr[0],fr[1],fr[2]))

        x = float(string.split(infile.readline(), ':')[1])
        y = float(string.split(infile.readline(), ':')[1])
        z = float(string.split(infile.readline(), ':')[1])

        ofile.write("\t<to x=\"%f\" y=\"%f\" z=\"%f\" />\n"%(x,y,z))
        ofile.write("\t<color r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))

        ofile.write("</light>\n\n")

    
    ## Detect Caustic Photon light properties and save as a new lightsource  ##
    
    addcaus=float(string.split(infile.readline(), ':')[1])
    if addcaus:
        nameC = name + "caus"
        p=float(string.split(infile.readline(), ':')[1])
        r=float(string.split(infile.readline(), ':')[1])
        g=float(string.split(infile.readline(), ':')[1])
        b=float(string.split(infile.readline(), ':')[1])    
        ph=float(string.split(infile.readline(), ':')[1])
        de=float(string.split(infile.readline(), ':')[1])
        se=float(string.split(infile.readline(), ':')[1])   
        an=float(string.split(infile.readline(), ':')[1])

        head = "<light type= \"photonlight\" name=\"%s\" " %nameC
        head += "power =\"%f\" photons =\"%d\" depth =\"%d\" "%(p, ph, de)
        head += "search = \"%d\" angle =\"%f\"\n " %(se,an)
        head += "\tmode = \"caustic\" "

        frad=float(string.split(infile.readline(), ':')[1])
        head += "fixedradius = \"%f\" " %frad

        cl=float(string.split(infile.readline(), ':')[1])
        head += "cluster =\"%f\" " % cl

        qm=float(string.split(infile.readline(), ':')[1])
        if qm:
            onoff="on"
        else:
            onoff="off"
        head += "use_QMC =\"%s\" "%onoff

        head += ">\n"
        ofile.write(head)
    
        #dummy read
        #x = float(string.split(infile.readline(), ':')[1])
        #y = float(string.split(infile.readline(), ':')[1])
        #z = float(string.split(infile.readline(), ':')[1])
        ofile.write("\t<from x=\"%f\" y=\"%f\" z=\"%f\" />\n"%(fr[0],fr[1],fr[2]))

        x = float(string.split(infile.readline(), ':')[1])
        y = float(string.split(infile.readline(), ':')[1])
        z = float(string.split(infile.readline(), ':')[1])

        ofile.write("\t<to x=\"%f\" y=\"%f\" z=\"%f\" />\n"%(x,y,z))
        ofile.write("\t<color r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(r,g,b))

        ofile.write("</light>\n\n")


def addlamp(name, file):

    luzname =   lightlist[name]
    luzfile =   os.path.join(matdir,luzname)
    name,ext = os.path.splitext(luzname)
    fr=[0,0,0]
    lampob = Blender.Object.Get(name)
    fr[0],fr[1],fr[2] = lampob.loc
    print "from ", fr
    luzf    =   open(luzfile, 'r')
    ltype   =   readstr(luzf)

    writelight(ltype, luzf, file, name, fr) 

    luzf.close()




def writeDOF(outfile):
    fil="<filter type=\"dof\" name=\"dof\" "
    fil+="focus = \"%s\" " %focus
    fil+="near_blur =\"%f\" " %nearblur.val
    fil+="far_blur =\"%f\" " %farblur.val
    fil+="scale =\"%f\"> \n" %scale.val
    fil+="</filter>\n\n"
    outfile.write(fil)


def writeANoise(outfile):
    fil="<filter type=\"antinoise\" name=\"Anti Noise\" "
    fil+="radius = \"%f\" " % anoise_radius.val
    fil+="max_delta = \"%f\">\n" % anoise_maxdelta.val
    fil+="</filter>\n\n"
    outfile.write(fil)



# Find the full path of the first texture in the mesh
#Modified by Andrea
#flag ==1: Search for a mesh
#flag ==0: Search for a generic pattern
def GetTexturePath(flag, mesh=None, pattern=""):
    # TEXROOT must be defined
    if len(TEXROOT.val)==0:
        print "No texture directory defined!"
        return
    # Since the image name does not contain the full path and can actually be only a part of
    # the actual name, find the actual file matching the namefragment including the full path.
    if flag:
        if not mesh.hasFaceUV(): return # no uv-coords, no texture(s)
        texname = None
        for f in mesh.faces:
            if f.image:
                texname = f.image.name
                break
        if texname==None:
            print "DEBUG: GetTexturePath -> Mesh has UV coords, but no textures found!"
            return
        # Split of any blender appended numbers like .001, leaving only the real texture name
        texname = texname.split('.')
        # recombine with the extension, ignore all extras
        if len(texname)>1:
            texname = texname[0] + '.' + texname[1]
        else:
            texname = texname[0]
    else:
        if pattern==None or pattern=="":
            print "Cannot find the texture The name is a Void String!"
            return
        else:
            texname = pattern
    if texname:
        # find the full path to the texture matching the name, including subdirectories
        results_matched = WalkFind(TEXROOT.val, 1, texname)
        if results_matched:
            # if there are more matches, use the one which has .jpeg as extension
            if len(results_matched)>1:
                imgname = None
                for r in results_matched:
                    
                    if r.lower().find('.tga')!=-1:
                        imagename = r
                        break
                    
                    elif (r.lower().find('.jpeg')!=-1 or r.lower().find('.jpg')!=-1):
                        #or r.lower().find('.JPEG')!=-1 or r.lower().find('.JPG')!=-1):
                        imagename = r
                        break
            else:
                # otherwise use the first in the list, will be converted if possible and not a .tga
                imagename = results_matched[0]
            return imagename
    return


# A function to find a full path to a file matching the argument (possibly incomplete fragment), in any directory starting with 'root'
def WalkFind(root, recurse=0, pattern=''):
    # initialize
    result = []
    # must have at least root folder
    try:
        names = os.listdir(root)
    except os.error:
        return result
    # check each file
    for name in names:
        fullname = os.path.normpath(os.path.join(root, name))
        # grab if it matches our pattern and entry type
        # In case the name is exactly the same, nothing to be done further
        # only the name needs to be compared with pattern, fullname is used for result
        if name==pattern:
            if os.path.isfile(fullname):
                result = [fullname]
                break
        elif name.find(pattern)!=-1:
            if os.path.isfile(fullname):
                result.append(fullname)
        # recursively scan other folders, appending results
        if recurse:
            #TODO: skip hidden directories
            if os.path.isdir(fullname) and not os.path.islink(fullname):
                result += WalkFind(fullname, recurse, pattern)
    return result



def addmesh(name, file):

    global expshaders, savedmesh, triangles
    numfaces = 0
    numvertlist = []
    facepointlist = []
    shadername = "Default"
    autosm = 0
    if (meshmat.has_key(name)):
        shadername = meshmat[name]

    # mesh data must be here, to get possible texture info for shader
    #Needs a FIX ?
    #Better add a Check if blendobject returns None..
    blendobject = Blender.Object.Get(name)
    blendmesh = blendobject.data

    # get mesh, and write any uv-textures
    blendmeshraw = Blender.NMesh.GetRawFromObject(name) #ORIGINAL

    hasuv = blendmeshraw.hasFaceUV() 

    print "DEBUG: addmesh -> hasuv  ", hasuv
    # get full path to texture
    texpath = GetTexturePath(1, blendmeshraw) #ORIGINAL
    #texpath = GetTexturePath(blendmesh)
    print "DEBUG: addmesh -> texpath ", texpath

    if texpath:
        # replace win32 '\' with '/', otherwise it is written with a single '\' which will be interpreted as an escape char
        texpath = texpath.replace('\\', '/')

    if not (shadername in expshaders):

        expshaders.append(shadername)
        if not (shadername == "Default"):

            shadfile = os.path.join(matdir,shadername)

            matfile = open(shadfile, 'r')

            stype = matfile.readline()

            if texpath:
                pass
            else:
                #tex_modulator = None
                texpath = None
                
            if(stype == "Crafter\n"):
                autosm=float(string.split(matfile.readline(),':')[1])
                if autosm: autosmdeg=float(string.split(matfile.readline(),':')[1])
            
            writeshader(stype, matfile, file, shadername, texpath)

            if(stype != "Crafter\n"):
                autosm=float(string.split(matfile.readline(),':')[1])
                if autosm: autosmdeg=float(string.split(matfile.readline(),':')[1])
                
            matfile.close()
        else:

            writedefault(file)

    print "nome: %s - shader: %s" %(name, shadername)

    #transform = buildmat(blendobject)
    transform = buildmat(name)

    #E: vertex check moved here, since in case the mesh has 'illegal' vertices (less than three),
    # an empty mesh file would still be written, which causes yafray to crash
    if (update.val or DISABLE.val or not(savedmesh.has_key(name) or savedmesh.has_key(name+".xml"))):
        #E: check number of actual vertices used (triangles & quads)
        used_verts_ids = {} # dictionary to map vertex indices as used in the mesh to vertices as listed in export
        for f in blendmeshraw.faces:  #ORIGNAL
            nv = len(f.v)
            if (nv==3) or (nv==4):
                for v in f.v:
                    used_verts_ids[v.index] = 1

        if len(used_verts_ids)==0:
            print "Mesh invalid, skipped"
            return

    writeMatrix(file,transform)
    objtag = "<object name = \"%s\" shader_name = \"%s\" " %(name, shadername)
    attrtag= "\t<attributes>\n"

    if (meshattr.has_key(name)):
        optargs = loadopt(meshattr[name])    #optargs e' a sua volta un dizionario
        if optargs.has_key("CausIOR"):
            objtag += "caus_IOR = \"%f\" " %optargs["CausIOR"]
        objtag += "recv_rad = \"%s\" " %optargs["recrad"]
        objtag += "emit_rad =\"%s\" " %optargs["emirad"]
        objtag +="\n\tshadow = \"%s\""%(optargs["cast"])
        if (optargs["caus_tcolor"]):
            attrtag += "\t\t<caus_tcolor r = \"%f\" g = \"%f\" b = \"%f\"/>\n"\
                %(optargs["tR"],optargs["tG"],optargs["tB"])

        if (optargs["caus_rcolor"]):
            attrtag += "\t\t<caus_rcolor r = \"%f\" g = \"%f\" b = \"%f\" />\n" \
                %(optargs["rR"],optargs["rG"],optargs["rB"])

    attrtag += "\t</attributes>\n"
    objtag  += ">\n"
    file.write(objtag)
    file.write(attrtag)

    mesh_str="\t\t<mesh" 

    if autosm:
        print "Applying autosmooth %s" % autosmdeg
        mesh_str+="    autosmooth = \"%s\" > \n" % autosmdeg
    else:
        mesh_str+=">\n"

    file.write(mesh_str)

    if GZIPOK and gzipit.val:
        savingname=os.path.join(meshdir, name+".xml.gz")
        if anim.val:
            yafsavingname = os.path.join(".","Meshes",name+".xml.gz" )
        else:
            yafsavingname = os.path.join("Meshes",name+".xml.gz" )
            #on Windows it fails... wrong slashes!
            if not POSIX:
                yafsavingname.replace('/', '\\')

    else:
        savingname=os.path.join(meshdir, name+".xml")
        if anim.val:
            yafsavingname = os.path.join("Meshes",name+".xml" )
        else:
            yafsavingname = os.path.join(".","Meshes",name+".xml" )
            #on Windows it fails... wrong slashes!
            if not POSIX:
                yafsavingname.replace('/', '\\')

    if (update.val or DISABLE.val or not(savedmesh.has_key(name) or savedmesh.has_key(name+".xml"))):
        #E: indent_st -> indentation string for nicer formatting when not using gzip, not essential at all of course, but why not...
        if (GZIPOK and gzipit.val and not DISABLE.val):
            meshfile = gzip.open(savingname, 'wb', 9)
            indent_st = ''
        elif DISABLE.val:
            meshfile = file
            indent_st = '\t\t\t'
        else:
            meshfile = open(savingname, 'w')
            indent_st = '\t\t\t'

        print "number of original vertices: %d" % len(blendmeshraw.verts) #ORIGINAL
        
        nfaces = len(blendmeshraw.faces) #ORIGINAL

        print "number of faces %d" % nfaces

        meshfile.write("<points>\n")

        #E: write only the used vertices, creating new indices for the actual used vertices
        new_idx = idx = 0
        for v in blendmeshraw.verts: #ORIGINAL

            if used_verts_ids.has_key(idx):
                meshfile.write('\t<p x="%f"  y="%f"  z="%f" />\n ' % tuple(v.co))
                # create new index
                used_verts_ids[idx] = new_idx
                new_idx += 1
            idx += 1

        meshfile.write("</points>\n\n<faces>\n")

        #E: using the index mapping dict. the real mesh face vertex indices are converted to the export list indices
        for blface in blendmeshraw.faces:#ORIGINAL
            nvert = len(blface.v)
            if nvert==3:
                triangles+=1
                meshfile.write('\t<f a="%d" b="%d" c="%d" ' % (used_verts_ids[blface.v[0].index],used_verts_ids[blface.v[1].index], used_verts_ids[blface.v[2].index]))
                if hasuv:   
                    meshfile.write('u_a="%f" v_a="%f" ' % (blface.uv[0][0], 1.0-blface.uv[0][1]))
                    meshfile.write('u_b="%f" v_b="%f" ' % (blface.uv[1][0], 1.0-blface.uv[1][1]))
                    meshfile.write('u_c="%f" v_c="%f" ' % (blface.uv[2][0], 1.0-blface.uv[2][1]))
                meshfile.write('/>\n')

            elif nvert==4 :
                triangles+=2
                meshfile.write('\t<f a="%d" b="%d" c="%d" ' % (used_verts_ids[blface.v[0].index],used_verts_ids[blface.v[1].index], used_verts_ids[blface.v[2].index]))
                if hasuv:
                    meshfile.write('u_a="%f" v_a="%f" ' % (blface.uv[0][0], 1.0-blface.uv[0][1]))
                    meshfile.write('u_b="%f" v_b="%f" ' % (blface.uv[1][0], 1.0-blface.uv[1][1]))
                    meshfile.write('u_c="%f" v_c="%f" ' % (blface.uv[2][0], 1.0-blface.uv[2][1]))
                meshfile.write('/>\n')

                meshfile.write('\t<f a="%d" b="%d" c="%d" ' % (used_verts_ids[blface.v[2].index], used_verts_ids[blface.v[3].index], used_verts_ids[blface.v[0].index]))
                if hasuv:
                    meshfile.write('u_a="%f" v_a="%f" ' % (blface.uv[2][0], 1.0-blface.uv[2][1]))
                    meshfile.write('u_b="%f" v_b="%f" ' % (blface.uv[3][0], 1.0-blface.uv[3][1]))
                    meshfile.write('u_c="%f" v_c="%f" ' % (blface.uv[0][0], 1.0-blface.uv[0][1]))
                meshfile.write('/>\n')

        meshfile.write(indent_st + "</faces>\n")
        if not DISABLE.val:
            meshfile.close()
    else:
        print "Including mesh from: %s\n"%yafsavingname

    if not DISABLE.val:

        if fullpath.val:
            file.write("\t\t<include file = \"%s\" />\n" %savingname)
        else:
            file.write("\t\t<include file = \"%s\" />\n" %yafsavingname)

    file.write("\t\t</mesh>\n")
    file.write("</object>\n")
    file.write("</transform>\n\n")

#########################
#        MAIN PART 
#########################

def exporttoyafray():

    #JWM appviewfilename
    global expshaders, savedmesh, focus, triangles, appviewfilename
    global WRONGTEX
    #E: list of texture names written to xml file, reset
    global tex_written
    tex_written = []
    WRONGTEX = []
    suntrack = [0,0,0]

    #Lista delle Mesh memorizzate nella directory Meshes
    if GZIPOK and gzipit.val:
        xmlext=".gz"
    else:
        xmlext=".xml"

    savedmesh=scanlocal(meshdir, xmlext)
    # Tiene traccia degli shader gia' descritti
    expshaders = []
    if anim.val:
        # Creo la cartella locale dei materiali
        moviedir = os.path.join(outdir, image.val+"_MOVIE")

        if not os.path.exists(moviedir):

            try:
                os.mkdir(moviedir)
            except OSError:
                raise "Could not create Movie output directory"
        else:
            print "%s already exists" %moviedir

    if fullpath.val:
        if anim.val:
            imagefilename = os.path.join(moviedir,image.val)
        else:
            imagefilename = os.path.join(outdir,image.val)
    else:
        imagefilename = image.val
    exfname = fname.val 

    if finname.val:
        frame = str(Blender.Get("curframe"))
        frame = frame.rjust(1+len(str(Blender.Get("endframe"))))
        frame = frame.replace(" ", "0")
        exfname += "."+frame
        imagefilename += "."+frame

    exfname += ".xml"
    if anim.val:
        exportfilename = os.path.join(moviedir, exfname)
    else:
        exportfilename = os.path.join(outdir, exfname)

    imagefilename += ".tga"
    
    #Now xml output is always  NON-gzipped
    file = open(exportfilename, 'w')

    meshlist = []
    lamplist = []
    
    file.write(commento("Exported with YaBlE.\n For any comment or question refer to tredee@katamail.com"))
    file.write(commento("Visit http://kino3d.com/~yable for news and announcements"))
    file.write(commento("Thanks to Jandro, Eeshlo and all the guys from www.kino3d.com"))

    """
    print("<Getting List of Meshes>\n")
    for i in Blender.Object.Get():
        try:
            if BL_VERSION <= 223:
                dt = i.data
                lay = i.Layer
            else:
                dt = i.getData()
                lay = i.layer
            if dt :
                if lay & bmask:
                    print "Object don't belong to exported layers!"
                    if dt.block_type=='NMesh':

                        if not (dt.name == "DELETEME"):
                            meshlist.append(i.name)
                            print i.name
                        else:
                            print "Skipped Exporting AreaLight Quad"
    
                    elif dt.block_type=='Lamp':
                        lamplist.append(i.name)
        except:
            #Skipping, can export only mesh
            print "\nskipping\n"
            continue
    """
## jms : fix for Blender 2.28   2/n --  start    
    print("<Getting List of Meshes>\n")
    for i in Blender.Object.Get():
        try:
            if BL_VERSION <= 223:
                dt = i.data
                lay = i.Layer
            elif BL_VERSION < 228:
                dt = i.getData()
                lay = i.layer
            else:
                dt = i.getData()
                lay = i.Layer
            if dt :
                if lay & bmask:
                    print "Object don't belong to exported layers!"
                    if BL_VERSION >= 228:
                        if i.getType()=='Mesh':
                            if not (dt.name == "DELETEME"):
                                meshlist.append(i.name)
                                print i.name
                            else:
                                print "Skipped Exporting AreaLight Quad"
                        elif i.getType()=='Lamp':
                            lamplist.append(i.name)
                    else:
                        if dt.block_type=='NMesh':
                            if not (dt.name == "DELETEME"):
                                meshlist.append(i.name)
                                print i.name
                            else:
                                print "Skipped Exporting AreaLight Quad"    
                        elif dt.block_type=='Lamp':
                            lamplist.append(i.name)
        except:
            #Skipping, can export only mesh
            print "\nskipping\n"
            continue
## jms : fix for Blender 2.28   2/n --  start
        
    """
    #Better skip this...
    #Controlo la directory Materials
    #Se ho cancellato mesh durante l'esecuzione
    #Elimino anche gli shader associati
    for i in meshmat.keys():
        if i in meshlist:
            pass
        else:
            del meshmat[i]
            elimina(matdir, i+".mat")
            #Check the emptiness of meshattr
            if len(meshattr):
                if meshattr.has_key(i):
                    elimina(matdir, i+".objattr")
                    del meshattr[i]
            if len(savedmesh):
                if savedmesh.has_key(i):
                    elimina(meshdir,savedmesh[i])
                    del savedmesh[i]
    """
    print("\nGot Meshes\n") 

    curfr=Blender.Get("curframe")
    if BL_VERSION >= 225:
        Blender.Window.drawProgressBar(0,"Frame:%d")%curfr
    # START WRITING THE .XML FILE
    file.write("<scene>\n\n")
    if len(meshlist):
        bar = 1/len(meshlist)
    mi=0
    # NOW WRITE THE MESHES
    for meshname in meshlist:
        print "adding mesh %s " % meshname, "\n"
        if BL_VERSION >= 225:
            Blender.Window.drawProgressBar(bar*mi,"Frame: %d"%curfr)
        mi+=1
        addmesh(meshname,file)

    if (hemi.val):
        print "Adding Hemi Light"
        #Add Hemi Light
        addhemi(file)

    if (pathtrace.val):
        print "Adding Path Tracing"
        addpathtrace(file)

    if len(lamplist):
        print"<Adding Lights>\n"
        for light in lamplist:
            if lightlist.has_key(light):
                print light
                addlamp(light, file)

        print "</Adding lights>"
    else:
        print "No lights found"


    print "Adding Scene info:\n"

    outval = OutPicMenu.val
    if outval == 9:
        #Custom
        xRes = CustSizeX.val
        yRes = CustSizeY.val

    elif outval == 1:
        xRes = 320
        yRes = 240
    elif outval == 2:
        xRes = 480
        yRes = 320
    elif outval == 3:
        xRes = 640
        yRes = 480
    elif outval == 4:
        xRes = 640
        yRes = 512
    elif outval == 5:
        xRes = 768
        yRes = 470
    elif outval == 6:
        xRes = 1024
        yRes =  576
    elif outval == 7:
        xRes = 1024
        yRes = 768
    elif outval == 8:
        xRes = 1280
        yRes = 960

    print "\tResolution: %d, %d" %(xRes,yRes)

    if BL_VERSION<=223:
        camob = Blender210.getCurrentScene().getCurrentCamera()
        camname = camob.data
        cam = Blender210.getCamera(camname)
    else:
        camob = Blender.Scene.getCurrent().getCurrentCamera()
        cam = camob.getData()
        camname = cam.name
        #camname = "Camera"

    """
    jms : fix for blender 2.28
     
    Sorry but there is no need for this fix.
    There is no error if a F12 call to rendering is
    done in blender at least one time.
    
## FIX by Robert Scott - start ##
    
    if BL_VERSION<=223:
        camob = Blender210.getCurrentScene().getCurrentCamera()
        camname = camob.data
        cam = Blender210.getCamera(camname)
    else:
        camob = Object.get('Camera')        
        camname = "Camera"
## Meino Cramer - mcc - start ##
        cam=camob.getData() 
## Meino Cramer - mcc - end ##  
## FIX by Robert Scott - end ##
    """
    
    #E: calculate yafray focal distance
    if xRes > yRes:
        aspect = 1
    else:
        aspect = float(xRes) / float(yRes)

##     jms : fix for blender 2.28    start -- 1/n   
    if BL_VERSION<228:       
       focale = cam.Lens / (aspect * 32.0)
    else:
       focale = cam.lens / (aspect * 32.0)   
##     jms : fix for blender 2.28    end  --  1/n        

    file.write("<camera name=\"%s\" resx=\"%d\" resy=\"%d\" focal=\"%f\" >\n"\
        %( camname, xRes, yRes, focale))
    print "\tCamera Focal: %f"%focale

    #Camera Settings:  Copy & Paste From Eeshlo code .... Thanks!
    camtx = camob.matrix
    loc, size = tuple(camob.loc), tuple(camob.size)
    isy, isz = size[1], size[2]
    if isy!=0.0: isy = 1.0/isy
    if isz!=0.0: isz = 1.0/isz
    lookdir = (-camtx[2][0]*isz, -camtx[2][1]*isz, -camtx[2][2]*isz)
    lookAt = (loc[0] + lookdir[0], loc[1] + lookdir[1], loc[2] + lookdir[2])
    up = (camtx[1][0]*isy, camtx[1][1]*isy, camtx[1][2]*isy)

    st  = '\t<from x="%f" y="%f" z="%f" />\n' % loc
    st += '\t<to x="%f" y="%f" z="%f" />\n' % lookAt
    st += '\t<up x="%f" y="%f" z="%f" />\n' % (loc[0]+up[0], loc[1]+up[1], loc[2]+up[2])
    st += "</camera>\n\n"
    file.write(st)

    if FilterDOF.val:
        print "Adding DOF filter"
        focus=veclen(diff(loc,to_focus))
        writeDOF(file)

    if FilterANoise.val:
        print "Adding AntiNoise filter"
        writeANoise(file)
  
    ## background zone ##
    if constbg.val:
        bgname = "\"constbackg\""
        file.write("<background type=\"constant\" name =\"constbackg\">\n ")
        file.write("<color r=\"%f\" g=\"%f\" b=\"%f\"/>\n"\
                %(bgroundR.val,bgroundG.val,bgroundB.val))
        file.write("</background>\n")

    if sunbg.val:
        bgname = "\"Here Comes The Sun!\""
        if addsun.val:
            sunon="on"
        else:
            sunon="off"
        file.write("<background type=\"sunsky\" name =\"Here Comes The Sun!\" turbidity =\"%f\" add_sun=\"%s\" "\
                    %(turb.val, sunon))
        if addsun.val:
            file.write("sun_power=\"%f\"" %sunbgsunpower.val)
        file.write("a_var=\"%f\" b_var=\"%f\" c_var=\"%f\" d_var=\"%f\" e_var=\"%f\" "\
                    %(sunbgavar.val, sunbgbvar.val, sunbgbvar.val, sunbgdvar.val, sunbgevar.val))
        file.write(">\n")

        #Retrieve the sun position, good for animations
        suntrack = getsunpos()

        file.write("\t<from x=\"%f\" y=\"%f\" z=\"%f\"/>\n"%(suntrack[0], suntrack[1],suntrack[2]))
        file.write("</background>\n")
    
    if (hdribg.val == 1 and not hdriprobe.val == "EMPTY"):
        bgname = "\"envhdri\""
        fullhdriprobe = os.path.join(TEXROOT.val , hdriprobe.val)
        file.write("<background type = \"HDRI\" name = \"envhdri\" exposure_adjust = \"%d\">\n" %hdriexp.val)
        file.write("\t<filename value = \"%s\" />\n" %fullhdriprobe)
        file.write("</background>\n")

    if (normbg.val == 1 and not normbgimg.val == "EMPTY"):
        bgname = "\"envnorm\""
        fullnormbg = os.path.join(TEXROOT.val , normbgimg.val)
        file.write("<background type = \"image\" name = \"envnorm\" power = \"%f\">\n" %normbgpower.val)
        file.write("\t<filename value = \"%s\" />\n" %fullnormbg)
        file.write("</background>\n")
    
    rst = "\n<render camera_name = \"%s\" AA_passes = \"%d\" AA_minsamples = \"%d\" AA_pixelwidth = \"%f\" AA_threshold = \"%f\" \n\
\traydepth = \"%d\" bias = \"%f\" "\
                %(camname,AAPasses.val, AAMinSamples.val, AAPixelWidth.val, AAThreshold.val, CamRayDepth.val, CamBias.val)
    if CamIndSamples.val:
        rst += "indirect_samples = \"%d\" "%(CamIndSamples.val*CamIndSamples.val)
    rst += "gamma = \"%f\" exposure = \"%f\" "%(gamma.val, exposure.val)
    if (constbg.val or sunbg.val or hdribg.val or normbg.val):
        rst += "background_name=" + bgname

    if fog.val> 0:
        rst+="\nfog_density=\"%f\" "%fog.val

    rst+= " >\n"

    file.write(rst)
    if fog.val>0:
        file.write("<fog_color r=\"%f\" g=\"%f\" b=\"%f\"/>\n"%(fogR.val,fogG.val,fogB.val))
    file.write("\t<outfile value=\"%s\"/>\n"%(imagefilename))
    if alphatga.val:
        file.write("\t<save_alpha value=\"on\"/>\n")

    file.write("\n</render>\n")
    file.write("</scene>")
    print "Done adding Scene info"

    file.close()

    if triangles:
        print "\t\nExported %d Triangles\n"%triangles
        triangles = 0
    if BL_VERSION >= 225:
        Blender.Window.drawProgressBar(1,"Exporting")
    print "\nScene saved in %s\n" %exportfilename
    print "Rendered picture will be stored in: %s"%imagefilename
    #JWM
    appviewfilename = imagefilename
    print "This is the End...."
    sys.stdout.flush()
    return file.name

def matpreview():
    filenamepreview = os.path.join(YABLEROOT, "previewmat.xml")
    #preview_file = open("previewmat.xml","w")
    
    preview_file = open(filenamepreview, "w")
    preview_file.write(commento("Mat Preview Scene"))
    preview_file.write("<scene>\n")
    
#shader
    preview_file.write("<shader type = \"generic\" name = \"matprv\">\n")
    preview_file.write("\t<attributes>\n")
    preview_file.write("\t\t<color r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(GenR.val, GenG.val, GenB.val))
    if addSpec.val:
        preview_file.write("\t\t<specular r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(GenSpecR.val, GenSpecG.val, GenSpecB.val))
    if addRef.val:
        preview_file.write("\t\t<reflected r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(GenRefR.val, GenRefG.val, GenRefB.val))
        if addRef2.val:
            preview_file.write("\t\t<reflected2 r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(GenRef2R.val, GenRef2G.val, GenRef2B.val))
    if addTra.val:
        preview_file.write("\t\t<transmitted r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(GenTraR.val, GenTraG.val, GenTraB.val))
        if addTra2.val:
            preview_file.write("\t\t<transmitted2 r=\"%f\" g=\"%f\" b=\"%f\" />\n" %(GenTra2R.val, GenTra2G.val, GenTra2B.val))
    preview_file.write("\t\t<hard value = \"%f\" />\n" %GenHard.val)
    if addIOR.val:
        preview_file.write("\t\t<IOR value = \"%f\" />\n" %GenIOR.val)
    preview_file.write("\t\t<min_refle value = \"%f\" />\n" %GenMinRef.val)
    if GenFastFresnel.val:
        preview_file.write("\t\t<fast_fresnel value = \"on\" />\n")
    preview_file.write("\t</attributes>\n")
    preview_file.write("</shader>\n")

#CUBO
    preview_file.write("<transform \n \
m00 = \"1.000000\" m01 = \"0.000000\" m02 = \"0.000000\" m03 = \"0.000000\" \n \
m10 = \"0.000000\" m11 = \"1.000000\" m12 = \"0.000000\" m13 = \"0.000000\" \n \
m20 = \"0.000000\" m21 = \"0.000000\" m22 = \"1.000000\" m23 = \"0.000000\" \n \
m30 = \"0.000000\" m31 = \"0.000000\" m32 = \"0.000000\" m33 = \"1.000000\" \n \
>\n \
<object name = \"cube\" shader_name = \"matprv\" ")
    if castsh:
        preview_file.write("shadow = \"on\" ")
    else:
        preview_file.write("shadow = \"off\" ") 
    if addCausIOR:
        preview_file.write("caus_IOR = \"%f\" " %CausIOR.val)
    if recrad:
        preview_file.write("recv_rad = \"on\" ")
    if emirad:
        preview_file.write("emit_rad = \"on\" ")
    preview_file.write(">\n")


    preview_file.write("\t<attributes>\n")
    if addCausTCol:
        preview_file.write("\t<caus_tcolor r = \"%f\" g = \"%f\" b = \"%f\" />\n" %(CausTR.val, CausTG.val, CausTB.val))
    if addCausRCol:
        preview_file.write("\t<caus_rcolor r = \"%f\" g = \"%f\" b = \"%f\" />\n" %(CausRR.val, CausRG.val, CausRB.val))

    preview_file.write("\t</attributes>\n \
\t<mesh>\n \
\t\t<points>\n \
\t\t\t<p x=\"1.000000\"  y=\"1.000000\"  z=\"-1.000000\" /> \n \
\t\t\t<p x=\"1.000000\"  y=\"-1.000000\"  z=\"-1.000000\" /> \n \
\t\t\t<p x=\"-1.000000\"  y=\"-1.000000\"  z=\"-1.000000\" /> \n \
\t\t\t<p x=\"-1.000000\"  y=\"1.000000\"  z=\"-1.000000\" /> \n \
\t\t\t<p x=\"1.000000\"  y=\"0.999999\"  z=\"1.000000\" /> \n \
\t\t\t<p x=\"0.999999\"  y=\"-1.000001\"  z=\"1.000000\" /> \n \
\t\t\t<p x=\"-1.000000\"  y=\"-1.000000\"  z=\"1.000000\" /> \n \
<p x=\"-1.000000\"  y=\"1.000000\"  z=\"1.000000\" /> \n \
\t\t</points>\n \
\t\t<faces>\n \
\t\t\t<f a=\"0\" b=\"1\" c=\"2\" /> \n \
\t\t\t<f a=\"2\" b=\"3\" c=\"0\" /> \n \
\t\t\t<f a=\"4\" b=\"7\" c=\"6\" /> \n \
\t\t\t<f a=\"6\" b=\"5\" c=\"4\" /> \n \
\t\t\t<f a=\"0\" b=\"4\" c=\"5\" /> \n \
\t\t\t<f a=\"5\" b=\"1\" c=\"0\" /> \n \
\t\t\t<f a=\"1\" b=\"5\" c=\"6\" /> \n \
\t\t\t<f a=\"6\" b=\"2\" c=\"1\" /> \n \
\t\t\t<f a=\"2\" b=\"6\" c=\"7\" /> \n \
\t\t\t<f a=\"7\" b=\"3\" c=\"2\" /> \n \
\t\t\t<f a=\"4\" b=\"0\" c=\"3\" /> \n \
\t\t\t<f a=\"3\" b=\"7\" c=\"4\" /> \n \
\t\t</faces>\n \
\t</mesh>\n \
</object>\n \
</transform>\n")

#SFERA
    preview_file.write("<transform \n \
m00 = \"0.500000\" m01 = \"0.000000\" m02 = \"0.000000\" m03 = \"0.000000\" \n \
m10 = \"0.000000\" m11 = \"0.500000\" m12 = \"0.000000\" m13 = \"0.000000\" \n \
m20 = \"0.000000\" m21 = \"0.000000\" m22 = \"0.500000\" m23 = \"2.000000\" \n \
m30 = \"0.000000\" m31 = \"0.000000\" m32 = \"0.000000\" m33 = \"1.000000\" \n \
> \n \
<object name = \"Sphere\" shader_name = \"matprv\" ")
    if castsh:
        preview_file.write("shadow = \"on\" ")
    else:
        preview_file.write("shadow = \"off\" ") 
    if addCausIOR:
        preview_file.write("caus_IOR = \"%f\" " %CausIOR.val)
    if recrad:
        preview_file.write("recv_rad = \"on\" ")
    if emirad:
        preview_file.write("emit_rad = \"on\" ")
    preview_file.write(">\n")


    preview_file.write("\t<attributes>\n")
    if addCausTCol:
        preview_file.write("\t<caus_tcolor r = \"%f\" g = \"%f\" b = \"%f\" />\n" %(CausTR.val, CausTG.val, CausTB.val))
    if addCausRCol:
        preview_file.write("\t<caus_rcolor r = \"%f\" g = \"%f\" b = \"%f\" />\n" %(CausRR.val, CausRG.val, CausRB.val))

    preview_file.write("\t</attributes>\n \
\t<sphere radius = \"1.0\"> \n \
\t\t<center x=\"0.0\" y=\"0.0\" z=\"0.0\" /> \n \
\t</sphere> \n \
</object> \n \
</transform> \n")

    preview_file.write("<texture type = \"image\" name = \"texwall\"> \n \
<filename value = \"previewbg.jpg\" /> \n \
</texture> \n")

    preview_file.write("<shader type = \"generic\" name = \"matwall\"> \n \
\t<attributes> \n \
\t\t<color r=\"1.000000\" g=\"1.000000\" b=\"1.000000\" /> \n \
\t\t<specular r=\"1.000000\" g=\"1.000000\" b=\"1.000000\" /> \n \
\t\t<hard value = \"25.000000\"/> \n \
\t\t<min_refle value = \"0.200000\"/> \n \
\t</attributes> \n \
\t<modulator texname=\"texwall\" mode=\"mix\" size=\"1.000000\" sizex=\"1.000000\" sizey=\"1.000000\" sizez=\"1.000000\" > \n \
\t\t<color value=\"1.000000\" /> \n \
\t</modulator> \n \
</shader> \n")


#PARETI
    preview_file.write("<transform \n \
m00 = \"1.427173\" m01 = \"1.426519\" m02 = \"0.000000\" m03 = \"0.000000\" \n \
m10 = \"-1.426519\" m11 = \"1.427173\" m12 = \"0.000000\" m13 = \"0.000000\" \n \
m20 = \"0.000000\" m21 = \"0.000000\" m22 = \"2.017865\" m23 = \"1.000000\" \n \
m30 = \"0.000000\" m31 = \"0.000000\" m32 = \"0.000000\" m33 = \"1.000000\" \n \
> \n \
<object name = \"Plane\" shader_name = \"matwall\" > \n \
\t<attributes> \n \
\t</attributes> \n \
\t<mesh> \n \
\t\t<points> \n \
\t\t\t<p x=\"0.991147\"  y=\"0.991146\"  z=\"0.991147\" /> \n \
\t\t\t<p x=\"0.991147\"  y=\"0.991146\"  z=\"-0.991147\" /> \n \
\t\t\t<p x=\"-0.991147\"  y=\"0.991146\"  z=\"-0.991147\" /> \n \
\t\t\t<p x=\"-0.991147\"  y=\"0.991146\"  z=\"0.991147\" /> \n \
\t\t\t<p x=\"-0.991147\"  y=\"0.991147\"  z=\"0.991146\" /> \n \
\t\t\t<p x=\"-0.991147\"  y=\"0.991147\"  z=\"-0.991147\" /> \n \
\t\t\t<p x=\"-0.991147\"  y=\"-0.991147\"  z=\"-0.991147\" /> \n \
\t\t\t<p x=\"-0.991147\"  y=\"-0.991147\"  z=\"0.991146\" /> \n \
\t\t\t<p x=\"0.991146\"  y=\"0.991146\"  z=\"-0.991147\" /> \n \
\t\t\t<p x=\"0.991146\"  y=\"-0.991147\"  z=\"-0.991147\" /> \n \
\t\t\t<p x=\"-0.991146\"  y=\"-0.991147\"  z=\"-0.991147\" /> \n \
\t\t\t<p x=\"-0.991146\"  y=\"0.991146\"  z=\"-0.991147\" /> \n \
\t\t</points> \n \
\t\t<faces> \n \
\t\t\t<f a=\"0\" b=\"3\" c=\"2\" u_a=\"0.000000\" v_a=\"0.000000\" u_b=\"0.000000\" v_b=\"1.000000\" u_c=\"1.000000\" v_c=\"1.000000\" /> \n \
\t\t\t<f a=\"2\" b=\"1\" c=\"0\" u_a=\"1.000000\" v_a=\"1.000000\" u_b=\"1.000000\" v_b=\"0.000000\" u_c=\"0.000000\" v_c=\"0.000000\" /> \n \
\t\t\t<f a=\"4\" b=\"7\" c=\"6\" u_a=\"0.000000\" v_a=\"0.000000\" u_b=\"0.000000\" v_b=\"1.000000\" u_c=\"1.000000\" v_c=\"1.000000\" /> \n \
\t\t\t<f a=\"6\" b=\"5\" c=\"4\" u_a=\"1.000000\" v_a=\"1.000000\" u_b=\"1.000000\" v_b=\"0.000000\" u_c=\"0.000000\" v_c=\"0.000000\" /> \n \
\t\t\t<f a=\"8\" b=\"11\" c=\"10\" u_a=\"0.000000\" v_a=\"0.000000\" u_b=\"0.000000\" v_b=\"1.000000\" u_c=\"1.000000\" v_c=\"1.000000\" /> \n \
\t\t\t<f a=\"10\" b=\"9\" c=\"8\" u_a=\"1.000000\" v_a=\"1.000000\" u_b=\"1.000000\" v_b=\"0.000000\" u_c=\"0.000000\" v_c=\"0.000000\" /> \n \
\t\t</faces> \n \
\t</mesh> \n \
</object> \n \
</transform> \n")

#LUCI
    if phlprv.val:
        preview_file.write("<light type = \"photonlight\" name = \"lucefotdiff\" power = \"80\" photons = \"5000\" depth = \"4\" search = \"200\" angle = \"20\" \n \
\tmode = \"diffuse\" fixedradius = \"1.000000\" cluster = \"0.200000\" use_QMC = \"on\" > \n \
\t<from x=\"3.000000\" y=\"-3.000000\" z=\"5.000000\" /> \n \
\t<to x=\"0.000000\" y=\"0.000000\" z=\"0.000000\" /> \n \
\t<color r=\"1.000000\" g=\"1.000000\" b=\"1.000000\" /> \n \
</light> \n")
        preview_file.write("<light type = \"photonlight\" name = \"lucefotcaus\" power = \"100\" photons = \"10000\" depth = \"4\" search = \"100\" angle = \"20\" \n \
\tmode = \"caustic\" fixedradius = \"0.100000\" cluster = \"0.010000\" use_QMC = \"on\" > \n \
\t<from x=\"3.000000\" y=\"-3.000000\" z=\"5.000000\" /> \n \
\t<to x=\"0.000000\" y=\"0.000000\" z=\"0.000000\" /> \n \
\t<color r=\"1.000000\" g=\"1.000000\" b=\"1.000000\" /> \n \
</light> \n")
    
    preview_file.write("<light type = \"pointlight\" name= \"Lamp.001\" power= \"10.000000\" cast_shadows= \"on\" > \n \
\t<from x=\"3.000000\" y=\"-3.000000\" z=\"5.000000\" /> \n \
\t<color r=\"1.000000\" g=\"1.000000\" b=\"1.000000\" /> \n \
</light> \n\n \
<light type = \"pointlight\" name= \"Lamp\" power= \"20.000000\" cast_shadows= \"on\" > \n \
\t<from x=\"-5.000000\" y=\"-5.000000\" z=\"3.000000\" /> \n \
\t<color r=\"1.000000\" g=\"1.000000\" b=\"1.000000\" /> \n \
</light> \n")

#CAMERA
    preview_file.write("<camera name=\"Camera\" resx=\"160\" resy=\"120\" focal=\"1.093750\" > \n \
\t<from x=\"0.000000\" y=\"-8.128851\" z=\"4.000000\" /> \n \
\t<to x=\"0.000000\" y=\"-7.218724\" z=\"3.585670\" /> \n \
\t<up x=\"-0.000000\" y=\"-7.714521\" z=\"4.910127\" /> \n \
</camera> \n")
    
#RENDER
    preview_file.write("<render camera_name = \"Camera\" AA_passes = \"0\" AA_threshold = \"0.060000\" raydepth = \"5\" bias = \"0.300000\" gamma = \"1.000000\" exposure = \"0.000000\"  > \n \
<outfile value=\"previewmat.tga\"/> \n \
</render>\n")

    preview_file.write("</scene>\n")    
        
    preview_file.close()

#lancia il render del preview

    try:
            if POSIX:
                os.spawnlp(1, "yafray", "", filenamepreview)
            else:
                os.spawnl(os.P_NOWAIT, 'C:\yafray\yafray.exe','C:\yafray\yafray.exe', filenamepreview) 
    except OSError:
            print "Cannot launch renderer!!"
        
    sys.stdout.flush()
    return preview_file.name